@@ -117,6 +117,7 @@ enum GLNVGuniformLoc {
 	GLNVG_LOC_VIEWSIZE,
 	GLNVG_LOC_TEX,
 	GLNVG_LOC_FRAG,
+	GLNVG_LOC_GRADIENTCOLORS,
 	GLNVG_MAX_LOCS
 };
 
@@ -124,7 +125,9 @@ enum GLNVGshaderType {
 	NSVG_SHADER_FILLGRAD,
 	NSVG_SHADER_FILLIMG,
 	NSVG_SHADER_SIMPLE,
-	NSVG_SHADER_IMG
+	NSVG_SHADER_IMG,
+	NSVG_SHADER_FILLGRADIMG,
+	NSVG_SHADER_FILLGRADIMGFOCAL
 };
 
 #if NANOVG_GL_USE_UNIFORMBUFFER
@@ -170,12 +173,15 @@ enum GLNVGcallType {
 struct GLNVGcall {
 	int type;
 	int image;
+	int clipOffset;
+	int clipCount;
 	int pathOffset;
 	int pathCount;
 	int triangleOffset;
 	int triangleCount;
 	int uniformOffset;
 	GLNVGblend blendFunc;
+	float* gradientcolors;
 };
 typedef struct GLNVGcall GLNVGcall;
 
@@ -202,10 +208,13 @@ struct GLNVGfragUniforms {
 		float strokeThr;
 		int texType;
 		int type;
+		int spreadMode;
+		int interpolationmode;
+		float focalpoint;
 	#else
 		// note: after modifying layout or size of uniform array,
 		// don't forget to also update the fragment shader source!
-		#define NANOVG_GL_UNIFORMARRAY_SIZE 11
+		#define NANOVG_GL_UNIFORMARRAY_SIZE 12
 		union {
 			struct {
 				float scissorMat[12]; // matrices are actually 3 vec4s
@@ -221,6 +230,9 @@ struct GLNVGfragUniforms {
 				float strokeThr;
 				float texType;
 				float type;
+				float spreadMode;
+				float interpolationmode;
+				float focalpoint;
 			};
 			float uniformArray[NANOVG_GL_UNIFORMARRAY_SIZE][4];
 		};
@@ -236,8 +248,10 @@ struct GLNVGcontext {
 	int ctextures;
 	int textureId;
 	GLuint vertBuf;
+	GLuint clipVertBuf;
 #if defined NANOVG_GL3
 	GLuint vertArr;
+	GLuint clipVertArr;
 #endif
 #if NANOVG_GL_USE_UNIFORMBUFFER
 	GLuint fragBuf;
@@ -252,6 +266,9 @@ struct GLNVGcontext {
 	GLNVGpath* paths;
 	int cpaths;
 	int npaths;
+	struct NVGclipPath* clips;
+	struct NVGclipPath* lastClip;
+	int clipactive;
 	struct NVGvertex* verts;
 	int cverts;
 	int nverts;
@@ -500,6 +517,7 @@ static void glnvg__getUniforms(GLNVGshader* shader)
 #else
 	shader->loc[GLNVG_LOC_FRAG] = glGetUniformLocation(shader->prog, "frag");
 #endif
+	shader->loc[GLNVG_LOC_GRADIENTCOLORS] = glGetUniformLocation(shader->prog, "gradientcolors");
 }
 
 static int glnvg__renderCreateTexture(void* uptr, int type, int w, int h, int imageFlags, const unsigned char* data);
@@ -528,7 +546,7 @@ static int glnvg__renderCreate(void* uptr)
 #if NANOVG_GL_USE_UNIFORMBUFFER
 	"#define USE_UNIFORMBUFFER 1\n"
 #else
-	"#define UNIFORMARRAY_SIZE 11\n"
+	"#define UNIFORMARRAY_SIZE 12\n"
 #endif
 	"\n";
 
@@ -576,6 +594,9 @@ static int glnvg__renderCreate(void* uptr)
 		"		float strokeThr;\n"
 		"		int texType;\n"
 		"		int type;\n"
+		"		int spreadMode;\n"
+		"		int interpolationmode;\n"
+		"		float focalpoint;\n"
 		"	};\n"
 		"#else\n" // NANOVG_GL3 && !USE_UNIFORMBUFFER
 		"	uniform vec4 frag[UNIFORMARRAY_SIZE];\n"
@@ -604,7 +625,28 @@ static int glnvg__renderCreate(void* uptr)
 		"	#define strokeThr frag[10].y\n"
 		"	#define texType int(frag[10].z)\n"
 		"	#define type int(frag[10].w)\n"
+		"	#define spreadMode int(frag[11].x)\n"
+		"	#define interpolationmode int(frag[11].y)\n"
+		"	#define focalpoint frag[11].z\n"
 		"#endif\n"
+		"	uniform vec4 gradientcolors[256];\n"
+		"\n"
+		"float applySpread(float d) {\n"
+		"	if (spreadMode == 0) // Pad\n"
+		"		return clamp(d, 0.0, 1.0);\n"
+		"	else if (spreadMode == 1) // Repeat\n"
+		"		return fract(d);\n"
+		"	else if (spreadMode == 2) { // Reflect\n"
+		"		d = abs(d);\n"
+		"		#ifdef NANOVG_GL3\n"
+		"		bool even = !bool(int(d) & 1);\n"
+		"		#else\n" // !NANOVG_GL3
+		"		bool even = !bool(int(mod(d, 2.0)));\n"
+		"		#endif\n"
+		"		return (even) ? fract(d) : 1.0 - fract(d);\n"
+		"	}\n"
+		"	return d;\n"
+		"}\n"
 		"\n"
 		"float sdroundrect(vec2 pt, vec2 ext, float rad) {\n"
 		"	vec2 ext2 = ext - vec2(rad,rad);\n"
@@ -618,6 +660,13 @@ static int glnvg__renderCreate(void* uptr)
 		"	sc = vec2(0.5,0.5) - sc * scissorScale;\n"
 		"	return clamp(sc.x,0.0,1.0) * clamp(sc.y,0.0,1.0);\n"
 		"}\n"
+		"// linear RGB conversion (taken from ruffle)\n"
+		"vec3 linear_to_srgb(vec3 linear) {\n"
+		"	vec3 a = 12.92 * linear;\n"
+		"	vec3 b = 1.055 * pow(linear, vec3(1.0 / 2.4)) - 0.055;\n"
+		"	vec3 c = step(vec3(0.0031308), linear);\n"
+		"	return mix(a, b, c);\n"
+		"}\n"
 		"#ifdef EDGE_AA\n"
 		"// Stroke - from [0..1] to clipped pyramid, where the slope is 1px.\n"
 		"float strokeMask() {\n"
@@ -637,13 +686,14 @@ static int glnvg__renderCreate(void* uptr)
 		"	if (type == 0) {			// Gradient\n"
 		"		// Calculate gradient color using box gradient\n"
 		"		vec2 pt = (paintMat * vec3(fpos,1.0)).xy;\n"
-		"		float d = clamp((sdroundrect(pt, extent, radius) + feather*0.5) / feather, 0.0, 1.0);\n"
+		"		float d = applySpread((sdroundrect(pt, extent, radius) + feather*0.5) / feather);\n"
 		"		vec4 color = mix(innerCol,outerCol,d);\n"
+		"		color.rgb = mix(color.rgb,linear_to_srgb(color.rgb),float(interpolationmode));\n"
 		"		// Combine alpha\n"
 		"		color *= strokeAlpha * scissor;\n"
 		"		result = color;\n"
 		"	} else if (type == 1) {		// Image\n"
-		"		// Calculate color fron texture\n"
+		"		// Calculate color from texture\n"
 		"		vec2 pt = (paintMat * vec3(fpos,1.0)).xy / extent;\n"
 		"#ifdef NANOVG_GL3\n"
 		"		vec4 color = texture(tex, pt);\n"
@@ -669,6 +719,27 @@ static int glnvg__renderCreate(void* uptr)
 		"		if (texType == 2) color = vec4(color.x);"
 		"		color *= scissor;\n"
 		"		result = color * innerCol;\n"
+		"	} else if (type == 4) {		// gradientcolor based gradient\n"
+		"		// Calculate gradient color using box gradient\n"
+		"		vec2 pt = (paintMat * vec3(fpos,1.0)).xy;\n"
+		"		float d = applySpread((sdroundrect(pt, extent, radius) + feather*0.5) / feather);\n"
+		"		vec4 color = gradientcolors[int(d*255.0)];\n"
+		"		color.rgb = mix(color.rgb,linear_to_srgb(color.rgb),float(interpolationmode));\n"
+		"		// Combine alpha\n"
+		"		color *= strokeAlpha * scissor;\n"
+		"		result = color;\n"
+		"	} else if (type == 5) { // gradientcolor based gradient with focal point, algorithm taken from ruffle\n"
+		"		vec2 pt = (paintMat * vec3(fpos,1.0)).xy;\n"
+		"		vec2 pt2 = vec2(focalpoint,0.0)-pt;\n"
+		"		float l = length(pt2);\n"
+		"		pt2 /= l;\n"
+		"		pt2.x = l/(sqrt(1.0-focalpoint*focalpoint*pt2.y*pt2.y) + focalpoint*pt2.x);\n"
+		"		float d = applySpread((sdroundrect(pt2, extent, radius) + feather*0.5) / feather);\n"
+		"		vec4 color = gradientcolors[int(d*255.0)];\n"
+		"		color.rgb = mix(color.rgb,linear_to_srgb(color.rgb),float(interpolationmode));\n"
+		"		// Combine alpha\n"
+		"		color *= strokeAlpha * scissor;\n"
+		"		result = color;\n"
 		"	}\n"
 		"#ifdef NANOVG_GL3\n"
 		"	outColor = result;\n"
@@ -693,8 +764,10 @@ static int glnvg__renderCreate(void* uptr)
 	// Create dynamic vertex array
 #if defined NANOVG_GL3
 	glGenVertexArrays(1, &gl->vertArr);
+	glGenVertexArrays(1, &gl->clipVertArr);
 #endif
 	glGenBuffers(1, &gl->vertBuf);
+	glGenBuffers(1, &gl->clipVertBuf);
 
 #if NANOVG_GL_USE_UNIFORMBUFFER
 	// Create UBOs
@@ -730,6 +803,11 @@ static int glnvg__renderCreateTexture(void* uptr, int type, int w, int h, int im
 			printf("Repeat X/Y is not supported for non power-of-two textures (%d x %d)\n", w, h);
 			imageFlags &= ~(NVG_IMAGE_REPEATX | NVG_IMAGE_REPEATY);
 		}
+		// No mirror
+		if ((imageFlags & NVG_IMAGE_MIRRORX) != 0 || (imageFlags & NVG_IMAGE_MIRRORY) != 0) {
+			printf("Mirror X/Y is not supported for non power-of-two textures (%d x %d)\n", w, h);
+			imageFlags &= ~(NVG_IMAGE_MIRRORX | NVG_IMAGE_MIRRORY);
+		}
 		// No mips.
 		if (imageFlags & NVG_IMAGE_GENERATE_MIPMAPS) {
 			printf("Mip-maps is not support for non power-of-two textures (%d x %d)\n", w, h);
@@ -793,11 +871,15 @@ static int glnvg__renderCreateTexture(void* uptr, int type, int w, int h, int im
 
 	if (imageFlags & NVG_IMAGE_REPEATX)
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+	else if (imageFlags & NVG_IMAGE_MIRRORX)
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
 	else
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 
 	if (imageFlags & NVG_IMAGE_REPEATY)
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+	else if (imageFlags & NVG_IMAGE_MIRRORY)
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
 	else
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
@@ -953,23 +1035,26 @@ static int glnvg__convertPaint(GLNVGcontext* gl, GLNVGfragUniforms* frag, NVGpai
 			nvgTransformInverse(invxform, paint->xform);
 		}
 		frag->type = NSVG_SHADER_FILLIMG;
-
-		#if NANOVG_GL_USE_UNIFORMBUFFER
+		
+#if NANOVG_GL_USE_UNIFORMBUFFER
 		if (tex->type == NVG_TEXTURE_RGBA)
 			frag->texType = (tex->flags & NVG_IMAGE_PREMULTIPLIED) ? 0 : 1;
 		else
 			frag->texType = 2;
-		#else
+#else
 		if (tex->type == NVG_TEXTURE_RGBA)
 			frag->texType = (tex->flags & NVG_IMAGE_PREMULTIPLIED) ? 0.0f : 1.0f;
 		else
 			frag->texType = 2.0f;
-		#endif
+#endif
 //		printf("frag->texType = %d\n", frag->texType);
 	} else {
-		frag->type = NSVG_SHADER_FILLGRAD;
+		frag->type = paint->gradientcolors ? (paint->isGradient == 2 ? NSVG_SHADER_FILLGRADIMGFOCAL : NSVG_SHADER_FILLGRADIMG) : NSVG_SHADER_FILLGRAD;
 		frag->radius = paint->radius;
 		frag->feather = paint->feather;
+		frag->spreadMode = paint->spreadMode;
+		frag->focalpoint = paint->focalpoint;
+		frag->interpolationmode = paint->interpolationmode;
 		nvgTransformInverse(invxform, paint->xform);
 	}
 
@@ -980,7 +1065,7 @@ static int glnvg__convertPaint(GLNVGcontext* gl, GLNVGfragUniforms* frag, NVGpai
 
 static GLNVGfragUniforms* nvg__fragUniformPtr(GLNVGcontext* gl, int i);
 
-static void glnvg__setUniforms(GLNVGcontext* gl, int uniformOffset, int image)
+static void glnvg__setUniforms(GLNVGcontext* gl, int uniformOffset, int image, float* gradientcolors=nullptr)
 {
 	GLNVGtexture* tex = NULL;
 #if NANOVG_GL_USE_UNIFORMBUFFER
@@ -989,6 +1074,9 @@ static void glnvg__setUniforms(GLNVGcontext* gl, int uniformOffset, int image)
 	GLNVGfragUniforms* frag = nvg__fragUniformPtr(gl, uniformOffset);
 	glUniform4fv(gl->shader.loc[GLNVG_LOC_FRAG], NANOVG_GL_UNIFORMARRAY_SIZE, &(frag->uniformArray[0][0]));
 #endif
+	if (gradientcolors != NULL) {
+		glUniform4fv(gl->shader.loc[GLNVG_LOC_GRADIENTCOLORS], NANOVG_GRADIENTCOLOR_SIZE, gradientcolors);
+	}
 
 	if (image != 0) {
 		tex = glnvg__findTexture(gl, image);
@@ -1009,17 +1097,125 @@ static void glnvg__renderViewport(void* uptr, float width, float height, float d
 	gl->view[1] = height;
 }
 
+static void glnvg__setClipVertexBuffer(GLNVGcontext* gl, NVGclipPath* clip)
+{
+	glBufferData(GL_ARRAY_BUFFER, clip->nverts * sizeof(NVGvertex), clip->verts, GL_STREAM_DRAW);
+	glnvg__checkError(gl, "setting clip vertex buffer");
+}
+
+static void glnvg__bindClipVertexBuffer(GLNVGcontext* gl)
+{
+	// Upload vertex data
+	#if defined NANOVG_GL3
+	glBindVertexArray(gl->clipVertArr);
+	#endif
+	glBindBuffer(GL_ARRAY_BUFFER, gl->clipVertBuf);
+	glEnableVertexAttribArray(0);
+	glEnableVertexAttribArray(1);
+	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, sizeof(NVGvertex), (const GLvoid*)(size_t)0);
+	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, sizeof(NVGvertex), (const GLvoid*)(0 + 2*sizeof(float)));
+	glnvg__checkError(gl, "binding clip vertex buffer");
+}
+
+static void glnvg__resetVertexBuffer(GLNVGcontext* gl)
+{
+	#if defined NANOVG_GL3
+	glBindVertexArray(gl->vertArr);
+	#endif
+	glBindBuffer(GL_ARRAY_BUFFER, gl->vertBuf);
+	glnvg__checkError(gl, "resetting vertex buffer");
+	glEnableVertexAttribArray(0);
+	glEnableVertexAttribArray(1);
+	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, sizeof(NVGvertex), (const GLvoid*)(size_t)0);
+	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, sizeof(NVGvertex), (const GLvoid*)(0 + 2*sizeof(float)));
+}
+
+static void glnvg__stencilClipCoverage(GLNVGcontext* gl, GLNVGcall* call)
+{
+	glEnable(GL_CULL_FACE);
+	glnvg__stencilFunc(gl, GL_NOTEQUAL, 0x80, 0x7f);
+	glnvg__stencilMask(gl, 0xff);
+	glStencilOp(GL_ZERO, GL_ZERO, GL_REPLACE);
+	glDrawArrays(GL_TRIANGLE_STRIP, call->triangleOffset, call->triangleCount);
+	glnvg__checkError(gl, "drawing clip path stencil");
+}
+
+static void glnvg__stencilClipPaths(GLNVGcontext* gl, GLNVGcall* call)
+{
+	int i, j;
+	int hasClipStack = (gl->clipactive && gl->clips != NULL && gl->lastClip != NULL);
+	glnvg__stencilMask(gl, 0x7f);
+	glnvg__stencilFunc(gl, GL_ALWAYS, 0x00, 0xff);
+	glnvg__setUniforms(gl, call->uniformOffset, 0);
+
+	if (hasClipStack) {
+		NVGclipPath* clip;
+		for (clip = gl->clips, i = 0; clip != NULL; clip = clip->next, i++) {
+			GLNVGpath* paths = (GLNVGpath*)clip->paths;
+			glnvg__bindClipVertexBuffer(gl);
+			glnvg__setClipVertexBuffer(gl, clip);
+
+			glDisable(GL_CULL_FACE);
+			glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP);
+			glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP);
+
+			if (i > 0) {
+				glnvg__stencilMask(gl, 0x7f);
+				glnvg__stencilFunc(gl, GL_EQUAL, 0x80, 0x80);
+			}
+
+			for (j = 0; j < clip->npaths; j++) {
+				if (paths[j].fillCount > 0)
+					glDrawArrays(GL_TRIANGLE_FAN, paths[j].fillOffset, paths[j].fillCount);
+				else if (paths[j].strokeCount > 0)
+					glDrawArrays(GL_TRIANGLE_FAN, paths[j].strokeOffset, paths[j].strokeCount);
+			}
+
+			glnvg__resetVertexBuffer(gl);
+			glnvg__stencilClipCoverage(gl, call);
+		}
+	} else {
+		GLNVGpath* paths = &gl->paths[call->clipOffset];
+
+		glDisable(GL_CULL_FACE);
+		glStencilOpSeparate(GL_FRONT, GL_KEEP, GL_KEEP, GL_INCR_WRAP);
+		glStencilOpSeparate(GL_BACK, GL_KEEP, GL_KEEP, GL_DECR_WRAP);
+
+		for (i = 0; i < call->clipCount; i++) {
+			if (paths[i].fillCount > 0)
+				glDrawArrays(GL_TRIANGLE_FAN, paths[i].fillOffset, paths[i].fillCount);
+			else if (paths[i].strokeCount > 0)
+				glDrawArrays(GL_TRIANGLE_FAN, paths[i].strokeOffset, paths[i].strokeCount);
+		}
+
+		glnvg__stencilClipCoverage(gl, call);
+	}
+}
+
 static void glnvg__fill(GLNVGcontext* gl, GLNVGcall* call)
 {
 	GLNVGpath* paths = &gl->paths[call->pathOffset];
 	int i, npaths = call->pathCount;
 
+	int hasClipStack = (gl->clipactive && gl->clips != NULL && gl->lastClip != NULL);
+	int hasClipPaths = gl->clipactive && (call->clipCount > 0 || hasClipStack);
+
+	if (npaths <= 0) return;
+	for (i = 0; i < npaths && paths[i].fillCount > 0; i++);
+	if (i <= 0) return;
+
 	// Draw shapes
 	glEnable(GL_STENCIL_TEST);
-	glnvg__stencilMask(gl, 0xff);
-	glnvg__stencilFunc(gl, GL_ALWAYS, 0, 0xff);
 	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
 
+	if (hasClipPaths != 0) {
+		glnvg__stencilClipPaths(gl, call);
+		glnvg__stencilFunc(gl, GL_EQUAL, 0x80, 0x80);
+		glnvg__stencilMask(gl, 0x7f);
+	} else {
+		glnvg__stencilFunc(gl, GL_ALWAYS, 0, 0xff);
+	}
+
 	// set bindpoint for solid loc
 	glnvg__setUniforms(gl, call->uniformOffset, 0);
 	glnvg__checkError(gl, "fill simple");
@@ -1034,7 +1230,7 @@ static void glnvg__fill(GLNVGcontext* gl, GLNVGcall* call)
 	// Draw anti-aliased pixels
 	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 
-	glnvg__setUniforms(gl, call->uniformOffset + gl->fragSize, call->image);
+	glnvg__setUniforms(gl, call->uniformOffset + gl->fragSize, call->image,call->gradientcolors);
 	glnvg__checkError(gl, "fill fill");
 
 	if (gl->flags & NVG_ANTIALIAS) {
@@ -1046,7 +1242,8 @@ static void glnvg__fill(GLNVGcontext* gl, GLNVGcall* call)
 	}
 
 	// Draw fill
-	glnvg__stencilFunc(gl, GL_NOTEQUAL, 0x0, 0xff);
+	glnvg__stencilFunc(gl, GL_NOTEQUAL, 0x0, 0x7f);
+	glnvg__stencilMask(gl, 0x7f);
 	glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
 	glDrawArrays(GL_TRIANGLE_STRIP, call->triangleOffset, call->triangleCount);
 
@@ -1058,7 +1255,27 @@ static void glnvg__convexFill(GLNVGcontext* gl, GLNVGcall* call)
 	GLNVGpath* paths = &gl->paths[call->pathOffset];
 	int i, npaths = call->pathCount;
 
-	glnvg__setUniforms(gl, call->uniformOffset, call->image);
+	int hasClipStack = (gl->clipactive && gl->clips != NULL && gl->lastClip != NULL);
+	int hasClipPaths = gl->clipactive && (call->clipCount > 0 || hasClipStack);
+
+	if (npaths <= 0) return;
+	for (i = 0; i < npaths && paths[i].fillCount > 0; i++);
+	if (i <= 0) return;
+
+	if (hasClipPaths != 0) {
+		glEnable(GL_STENCIL_TEST);
+		glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
+
+		glnvg__stencilClipPaths(gl, call);
+		glnvg__stencilFunc(gl, GL_EQUAL, 0x80, 0xff);
+		glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
+
+		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+		glnvg__setUniforms(gl, call->uniformOffset + gl->fragSize, call->image, call->gradientcolors);
+	} else {
+		glnvg__setUniforms(gl, call->uniformOffset, call->image, call->gradientcolors);
+	}
+
 	glnvg__checkError(gl, "convex fill");
 
 	for (i = 0; i < npaths; i++) {
@@ -1068,6 +1285,8 @@ static void glnvg__convexFill(GLNVGcontext* gl, GLNVGcall* call)
 			glDrawArrays(GL_TRIANGLE_STRIP, paths[i].strokeOffset, paths[i].strokeCount);
 		}
 	}
+	if (hasClipPaths != 0)
+		glDisable(GL_STENCIL_TEST);
 }
 
 static void glnvg__stroke(GLNVGcontext* gl, GLNVGcall* call)
@@ -1075,51 +1294,39 @@ static void glnvg__stroke(GLNVGcontext* gl, GLNVGcall* call)
 	GLNVGpath* paths = &gl->paths[call->pathOffset];
 	int npaths = call->pathCount, i;
 
-	if (gl->flags & NVG_STENCIL_STROKES) {
+	int hasClipStack = (gl->clipactive && gl->clips != NULL && gl->lastClip != NULL);
+	int hasClipPaths = gl->clipactive && (call->clipCount > 0 || hasClipStack);
 
+	if (npaths <= 0) return;
+	for (i = 0; i < npaths && paths[i].strokeCount > 0; i++);
+	if (i <= 0) return;
+
+	if (hasClipPaths != 0) {
 		glEnable(GL_STENCIL_TEST);
-		glnvg__stencilMask(gl, 0xff);
-
-		// Fill the stroke base without overlap
-		glnvg__stencilFunc(gl, GL_EQUAL, 0x0, 0xff);
-		glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
-		glnvg__setUniforms(gl, call->uniformOffset + gl->fragSize, call->image);
-		glnvg__checkError(gl, "stroke fill 0");
-		for (i = 0; i < npaths; i++)
-			glDrawArrays(GL_TRIANGLE_STRIP, paths[i].strokeOffset, paths[i].strokeCount);
-
-		// Draw anti-aliased pixels.
-		glnvg__setUniforms(gl, call->uniformOffset, call->image);
-		glnvg__stencilFunc(gl, GL_EQUAL, 0x00, 0xff);
-		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
-		for (i = 0; i < npaths; i++)
-			glDrawArrays(GL_TRIANGLE_STRIP, paths[i].strokeOffset, paths[i].strokeCount);
-
-		// Clear stencil buffer.
 		glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
-		glnvg__stencilFunc(gl, GL_ALWAYS, 0x0, 0xff);
+
+		glnvg__stencilClipPaths(gl, call);
+		glnvg__stencilFunc(gl, GL_EQUAL, 0x80, 0xff);
 		glStencilOp(GL_ZERO, GL_ZERO, GL_ZERO);
-		glnvg__checkError(gl, "stroke fill 1");
-		for (i = 0; i < npaths; i++)
-			glDrawArrays(GL_TRIANGLE_STRIP, paths[i].strokeOffset, paths[i].strokeCount);
+		
 		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-
-		glDisable(GL_STENCIL_TEST);
-
-//		glnvg__convertPaint(gl, nvg__fragUniformPtr(gl, call->uniformOffset + gl->fragSize), paint, scissor, strokeWidth, fringe, 1.0f - 0.5f/255.0f);
-
+		glnvg__setUniforms(gl, call->uniformOffset + gl->fragSize, call->image, call->gradientcolors);
 	} else {
-		glnvg__setUniforms(gl, call->uniformOffset, call->image);
-		glnvg__checkError(gl, "stroke fill");
-		// Draw Strokes
-		for (i = 0; i < npaths; i++)
-			glDrawArrays(GL_TRIANGLE_STRIP, paths[i].strokeOffset, paths[i].strokeCount);
+		glnvg__setUniforms(gl, call->uniformOffset, call->image, call->gradientcolors);
 	}
+
+	glnvg__checkError(gl, "stroke fill");
+	// Draw Strokes
+	for (i = 0; i < npaths; i++)
+		glDrawArrays(GL_TRIANGLE_STRIP, paths[i].strokeOffset, paths[i].strokeCount);
+
+	if (hasClipPaths != 0)
+		glDisable(GL_STENCIL_TEST);
 }
 
 static void glnvg__triangles(GLNVGcontext* gl, GLNVGcall* call)
 {
-	glnvg__setUniforms(gl, call->uniformOffset, call->image);
+	glnvg__setUniforms(gl, call->uniformOffset, call->image, call->gradientcolors);
 	glnvg__checkError(gl, "triangles fill");
 
 	glDrawArrays(GL_TRIANGLES, call->triangleOffset, call->triangleCount);
@@ -1239,6 +1446,7 @@ static void glnvg__renderFlush(void* uptr)
 		for (i = 0; i < gl->ncalls; i++) {
 			GLNVGcall* call = &gl->calls[i];
 			glnvg__blendFuncSeparate(gl,&call->blendFunc);
+			//GLNVGfragUniforms* frag = nvg__fragUniformPtr(gl, call->uniformOffset);
 			if (call->type == GLNVG_FILL)
 				glnvg__fill(gl, call);
 			else if (call->type == GLNVG_CONVEXFILL)
@@ -1271,8 +1479,22 @@ static int glnvg__maxVertCount(const NVGpath* paths, int npaths)
 {
 	int i, count = 0;
 	for (i = 0; i < npaths; i++) {
-		count += paths[i].nfill;
-		count += paths[i].nstroke;
+		if (paths[i].clip == 0) {
+			count += paths[i].nfill;
+			count += paths[i].nstroke;
+		}
+	}
+	return count;
+}
+
+static int glnvg__maxClipVertCount(const NVGpath* paths, int npaths)
+{
+	int i, count = 0;
+	for (i = 0; i < npaths; i++) {
+		if (paths[i].clip != 0) {
+			count += paths[i].nfill;
+			count += paths[i].nstroke;
+		}
 	}
 	return count;
 }
@@ -1325,6 +1547,38 @@ static int glnvg__allocVerts(GLNVGcontext* gl, int n)
 	return ret;
 }
 
+static int glnvg__allocClipVerts(NVGclipPath* clip, int n)
+{
+	int ret = 0;
+	if (clip->nverts+n > clip->cverts) {
+		NVGvertex* verts;
+		int cverts = glnvg__maxi(clip->nverts + n, 4096) + clip->cverts/2; // 1.5x Overallocate
+		verts = (NVGvertex*)realloc(clip->verts, sizeof(NVGvertex) * cverts);
+		if (verts == NULL) return -1;
+		clip->verts = verts;
+		clip->cverts = cverts;
+	}
+	ret = clip->nverts;
+	clip->nverts += n;
+	return ret;
+}
+
+static int glnvg__allocClipPaths(NVGclipPath* clip, int n)
+{
+	int ret = 0;
+	if (clip->npaths+n > clip->cpaths) {
+		GLNVGpath* paths;
+		int cpaths = glnvg__maxi(clip->npaths + n, 128) + clip->cpaths/2; // 1.5x Overallocate
+		paths = (GLNVGpath*)realloc(clip->paths, sizeof(GLNVGpath) * cpaths);
+		if (paths == NULL) return -1;
+		clip->paths = paths;
+		clip->cpaths = cpaths;
+	}
+	ret = clip->npaths;
+	clip->npaths += n;
+	return ret;
+}
+
 static int glnvg__allocFragUniforms(GLNVGcontext* gl, int n)
 {
 	int ret = 0, structSize = gl->fragSize;
@@ -1354,56 +1608,129 @@ static void glnvg__vset(NVGvertex* vtx, float x, float y, float u, float v)
 	vtx->v = v;
 }
 
+static void glnvg__setClip(void* uptr, NVGclipPath* clip)
+{
+	GLNVGcontext* gl = (GLNVGcontext*)uptr;
+	gl->clips = clip;
+}
+
+static void glnvg__setLastClip(void* uptr, NVGclipPath* last)
+{
+	GLNVGcontext* gl = (GLNVGcontext*)uptr;
+	gl->lastClip = last;
+}
+
+static void glnvg__setClipActive(void* uptr, int active)
+{
+	GLNVGcontext* gl = (GLNVGcontext*)uptr;
+	gl->clipactive = active;
+}
+
 static void glnvg__renderFill(void* uptr, NVGpaint* paint, NVGcompositeOperationState compositeOperation, NVGscissor* scissor, float fringe,
-							  const float* bounds, const NVGpath* paths, int npaths)
+							  const float* bounds, const NVGpath* clipPaths, int nclipPaths, const NVGpath* paths, int npaths)
 {
 	GLNVGcontext* gl = (GLNVGcontext*)uptr;
 	GLNVGcall* call = glnvg__allocCall(gl);
 	NVGvertex* quad;
 	GLNVGfragUniforms* frag;
-	int i, maxverts, offset;
+	int i, j, maxverts, maxclipverts, offset, clipOffset, clipPathOffset;
+	int hasClipPaths = (gl->clipactive && clipPaths != NULL && nclipPaths > 0);
+	int hasClipStack = (gl->clipactive && gl->clips != NULL && gl->lastClip != NULL);
 
 	if (call == NULL) return;
 
 	call->type = GLNVG_FILL;
-	call->triangleCount = 4;
 	call->pathOffset = glnvg__allocPaths(gl, npaths);
 	if (call->pathOffset == -1) goto error;
 	call->pathCount = npaths;
-	call->image = paint->image;
+	if (paint->isGradient) {
+		call->gradientcolors = paint->gradientcolors;
+	} else {
+		call->image = paint->image;
+	}
 	call->blendFunc = glnvg__blendCompositeOperation(compositeOperation);
 
-	if (npaths == 1 && paths[0].convex)
-	{
-		call->type = GLNVG_CONVEXFILL;
-		call->triangleCount = 0;	// Bounding box fill quad not needed for convex fill
-	}
+	if (npaths == 1 && paths[0].convex) call->type = GLNVG_CONVEXFILL;
+	call->triangleCount = (call->type == GLNVG_FILL || hasClipPaths != 0 || hasClipStack != 0) ? 4 : 0;	// Bounding box fill quad not needed for non-clipped convex fill
 
 	// Allocate vertices for all the paths.
 	maxverts = glnvg__maxVertCount(paths, npaths) + call->triangleCount;
+	maxclipverts = glnvg__maxClipVertCount(paths, npaths);
 	offset = glnvg__allocVerts(gl, maxverts);
+
+	if (hasClipStack != 0 && hasClipPaths != 0) {
+		clipOffset = glnvg__allocClipVerts(gl->lastClip, maxclipverts);
+		clipPathOffset = glnvg__allocClipPaths(gl->lastClip, nclipPaths);
+
+		if (clipOffset == -1) goto error;
+		if (clipPathOffset == -1) goto error;
+
+		memset(&gl->lastClip->verts[clipOffset], 0, maxclipverts*sizeof(NVGvertex));
+		memset(&((GLNVGpath*)gl->lastClip->paths)[clipPathOffset], 0, nclipPaths*sizeof(GLNVGpath));
+	} else {
+		clipOffset = 0;
+		clipPathOffset = 0;
+	}
+
 	if (offset == -1) goto error;
 
-	for (i = 0; i < npaths; i++) {
+	for (i = 0, j = 0; i < npaths; i++) {
 		GLNVGpath* copy = &gl->paths[call->pathOffset + i];
 		const NVGpath* path = &paths[i];
 		memset(copy, 0, sizeof(GLNVGpath));
 		if (path->nfill > 0) {
-			copy->fillOffset = offset;
-			copy->fillCount = path->nfill;
-			memcpy(&gl->verts[offset], path->fill, sizeof(NVGvertex) * path->nfill);
-			offset += path->nfill;
+			if (path->clip == 0 || hasClipStack == 0) {
+				if (hasClipStack == 0 && path == clipPaths) {
+					call->clipOffset = offset;
+					call->clipCount = nclipPaths;
+				}
+				copy->fillOffset = offset;
+				copy->fillCount = path->nfill;
+				memcpy(&gl->verts[offset], path->fill, sizeof(NVGvertex) * path->nfill);
+				offset += path->nfill;
+			} else {
+				if (gl->lastClip->verts != NULL) {
+					NVGvertex* clipVerts = gl->lastClip->verts;
+					GLNVGpath* copyClip = &((GLNVGpath*)gl->lastClip->paths)[clipPathOffset + j];
+
+					copyClip->fillOffset = clipOffset;
+					copyClip->fillCount = path->nfill;
+
+					memcpy(&clipVerts[clipOffset], path->fill, sizeof(NVGvertex) * path->nfill);
+					clipOffset += path->nfill;
+					j++;
+				}
+			}
 		}
 		if (path->nstroke > 0) {
-			copy->strokeOffset = offset;
-			copy->strokeCount = path->nstroke;
-			memcpy(&gl->verts[offset], path->stroke, sizeof(NVGvertex) * path->nstroke);
-			offset += path->nstroke;
+			if (path->clip == 0 || hasClipStack == 0) {
+				if (hasClipStack == 0 && path == clipPaths) {
+					call->clipOffset = offset;
+					call->clipCount = nclipPaths;
+				}
+				copy->strokeOffset = offset;
+				copy->strokeCount = path->nstroke;
+				memcpy(&gl->verts[offset], path->stroke, sizeof(NVGvertex) * path->nstroke);
+				offset += path->nstroke;
+			} else {
+				if (gl->lastClip->verts != NULL) {
+					NVGvertex* clipVerts = gl->lastClip->verts;
+					GLNVGpath* copyClip = &((GLNVGpath*)gl->lastClip->paths)[clipPathOffset + j];
+
+					copyClip->strokeOffset = clipOffset;
+					copyClip->strokeCount = path->nstroke;
+
+					memcpy(&clipVerts[clipOffset], path->stroke, sizeof(NVGvertex) * path->nstroke);
+					clipOffset += path->nstroke;
+					j++;
+				}
+			}
 		}
 	}
 
+
 	// Setup uniforms for draw calls
-	if (call->type == GLNVG_FILL) {
+	if (call->triangleCount > 0) {
 		// Quad
 		call->triangleOffset = offset;
 		quad = &gl->verts[call->triangleOffset];
@@ -1437,11 +1764,14 @@ error:
 }
 
 static void glnvg__renderStroke(void* uptr, NVGpaint* paint, NVGcompositeOperationState compositeOperation, NVGscissor* scissor, float fringe,
-								float strokeWidth, const NVGpath* paths, int npaths)
+								float strokeWidth, const float* bounds, const NVGpath* clipPaths, int nclipPaths, const NVGpath* paths, int npaths)
 {
 	GLNVGcontext* gl = (GLNVGcontext*)uptr;
 	GLNVGcall* call = glnvg__allocCall(gl);
-	int i, maxverts, offset;
+	NVGvertex* quad;
+	int i, j, maxverts, maxclipverts, offset, clipOffset, clipPathOffset;
+	int hasClipPaths = (gl->clipactive && clipPaths != NULL && nclipPaths > 0);
+	int hasClipStack = (gl->clipactive && gl->clips != NULL && gl->lastClip != NULL);
 
 	if (call == NULL) return;
 
@@ -1449,39 +1779,88 @@ static void glnvg__renderStroke(void* uptr, NVGpaint* paint, NVGcompositeOperati
 	call->pathOffset = glnvg__allocPaths(gl, npaths);
 	if (call->pathOffset == -1) goto error;
 	call->pathCount = npaths;
-	call->image = paint->image;
+	if (paint->isGradient)
+		call->gradientcolors = paint->gradientcolors;
+	else
+		call->image = paint->image;
 	call->blendFunc = glnvg__blendCompositeOperation(compositeOperation);
+	call->triangleCount = (hasClipStack != 0 || hasClipPaths != 0) ? 4 : 0;	// Bounding box fill quad needed for clipped strokes
 
 	// Allocate vertices for all the paths.
-	maxverts = glnvg__maxVertCount(paths, npaths);
+	maxverts = glnvg__maxVertCount(paths, npaths) + call->triangleCount;
+	maxclipverts = glnvg__maxClipVertCount(paths, npaths);
 	offset = glnvg__allocVerts(gl, maxverts);
+
+	if (hasClipStack != 0 && hasClipPaths != 0) {
+		clipOffset = glnvg__allocClipVerts(gl->lastClip, maxclipverts);
+		clipPathOffset = glnvg__allocClipPaths(gl->lastClip, nclipPaths);
+
+		if (clipOffset == -1) goto error;
+		if (clipPathOffset == -1) goto error;
+
+		memset(&gl->lastClip->verts[clipOffset], 0, maxclipverts*sizeof(NVGvertex));
+		memset(&((GLNVGpath*)gl->lastClip->paths)[clipPathOffset], 0, nclipPaths*sizeof(GLNVGpath));
+	} else {
+		clipOffset = 0;
+		clipPathOffset = 0;
+	}
+
 	if (offset == -1) goto error;
 
-	for (i = 0; i < npaths; i++) {
+	for (i = 0, j = 0; i < npaths; i++) {
 		GLNVGpath* copy = &gl->paths[call->pathOffset + i];
 		const NVGpath* path = &paths[i];
 		memset(copy, 0, sizeof(GLNVGpath));
 		if (path->nstroke) {
-			copy->strokeOffset = offset;
-			copy->strokeCount = path->nstroke;
-			memcpy(&gl->verts[offset], path->stroke, sizeof(NVGvertex) * path->nstroke);
-			offset += path->nstroke;
+			if (path->clip == 0 || hasClipStack == 0) {
+				if (hasClipStack == 0 && path == clipPaths) {
+					call->clipOffset = offset;
+					call->clipCount = nclipPaths;
+				}
+				copy->strokeOffset = offset;
+				copy->strokeCount = path->nstroke;
+				memcpy(&gl->verts[offset], path->stroke, sizeof(NVGvertex) * path->nstroke);
+				offset += path->nstroke;
+			} else {
+				if (gl->lastClip->verts != NULL) {
+					NVGvertex* clipVerts = gl->lastClip->verts;
+					GLNVGpath* copyClip = &((GLNVGpath*)gl->lastClip->paths)[clipPathOffset + j];
+
+					copyClip->strokeOffset = clipOffset;
+					copyClip->strokeCount = path->nstroke;
+
+					memcpy(&clipVerts[clipOffset], path->stroke, sizeof(NVGvertex) * path->nstroke);
+					clipOffset += path->nstroke;
+					j++;
+				}
+			}
 		}
 	}
 
-	if (gl->flags & NVG_STENCIL_STROKES) {
-		// Fill shader
+	// Setup uniforms for draw calls
+	if (call->triangleCount > 0) {
+		// Quad
+		call->triangleOffset = offset;
+		quad = &gl->verts[call->triangleOffset];
+		glnvg__vset(&quad[0], bounds[2], bounds[3], 0.5f, 1.0f);
+		glnvg__vset(&quad[1], bounds[2], bounds[1], 0.5f, 1.0f);
+		glnvg__vset(&quad[2], bounds[0], bounds[3], 0.5f, 1.0f);
+		glnvg__vset(&quad[3], bounds[0], bounds[1], 0.5f, 1.0f);
+
 		call->uniformOffset = glnvg__allocFragUniforms(gl, 2);
 		if (call->uniformOffset == -1) goto error;
-
-		glnvg__convertPaint(gl, nvg__fragUniformPtr(gl, call->uniformOffset), paint, scissor, strokeWidth, fringe, -1.0f);
-		glnvg__convertPaint(gl, nvg__fragUniformPtr(gl, call->uniformOffset + gl->fragSize), paint, scissor, strokeWidth, fringe, 1.0f - 0.5f/255.0f);
-
-	} else {
+		// Simple shader for stencil
+		GLNVGfragUniforms* frag = nvg__fragUniformPtr(gl, call->uniformOffset);
+		memset(frag, 0, sizeof(*frag));
+		frag->strokeThr = -1.0f;
+		frag->type = NSVG_SHADER_SIMPLE;
 		// Fill shader
+		glnvg__convertPaint(gl, nvg__fragUniformPtr(gl, call->uniformOffset + gl->fragSize), paint, scissor, fringe, fringe, -1.0f);
+	} else {
 		call->uniformOffset = glnvg__allocFragUniforms(gl, 1);
 		if (call->uniformOffset == -1) goto error;
-		glnvg__convertPaint(gl, nvg__fragUniformPtr(gl, call->uniformOffset), paint, scissor, strokeWidth, fringe, -1.0f);
+		// Fill shader
+		glnvg__convertPaint(gl, nvg__fragUniformPtr(gl, call->uniformOffset), paint, scissor, fringe, fringe, -1.0f);
 	}
 
 	return;
@@ -1502,7 +1881,10 @@ static void glnvg__renderTriangles(void* uptr, NVGpaint* paint, NVGcompositeOper
 	if (call == NULL) return;
 
 	call->type = GLNVG_TRIANGLES;
-	call->image = paint->image;
+	if (paint->isGradient)
+		call->gradientcolors = paint->gradientcolors;
+	else
+		call->image = paint->image;
 	call->blendFunc = glnvg__blendCompositeOperation(compositeOperation);
 
 	// Allocate vertices for all the paths.
@@ -1590,10 +1972,15 @@ NVGcontext* nvgCreateGLES3(int flags)
 	params.renderStroke = glnvg__renderStroke;
 	params.renderTriangles = glnvg__renderTriangles;
 	params.renderDelete = glnvg__renderDelete;
+	params.setClip = glnvg__setClip;
+	params.setLastClip = glnvg__setLastClip;
+	params.setClipActive = glnvg__setClipActive;
 	params.userPtr = gl;
 	params.edgeAntiAlias = flags & NVG_ANTIALIAS ? 1 : 0;
 
 	gl->flags = flags;
+	gl->clips = NULL;
+	gl->lastClip = NULL;
 
 	ctx = nvgCreateInternal(&params);
 	if (ctx == NULL) goto error;
