@@ -130,6 +130,7 @@ REGISTER_CLASS_NAME(StageQuality,"flash.display")
 REGISTER_CLASS_NAME(SWFVersion,"flash.display")
 REGISTER_CLASS_NAME(TriangleCulling,"flash.display")
 REGISTER_CLASS_NAME(NativeWindow,"flash.display")
+REGISTER_CLASS_NAME(StageAspectRatio,"flash.display")
 
 //Display3D
 REGISTER_CLASS_NAME(Context3D,"flash.display3D")
@@ -179,6 +180,7 @@ REGISTER_CLASS_NAME(InvokeEvent,"flash.events")
 REGISTER_CLASS_NAME(KeyboardEvent,"flash.events")
 REGISTER_CLASS_NAME(MouseEvent,"flash.events")
 REGISTER_CLASS_NAME(NativeDragEvent,"flash.events")
+REGISTER_CLASS_NAME(NativeWindowBoundsEvent,"flash.events")
 REGISTER_CLASS_NAME(NetStatusEvent,"flash.events")
 REGISTER_CLASS_NAME(PressAndTapGestureEvent,"flash.events")
 REGISTER_CLASS_NAME(ProgressEvent,"flash.events")
@@ -440,6 +442,7 @@ REGISTER_CLASS_NAME(GameInputDevice,"flash.ui")
 //XML
 REGISTER_CLASS_NAME(XMLDocument,"flash.xml")
 REGISTER_CLASS_NAME(XMLNode,"flash.xml")
+REGISTER_CLASS_NAME(XMLNodeType,"flash.xml")
 
 //avmplus
 REGISTER_CLASS_NAME(avmplusFile,"avmplus")
@@ -447,26 +450,30 @@ REGISTER_CLASS_NAME(avmplusSystem,"avmplus")
 REGISTER_CLASS_NAME(avmplusDomain,"avmplus")
 
 //AVM1
-REGISTER_CLASS_NAME2(AVM1Key,"Key","")
-REGISTER_CLASS_NAME2(AVM1Mouse,"Mouse","")
-REGISTER_CLASS_NAME2(AVM1Sound,"Sound","")
-REGISTER_CLASS_NAME2(AVM1MovieClip,"MovieClip","")
-REGISTER_CLASS_NAME2(AVM1MovieClipLoader,"MovieClipLoader","")
-REGISTER_CLASS_NAME2(AVM1Shape,"Shape","")
-REGISTER_CLASS_NAME2(AVM1SimpleButton,"SimpleButton","")
-REGISTER_CLASS_NAME2(AVM1Stage,"Stage","")
-REGISTER_CLASS_NAME2(AVM1SharedObject,"SharedObject","")
-REGISTER_CLASS_NAME2(AVM1TextFormat,"TextFormat","")
-REGISTER_CLASS_NAME2(AVM1TextField,"TextField","")
-REGISTER_CLASS_NAME2(AVM1XMLDocument,"XML","")
-REGISTER_CLASS_NAME2(AVM1Video,"Video","")
-REGISTER_CLASS_NAME2(AVM1Color,"Color","")
-REGISTER_CLASS_NAME2(AVM1Broadcaster,"AsBroadcaster","")
-REGISTER_CLASS_NAME2(AVM1LocalConnection,"LocalConnection","")
-REGISTER_CLASS_NAME2(AVM1LoadVars,"LoadVars","")
-REGISTER_CLASS_NAME2(AVM1Bitmap,"Bitmap","")
-REGISTER_CLASS_NAME2(AVM1BitmapData,"BitmapData","")
-REGISTER_CLASS_NAME2(AVM1NetConnection,"NetConnection","")
-REGISTER_CLASS_NAME2(AVM1XMLSocket,"XMLSocket","")
-REGISTER_CLASS_NAME2(AVM1ContextMenuItem,"ContextMenuItem","")
-REGISTER_CLASS_NAME2(AVM1NetStream,"NetStream","")
+REGISTER_CLASS_NAME_AVM1(AVM1Key,"Key","")
+REGISTER_CLASS_NAME_AVM1(AVM1Mouse,"Mouse","")
+REGISTER_CLASS_NAME_AVM1(AVM1Sound,"Sound","")
+REGISTER_CLASS_NAME_AVM1(AVM1MovieClip,"MovieClip","")
+REGISTER_CLASS_NAME_AVM1(AVM1MovieClipLoader,"MovieClipLoader","")
+REGISTER_CLASS_NAME_AVM1(AVM1Shape,"Shape","")
+REGISTER_CLASS_NAME_AVM1(AVM1SimpleButton,"SimpleButton","")
+REGISTER_CLASS_NAME_AVM1(AVM1Stage,"Stage","")
+REGISTER_CLASS_NAME_AVM1(AVM1SharedObject,"SharedObject","")
+REGISTER_CLASS_NAME_AVM1(AVM1TextFormat,"TextFormat","")
+REGISTER_CLASS_NAME_AVM1(AVM1TextField,"TextField","")
+REGISTER_CLASS_NAME_AVM1(AVM1XMLDocument,"XML","")
+REGISTER_CLASS_NAME_AVM1(AVM1XMLNode,"XMLNode","")
+REGISTER_CLASS_NAME_AVM1(AVM1Video,"Video","")
+REGISTER_CLASS_NAME_AVM1(AVM1Color,"Color","")
+REGISTER_CLASS_NAME_AVM1(AVM1Broadcaster,"AsBroadcaster","")
+REGISTER_CLASS_NAME_AVM1(AVM1LocalConnection,"LocalConnection","")
+REGISTER_CLASS_NAME_AVM1(AVM1LoadVars,"LoadVars","")
+REGISTER_CLASS_NAME_AVM1(AVM1Bitmap,"Bitmap","")
+REGISTER_CLASS_NAME_AVM1(AVM1BitmapData,"BitmapData","")
+REGISTER_CLASS_NAME_AVM1(AVM1NetConnection,"NetConnection","")
+REGISTER_CLASS_NAME_AVM1(AVM1XMLSocket,"XMLSocket","")
+REGISTER_CLASS_NAME_AVM1(AVM1ContextMenuItem,"ContextMenuItem","")
+REGISTER_CLASS_NAME_AVM1(AVM1NetStream,"NetStream","")
+REGISTER_CLASS_NAME_AVM1(AVM1Array,"Array","")
+REGISTER_CLASS_NAME_AVM1(AVM1Date,"Date","")
+REGISTER_CLASS_NAME_AVM1(AVM1BitmapFilter,"BitmapFilter","flash.filters")
