@@ -38,7 +38,7 @@ extern "C"
 #ifndef AVCODEC_MAX_AUDIO_FRAME_SIZE
 #define AVCODEC_MAX_AUDIO_FRAME_SIZE 192000 // 1 second of 48khz 32bit audio
 #endif
-#ifdef HAVE_AVCODECID
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(54,51,100)
 #define CodecID AVCodecID
 #define CODEC_ID_NONE AV_CODEC_ID_NONE
 #define CODEC_ID_H264 AV_CODEC_ID_H264
@@ -47,10 +47,13 @@ extern "C"
 #define CODEC_ID_VP6A AV_CODEC_ID_VP6A
 #define CODEC_ID_AAC AV_CODEC_ID_AAC
 #define CODEC_ID_MP3 AV_CODEC_ID_MP3
+#define CODEC_ID_PCM_S16BE AV_CODEC_ID_PCM_S16BE
 #define CODEC_ID_PCM_S16LE AV_CODEC_ID_PCM_S16LE
 #define CODEC_ID_PCM_F32BE AV_CODEC_ID_PCM_F32BE
 #define CODEC_ID_PCM_F32LE AV_CODEC_ID_PCM_F32LE
 #define CODEC_ID_ADPCM_SWF AV_CODEC_ID_ADPCM_SWF
+#define CODEC_ID_GIF AV_CODEC_ID_GIF
+
 #endif
 #define MAX_AUDIO_FRAME_SIZE AVCODEC_MAX_AUDIO_FRAME_SIZE
 }
@@ -64,10 +67,6 @@ extern "C"
 namespace lightspark
 {
 
-enum LS_VIDEO_CODEC { H264=0, H263, VP6, VP6A };
-// "Audio coding formats" from Chapter 11 in SWF documentation (except for LINEAR_PCM_FLOAT_BE)
-enum LS_AUDIO_CODEC { CODEC_NONE=-1, LINEAR_PCM_PLATFORM_ENDIAN=0, ADPCM=1, MP3=2, LINEAR_PCM_LE=3, AAC=10, LINEAR_PCM_FLOAT_BE = 100 };
-
 class AudioFormat
 {
 public:
@@ -77,6 +76,7 @@ public:
 	int channels;
 };
 class NetStream;
+class EngineData;
 
 class Decoder
 {
@@ -95,12 +95,20 @@ public:
 	virtual void setFlushing()=0;
 	void waitFlushed()
 	{
+		if (status != VALID)
+			return;
 		flushed.wait();
 	}
+	bool isFlushed() const
+	{
+		return status == FLUSHED;
+	}
 };
 class DefineVideoStreamTag;
 class VideoDecoder: public Decoder, public ITextureUploadable
 {
+protected:
+	uint8_t* decodedframebuffer;
 public:
 	VideoDecoder();
 	virtual ~VideoDecoder();
@@ -126,11 +134,12 @@ public:
 	void waitForFencing();
 	//ITextureUploadable interface
 	void sizeNeeded(uint32_t& w, uint32_t& h) const;
-	const TextureChunk& getTexture();
+	TextureChunk& getTexture();
 	void uploadFence();
 	void markForDestruction();
 	bool isUploading() { return fenceCount; }
 	void setVideoFrameToDecode(uint32_t frame) { currentframe=frame; }
+	void clearFrameBuffer();
 protected:
 	TextureChunk videoTexture;
 	uint32_t frameWidth;
@@ -164,8 +173,9 @@ public:
 		flushing=true;
 	}
 	//ITextureUploadable interface
-	void upload(uint8_t* data, uint32_t w, uint32_t h) override
+	uint8_t* upload(bool refresh) override
 	{
+		return nullptr;
 	}
 };
 #ifdef ENABLE_LIBAVCODEC
@@ -200,29 +210,38 @@ private:
 			ch[2]=nullptr;
 			ch[3]=nullptr;
 		}
+		void init()
+		{
+			ch[0]=nullptr;
+			ch[1]=nullptr;
+			ch[2]=nullptr;
+			ch[3]=nullptr;
+		}
+		void cleanup()
+		{
+			setDecodedData(nullptr);
+		}
 	};
 	class YUVBufferGenerator
 	{
 	private:
 		uint32_t bufferSize;
 		bool hasAlpha;
+		bool hasChannels;
 	public:
-		YUVBufferGenerator(uint32_t b, bool _hasalpha):bufferSize(b),hasAlpha(_hasalpha){}
+		YUVBufferGenerator(uint32_t b, bool _hasalpha, bool _haschannels):bufferSize(b),hasAlpha(_hasalpha),hasChannels(_haschannels){}
 		void init(YUVBuffer& buf) const;
 	};
 	bool ownedContext;
 	uint32_t curBuffer;
 	AVCodecContext* codecContext;
-	BlockingCircularQueue<YUVBuffer,FFMPEGVIDEODECODERBUFFERSIZE> streamingbuffers;
-	BlockingCircularQueue<YUVBuffer,2> embeddedbuffers;
-	Mutex mutex;
+	BlockingCircularQueue<YUVBuffer> streamingbuffers;
+	BlockingCircularQueue<YUVBuffer> embeddedbuffers;
 	AVFrame* frameIn;
 	void copyFrameToBuffers(const AVFrame* frameIn, uint32_t time);
 	void setSize(uint32_t w, uint32_t h);
 	bool fillDataAndCheckValidity();
 	uint32_t curBufferOffset;
-	// used for embedded video
-	uint32_t currentcachedframe;
 	DefineVideoStreamTag* embeddedvideotag;
 public:
 	FFMpegVideoDecoder(LS_VIDEO_CODEC codec, uint8_t* initdata, uint32_t datalen, double frameRateHint,DefineVideoStreamTag* tag=nullptr);
@@ -265,14 +284,14 @@ public:
 		}
 	}
 	//ITextureUploadable interface
-	void upload(uint8_t* data, uint32_t w, uint32_t h) override;
+	uint8_t* upload(bool refresh) override;
 };
 #endif
 
 class AudioDecoder: public Decoder
 {
 protected:
-	class FrameSamples
+	class FrameSamplesS16
 	{
 	public:
 		int16_t samples[MAX_AUDIO_FRAME_SIZE/2];
@@ -280,37 +299,69 @@ protected:
 		int16_t* current;
 		uint32_t len;
 		uint32_t time;
-		FrameSamples():current(samples),len(0),time(0){}
+		FrameSamplesS16():current(samples),len(0),time(0){}
+		void init()
+		{
+			len=0;
+			time=0;
+		}
+		void cleanup()
+		{
+		}
+		
 	};
-	class FrameSamplesGenerator
+	class FrameSamplesF32
 	{
 	public:
-		void init(FrameSamples& f) const {f.len=0;}
+		float samples[MAX_AUDIO_FRAME_SIZE/2];
+		__attribute__ ((aligned (32)))
+		float* current;
+		uint32_t len;
+		uint32_t time;
+		FrameSamplesF32():current(samples),len(0),time(0){}
+		void init()
+		{
+			len=0;
+			time=0;
+		}
+		void cleanup()
+		{
+		}
+		
 	};
+#ifdef HAVE_LIBSWRESAMPLE
+	SwrContext* resamplecontext;
+#elif defined HAVE_LIBAVRESAMPLE
+	AVAudioResampleContext* resamplecontext;
+#endif
 public:
 	uint32_t sampleRate;
+	EngineData* engine;
 protected:
-	BlockingCircularQueue<FrameSamples,150> samplesBuffer;
+	BlockingCircularQueue<FrameSamplesS16> samplesBufferS16;
+	BlockingCircularQueue<FrameSamplesF32> samplesBufferF32;
+	virtual void samplesconsumed(uint32_t samples) {}
+	bool discardFrameS16();
+	bool discardFrameF32();
 public:
 	/**
 	  	The AudioDecoder contains audio buffers that must be aligned to 16 bytes, so we redefine the allocator
 	*/
-	void* operator new(size_t);
-	void operator delete(void*);
-	AudioDecoder():sampleRate(0),channelCount(0),initialTime(-1){}
-	virtual ~AudioDecoder(){}
+	AudioDecoder(uint32_t size, EngineData* _engine);
+	virtual ~AudioDecoder();
 	virtual void switchCodec(LS_AUDIO_CODEC codecId, uint8_t* initdata, uint32_t datalen)=0;
 	virtual uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time)=0;
 	bool hasDecodedFrames() const
 	{
-		return !samplesBuffer.isEmpty();
+		return !samplesBufferS16.isEmpty() || !samplesBufferF32.isEmpty();
 	}
 	uint32_t getFrontTime() const;
 	uint32_t getBytesPerMSec() const
 	{
 		return sampleRate*channelCount*2/1000;
 	}
-	uint32_t copyFrame(int16_t* dest, uint32_t len) DLL_PUBLIC;
+	uint32_t copyFrameS16(int16_t* dest, uint32_t len) DLL_PUBLIC;
+	uint32_t copyFrameF32(float* dest, uint32_t len) DLL_PUBLIC;
 	/**
 	  	Skip samples until the given time
 
@@ -323,10 +374,10 @@ public:
 	*/
 	void skipAll() DLL_PUBLIC;
 	bool discardFrame();
-	void setFlushing()
+	void setFlushing() override
 	{
 		flushing=true;
-		if(samplesBuffer.isEmpty())
+		if(samplesBufferS16.isEmpty() && samplesBufferF32.isEmpty())
 		{
 			status=FLUSHED;
 			flushed.signal();
@@ -335,66 +386,84 @@ public:
 	uint32_t channelCount;
 	//Saves the timestamp of the first decoded frame
 	uint32_t initialTime;
+	// if true, decoder is only used for extracting parts meaning audio data will always be provided as 32bit floats for 2 channels
+	bool forExtraction;
 };
 
 class NullAudioDecoder: public AudioDecoder
 {
 public:
-	NullAudioDecoder()
+	NullAudioDecoder():AudioDecoder(0,nullptr)
 	{
 		status=VALID;
 		sampleRate=44100;
 		channelCount=2;
 	}
-	void switchCodec(LS_AUDIO_CODEC codecId, uint8_t* initdata, uint32_t datalen){}
-	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time){return 0;}
+	void switchCodec(LS_AUDIO_CODEC codecId, uint8_t* initdata, uint32_t datalen) override {}
+	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time) override {return 0;}
 };
 
+class SoundChannel;
+// this is the AudioDecoder for streaming Sounds by SampleDataEvent
+class SampleDataAudioDecoder: public AudioDecoder
+{
+private:
+	ATOMIC_INT32(bufferedsamples);
+	SoundChannel* soundchannel;
+protected:
+	void samplesconsumed(uint32_t samples) override;
+public:
+	SampleDataAudioDecoder(SoundChannel* _soundchannel,uint32_t buffertime,EngineData* engine):AudioDecoder(buffertime,engine),soundchannel(_soundchannel)
+	{
+		sampleRate=44100;
+		channelCount=2;
+		bufferedsamples=0;
+	}
+	void switchCodec(LS_AUDIO_CODEC codecId, uint8_t* initdata, uint32_t datalen) override {}
+	// the data is always expected to be floats
+	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time) override;
+	inline int32_t getBufferedSamples() const { return  bufferedsamples; }
+};
+
+
 #ifdef ENABLE_LIBAVCODEC
-class EngineData;
 class FFMpegAudioDecoder: public AudioDecoder
 {
 private:
-	EngineData* engine;
 	bool ownedContext;
 	AVCodecContext* codecContext;
-#ifdef HAVE_LIBSWRESAMPLE
-	SwrContext* resamplecontext;
-#elif defined HAVE_LIBAVRESAMPLE
-	AVAudioResampleContext* resamplecontext;
-#endif
 	std::vector<uint8_t> overflowBuffer;
 	bool fillDataAndCheckValidity();
 	CodecID LSToFFMpegCodec(LS_AUDIO_CODEC lscodec);
-#ifdef HAVE_AVCODEC_DECODE_AUDIO4
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57,106,102)
 	AVFrame* frameIn;
-	int resampleFrameToS16(FrameSamples& curTail);
+	void resampleFrame(uint8_t** output, int& outputsize);
 #endif
 public:
-	FFMpegAudioDecoder(EngineData* eng,LS_AUDIO_CODEC codec, uint8_t* initdata, uint32_t datalen);
-	FFMpegAudioDecoder(EngineData* eng,LS_AUDIO_CODEC codec, int sampleRate, int channels, bool);
+	FFMpegAudioDecoder(EngineData* eng,LS_AUDIO_CODEC codec, uint8_t* initdata, uint32_t datalen, uint32_t buffertime);
+	FFMpegAudioDecoder(EngineData* eng,LS_AUDIO_CODEC codec, int sampleRate, int channels, uint32_t buffertime,bool);
 	/*
 	   Specialized constructor used by FFMpegStreamDecoder
 	*/
 #if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(57, 40, 101)
-	FFMpegAudioDecoder(EngineData* eng,AVCodecParameters* codecPar);
+	FFMpegAudioDecoder(EngineData* eng,AVCodecParameters* codecPar, uint32_t buffertime);
 #else
-	FFMpegAudioDecoder(EngineData* eng,AVCodecContext* codecContext);
+	FFMpegAudioDecoder(EngineData* eng,AVCodecContext* codecContext, uint32_t buffertime);
 #endif
 	~FFMpegAudioDecoder();
 	/*
 	   Specialized decoding used by FFMpegStreamDecoder
 	*/
-	uint32_t decodePacket(AVPacket* pkt, uint32_t time);
-	void switchCodec(LS_AUDIO_CODEC audioCodec, uint8_t* initdata, uint32_t datalen);
-	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time);
+	int decodePacket(AVPacket* pkt, uint32_t time);
+	void switchCodec(LS_AUDIO_CODEC audioCodec, uint8_t* initdata, uint32_t datalen) override;
+	uint32_t decodeData(uint8_t* data, int32_t datalen, uint32_t time) override;
 };
 #endif
 
 class StreamDecoder
 {
 public:
-	StreamDecoder():audioDecoder(NULL),videoDecoder(NULL),valid(false),hasvideo(false){}
+	StreamDecoder():audioDecoder(nullptr),videoDecoder(nullptr),valid(false),hasvideo(false),atend(false){}
 	virtual ~StreamDecoder();
 	virtual bool decodeNextFrame() = 0;
 	virtual void jumpToPosition(number_t position) = 0;
@@ -402,9 +471,11 @@ public:
 	AudioDecoder* audioDecoder;
 	VideoDecoder* videoDecoder;
 	bool hasVideo() const { return hasvideo; }
+	bool atEnd() const  { return atend; }
 protected:
 	bool valid;
 	bool hasvideo;
+	bool atend;
 };
 
 #ifdef ENABLE_LIBAVCODEC
@@ -424,6 +495,7 @@ private:
 	//Helpers for custom I/O of libavformat
 	uint8_t* avioBuffer;
 	static int avioReadPacket(void* t, uint8_t* buf, int buf_size);
+	static int64_t avioSeek(void *opaque, int64_t offset, int whence);
 	//NOTE: this will become AVIOContext in FFMpeg 0.7
 #if LIBAVUTIL_VERSION_MAJOR < 51
 	ByteIOContext* avioContext;
@@ -431,8 +503,9 @@ private:
 	AVIOContext* avioContext;
 #endif
 	int availablestreamlength;
+	int fullstreamlength;
 public:
-	FFMpegStreamDecoder(NetStream* ns,EngineData* eng,std::istream& s, AudioFormat* format = NULL, int streamsize = -1);
+	FFMpegStreamDecoder(NetStream* ns,EngineData* eng,std::istream& s, uint32_t buffertime, AudioFormat* format = nullptr, int streamsize = -1, bool forExtraction=false);
 	~FFMpegStreamDecoder();
 	void jumpToPosition(number_t position) override;
 	bool decodeNextFrame() override;
