@@ -24,7 +24,6 @@
 #include "logger.h"
 #include "backends/geometry.h"
 #include "compat.h"
-#include "scripting/flash/display/flashdisplay.h"
 #include "scripting/flash/display/BitmapData.h"
 
 using namespace std;
@@ -68,13 +67,13 @@ static void joinOutlines(vector<ShapePathSegment>& segments)
 	{
 		auto s = segments[i];
 		unconsumed.insert(i);
-		byStartPos.insert(make_pair(s.from, i));
-		byEndPos.insert(make_pair(s.to, i));
+		byStartPos.insert(make_pair(s.from.key, i));
+		byEndPos.insert(make_pair(s.to.key, i));
 	}
 
 	vector<ShapePathSegment> res;
 	int i = -1;
-	ShapePathSegment prev(0, 0, 0, 0);
+	ShapePathSegment prev;
 	while (!unconsumed.empty())
 	{
 		bool reverse = false;
@@ -82,11 +81,11 @@ static void joinOutlines(vector<ShapePathSegment>& segments)
 		{
 			multimap<uint64_t, int>::const_iterator it;
 			auto next_it = unconsumed.upper_bound(i);
-			if (next_it != unconsumed.end() && segments[*next_it].from == prev.to)
+			if (next_it != unconsumed.end() && segments[*next_it].from.key == prev.to.key)
 				i = *next_it;
-			else if ((it = byStartPos.find(prev.to)) != byStartPos.end())
+			else if ((it = byStartPos.find(prev.to.key)) != byStartPos.end())
 				i = it->second;
-			else if ((it = byEndPos.find(prev.to)) != byEndPos.end()) {
+			else if ((it = byEndPos.find(prev.to.key)) != byEndPos.end()) {
 				i = it->second;
 				reverse = true;
 			}
@@ -98,8 +97,8 @@ static void joinOutlines(vector<ShapePathSegment>& segments)
 			i = *unconsumed.begin();
 
 		prev = segments[i];
-		mmremove(byStartPos, prev.from, i);
-		mmremove(byEndPos, prev.to, i);
+		mmremove(byStartPos, prev.from.key, i);
+		mmremove(byEndPos, prev.to.key, i);
 		if (reverse)
 			prev = prev.reverse();
 		res.push_back(prev);
@@ -109,19 +108,19 @@ static void joinOutlines(vector<ShapePathSegment>& segments)
 	segments = res;
 }
 
-void ShapesBuilder::extendOutline(const Vector2& v1, const Vector2& v2, int linestyleindex)
+void ShapesBuilder::extendOutline(const Vector2f& v1, const Vector2f& v2, int linestyleindex)
 {
-	uint64_t v1Index = makeVertex(v1);
-	uint64_t v2Index = makeVertex(v2);
+	floatVec v1Index = makeVertex(v1);
+	floatVec v2Index = makeVertex(v2);
 
 	currentSubpath.emplace_back(v1Index, v2Index, v2Index, linestyleindex);
 }
 
-void ShapesBuilder::extendOutlineCurve(const Vector2& v1, const Vector2& v2, const Vector2& v3, int linestyleindex)
+void ShapesBuilder::extendOutlineCurve(const Vector2f& v1, const Vector2f& v2, const Vector2f& v3, int linestyleindex)
 {
-	uint64_t v1Index = makeVertex(v1);
-	uint64_t v2Index = makeVertex(v2);
-	uint64_t v3Index = makeVertex(v3);
+	floatVec v1Index = makeVertex(v1);
+	floatVec v2Index = makeVertex(v2);
+	floatVec v3Index = makeVertex(v3);
 
 	currentSubpath.emplace_back(v1Index, v2Index, v3Index, linestyleindex);
 }
@@ -159,56 +158,49 @@ void ShapesBuilder::endSubpathForStyles(unsigned fill0, unsigned fill1, unsigned
 	currentSubpath.clear();
 }
 
-void ShapesBuilder::outputTokens(const std::list<FILLSTYLE> &styles, const std::list<LINESTYLE2> &linestyles, tokensVector& tokens)
+void ShapesBuilder::outputTokens(const std::list<FILLSTYLE> &styles, const std::list<LINESTYLE2> &linestyles, tokensVector& tokens, bool isGlyph)
 {
 	assert(currentSubpath.empty());
 	auto it=filledShapesMap.begin();
 	//For each color
 	for(;it!=filledShapesMap.end();++it)
 	{
+		if (!tokens.filltokens)
+			tokens.filltokens=_MR(new tokenListRef());
 		joinOutlines(it->second);
-
-		//Find the style given the index
-		auto stylesIt=styles.begin();
-		assert(it->first);
-		for(unsigned int i=0;i<it->first-1;i++)
+		if(!isGlyph)
 		{
-			++stylesIt;
-			assert(stylesIt!=styles.end());
-		}
-		//Set the fill style
-		tokens.filltokens.push_back(GeomToken(SET_FILL).uval);
-		tokens.filltokens.push_back(GeomToken(*stylesIt).uval);
-		switch ((*stylesIt).FillStyleType)
-		{
-			case LINEAR_GRADIENT:
-			case RADIAL_GRADIENT:
-			case FOCAL_RADIAL_GRADIENT:
-				// TODO gradient fillstyles not yet implemented for nanoGL
-				tokens.canRenderToGL=false;
-				break;
-			default:
-				break;
+			//Find the style given the index
+			auto stylesIt=styles.begin();
+			assert(it->first);
+			for(unsigned int i=0;i<it->first-1;i++)
+			{
+				++stylesIt;
+				assert(stylesIt!=styles.end());
+			}
+			//Set the fill style
+			tokens.filltokens->tokens.push_back(GeomToken(SET_FILL).uval);
+			tokens.filltokens->tokens.push_back(GeomToken(*stylesIt).uval);
 		}
 		vector<ShapePathSegment>& segments = it->second;
 		for (size_t j = 0; j < segments.size(); ++j)
 		{
 			ShapePathSegment segment = segments[j];
-			if (j == 0 || segment.from != segments[j-1].to) {
-				tokens.filltokens.push_back(GeomToken(MOVE).uval);
-				tokens.filltokens.push_back(GeomToken(segment.from,true).uval);
+			if (j == 0 || segment.from.key != segments[j-1].to.key) {
+				tokens.filltokens->tokens.push_back(GeomToken(MOVE).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.from).uval);
 			}
-			if (segment.quadctrl == segment.from || segment.quadctrl == segment.to) {
-				tokens.filltokens.push_back(GeomToken(STRAIGHT).uval);
-				tokens.filltokens.push_back(GeomToken(segment.to,true).uval);
+			if (segment.quadctrl.key == segment.from.key || segment.quadctrl.key == segment.to.key) {
+				tokens.filltokens->tokens.push_back(GeomToken(STRAIGHT).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.to).uval);
 			}
 			else {
-				tokens.filltokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
-				tokens.filltokens.push_back(GeomToken(segment.quadctrl,true).uval);
-				tokens.filltokens.push_back(GeomToken(segment.to,true).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.quadctrl).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.to).uval);
 			}
 		}
-		tokens.filltokens.push_back(GeomToken(CLEAR_FILL).uval);
+		tokens.filltokens->tokens.push_back(GeomToken(CLEAR_FILL).uval);
 		int currentlinestyle=0;
 		// add tokens for strokes intertwined with current fill
 		for (size_t j = 0; j < segments.size(); ++j)
@@ -227,43 +219,41 @@ void ShapesBuilder::outputTokens(const std::list<FILLSTYLE> &styles, const std::
 						assert(linestylesIt!=linestyles.end());
 					}
 					//Set the line style for strokes inside filltokens
-					tokens.filltokens.push_back(GeomToken(SET_STROKE).uval);
-					tokens.filltokens.push_back(GeomToken(*linestylesIt).uval);
-					if (tokens.currentLineWidth < (*linestylesIt).Width)
-						tokens.currentLineWidth = (*linestylesIt).Width;
-					if ((*linestylesIt).HasFillFlag) // TODO linestyles with fill flag not yet implemented for nanoGL
-						tokens.canRenderToGL=false;
+					tokens.filltokens->tokens.push_back(GeomToken(SET_STROKE).uval);
+					tokens.filltokens->tokens.push_back(GeomToken(*linestylesIt).uval);
 					for (size_t k = j; k < segments.size(); ++k)
 					{
 						ShapePathSegment strokesegment = segments[k];
 						if (strokesegment.linestyleindex != currentlinestyle)
 						{
 							j=k;
-							tokens.filltokens.push_back(GeomToken(CLEAR_STROKE).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(CLEAR_STROKE).uval);
 							break;
 						}
-						if (k == j || strokesegment.from != segments[k-1].to) {
-							tokens.filltokens.push_back(GeomToken(MOVE).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.from,true).uval);
+						if (k == j || strokesegment.from.key != segments[k-1].to.key) {
+							tokens.filltokens->tokens.push_back(GeomToken(MOVE).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.from).uval);
 						}
-						if (strokesegment.quadctrl == strokesegment.from || strokesegment.quadctrl == strokesegment.to) {
-							tokens.filltokens.push_back(GeomToken(STRAIGHT).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.to,true).uval);
+						if (strokesegment.quadctrl.key == strokesegment.from.key || strokesegment.quadctrl.key == strokesegment.to.key) {
+							tokens.filltokens->tokens.push_back(GeomToken(STRAIGHT).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.to).uval);
 						}
 						else {
-							tokens.filltokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.quadctrl,true).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.to,true).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.quadctrl).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.to).uval);
 						}
 					}
 				}
 			}
 		}
 		if (currentlinestyle)
-			tokens.filltokens.push_back(GeomToken(CLEAR_STROKE).uval);
+			tokens.filltokens->tokens.push_back(GeomToken(CLEAR_STROKE).uval);
 	}
 	if (strokeShapesMap.size() > 0)
 	{
+		if (!tokens.stroketokens)
+			tokens.stroketokens=_MR(new tokenListRef());
 		it=strokeShapesMap.begin();
 		//For each stroke
 		for(;it!=strokeShapesMap.end();++it)
@@ -280,27 +270,23 @@ void ShapesBuilder::outputTokens(const std::list<FILLSTYLE> &styles, const std::
 			}
 			//Set the line style
 			vector<ShapePathSegment>& segments = it->second;
-			tokens.stroketokens.push_back(GeomToken(SET_STROKE).uval);
-			tokens.stroketokens.push_back(GeomToken(*stylesIt).uval);
-			if (tokens.currentLineWidth < (*stylesIt).Width)
-				tokens.currentLineWidth = (*stylesIt).Width;
-			if ((*stylesIt).HasFillFlag) // TODO linestyles with fill flag not yet implemented for nanoGL
-				tokens.canRenderToGL=false;
+			tokens.stroketokens->tokens.push_back(GeomToken(SET_STROKE).uval);
+			tokens.stroketokens->tokens.push_back(GeomToken(*stylesIt).uval);
 			for (size_t j = 0; j < segments.size(); ++j)
 			{
 				ShapePathSegment segment = segments[j];
-				if (j == 0 || segment.from != segments[j - 1].to) {
-					tokens.stroketokens.push_back(GeomToken(MOVE).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.from,true).uval);
+				if (j == 0 || segment.from.key != segments[j - 1].to.key) {
+					tokens.stroketokens->tokens.push_back(GeomToken(MOVE).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.from).uval);
 				}
-				if (segment.quadctrl == segment.from || segment.quadctrl == segment.to) {
-					tokens.stroketokens.push_back(GeomToken(STRAIGHT).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.to,true).uval);
+				if (segment.quadctrl.key == segment.from.key || segment.quadctrl.key == segment.to.key) {
+					tokens.stroketokens->tokens.push_back(GeomToken(STRAIGHT).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.to).uval);
 				}
 				else {
-					tokens.stroketokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.quadctrl,true).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.to,true).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.quadctrl).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.to).uval);
 				}
 			}
 		}
@@ -335,14 +321,16 @@ std::map<uint16_t,LINESTYLE2>::iterator ShapesBuilder::getStrokeLineStyle(const
 						number_t gradratio = float(ratio)/65535.0;
 						MATRIX ratiomatrix;
 						
-						ratiomatrix.scale(stylesIt->FillType.StartGradientMatrix.getScaleX()+(stylesIt->FillType.EndGradientMatrix.getScaleX()-stylesIt->FillType.StartGradientMatrix.getScaleX())*gradratio,
-										  stylesIt->FillType.StartGradientMatrix.getScaleY()+(stylesIt->FillType.EndGradientMatrix.getScaleY()-stylesIt->FillType.StartGradientMatrix.getScaleY())*gradratio);
-						ratiomatrix.rotate((stylesIt->FillType.StartGradientMatrix.getRotation()+(stylesIt->FillType.EndGradientMatrix.getRotation()-stylesIt->FillType.StartGradientMatrix.getRotation())*gradratio)*180.0/M_PI);
-						ratiomatrix.translate(stylesIt->FillType.StartGradientMatrix.getTranslateX() +(stylesIt->FillType.EndGradientMatrix.getTranslateX()-stylesIt->FillType.StartGradientMatrix.getTranslateX())*gradratio,
-											  stylesIt->FillType.StartGradientMatrix.getTranslateY() +(stylesIt->FillType.EndGradientMatrix.getTranslateY()-stylesIt->FillType.StartGradientMatrix.getTranslateY())*gradratio);
+						ratiomatrix.x0=stylesIt->FillType.StartGradientMatrix.x0*(1.0-gradratio)+stylesIt->FillType.EndGradientMatrix.x0*gradratio;
+						ratiomatrix.y0=stylesIt->FillType.StartGradientMatrix.y0*(1.0-gradratio)+stylesIt->FillType.EndGradientMatrix.y0*gradratio;
+						ratiomatrix.xx=stylesIt->FillType.StartGradientMatrix.xx*(1.0-gradratio)+stylesIt->FillType.EndGradientMatrix.xx*gradratio;
+						ratiomatrix.yx=stylesIt->FillType.StartGradientMatrix.yx*(1.0-gradratio)+stylesIt->FillType.EndGradientMatrix.yx*gradratio;
+						ratiomatrix.xy=stylesIt->FillType.StartGradientMatrix.xy*(1.0-gradratio)+stylesIt->FillType.EndGradientMatrix.xy*gradratio;
+						ratiomatrix.yy=stylesIt->FillType.StartGradientMatrix.yy*(1.0-gradratio)+stylesIt->FillType.EndGradientMatrix.yy*gradratio;
 						
 						ls.FillType.Matrix = ratiomatrix;
-						std::vector<GRADRECORD>& gradrecords = stylesIt->FillType.FillStyleType==FOCAL_RADIAL_GRADIENT ? ls.FillType.FocalGradient.GradientRecords : ls.FillType.Gradient.GradientRecords;
+						ls.FillType.bitmap=_MNR(new BitmapContainer(nullptr));
+						std::vector<GRADRECORD>& gradrecords = ls.FillType.Gradient.GradientRecords;
 						gradrecords.reserve(stylesIt->FillType.StartColors.size());
 						GRADRECORD gr(0xff);
 						for (uint32_t i1=0; i1 < stylesIt->FillType.StartColors.size(); i1++)
@@ -368,9 +356,9 @@ std::map<uint16_t,LINESTYLE2>::iterator ShapesBuilder::getStrokeLineStyle(const
 										break;
 									}
 							}
-							ls.FillType.FocalGradient.InterpolationMode=stylesIt->FillType.InterpolationMode;
-							ls.FillType.FocalGradient.SpreadMode=stylesIt->FillType.SpreadMode;
-							ls.FillType.FocalGradient.FocalPoint=stylesIt->FillType.StartFocalPoint + (stylesIt->FillType.EndFocalPoint-stylesIt->FillType.StartFocalPoint)*gradratio;
+							ls.FillType.Gradient.InterpolationMode=stylesIt->FillType.InterpolationMode;
+							ls.FillType.Gradient.SpreadMode=stylesIt->FillType.SpreadMode;
+							ls.FillType.Gradient.FocalPoint=stylesIt->FillType.StartFocalPoint + (stylesIt->FillType.EndFocalPoint-stylesIt->FillType.StartFocalPoint)*gradratio;
 							gradrecords.push_back(gr);
 							ls.FillType.ShapeBounds = boundsrc;
 						}
@@ -395,6 +383,7 @@ std::map<uint16_t,LINESTYLE2>::iterator ShapesBuilder::getStrokeLineStyle(const
 									break;
 								}
 						}
+						ls.FillType.ShapeBounds = boundsrc;
 						break;
 					}
 				default:
@@ -421,6 +410,7 @@ std::map<uint16_t,LINESTYLE2>::iterator ShapesBuilder::getStrokeLineStyle(const
 						break;
 					}
 			}
+			ls.Color = ls.FillType.Color;
 		}
 		itls = stylesIt->linestylecache.insert(make_pair(ratio,ls)).first;
 	}
@@ -428,11 +418,13 @@ std::map<uint16_t,LINESTYLE2>::iterator ShapesBuilder::getStrokeLineStyle(const
 }
 void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::list<MORPHLINESTYLE2> &linestyles, tokensVector &tokens, uint16_t ratio, const RECT& boundsrc)
 {
-	assert(currentSubpath.empty());
+//	assert(currentSubpath.empty());
 	auto it=filledShapesMap.begin();
 	//For each color
 	for(;it!=filledShapesMap.end();++it)
 	{
+		if (!tokens.filltokens)
+			tokens.filltokens=_MR(new tokenListRef());
 		joinOutlines(it->second);
 
 		//Find the style given the index
@@ -454,18 +446,19 @@ void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::li
 				case RADIAL_GRADIENT:
 				case FOCAL_RADIAL_GRADIENT:
 				{
-					tokens.canRenderToGL=false;//  TODO fillstyles other than solid fill not yet implemented for nanoGL
+					f.bitmap=_MNR(new BitmapContainer(nullptr));
 					number_t gradratio = float(ratio)/65535.0;
 					MATRIX ratiomatrix;
-
-					ratiomatrix.scale(stylesIt->StartGradientMatrix.getScaleX()+(stylesIt->EndGradientMatrix.getScaleX()-stylesIt->StartGradientMatrix.getScaleX())*gradratio,
-									  stylesIt->StartGradientMatrix.getScaleY()+(stylesIt->EndGradientMatrix.getScaleY()-stylesIt->StartGradientMatrix.getScaleY())*gradratio);
-					ratiomatrix.rotate((stylesIt->StartGradientMatrix.getRotation()+(stylesIt->EndGradientMatrix.getRotation()-stylesIt->StartGradientMatrix.getRotation())*gradratio)*180.0/M_PI);
-					ratiomatrix.translate(stylesIt->StartGradientMatrix.getTranslateX() +(stylesIt->EndGradientMatrix.getTranslateX()-stylesIt->StartGradientMatrix.getTranslateX())*gradratio,
-										  stylesIt->StartGradientMatrix.getTranslateY() +(stylesIt->EndGradientMatrix.getTranslateY()-stylesIt->StartGradientMatrix.getTranslateY())*gradratio);
+					
+					ratiomatrix.x0=stylesIt->StartGradientMatrix.x0*(1.0-gradratio)+stylesIt->EndGradientMatrix.x0*gradratio;
+					ratiomatrix.y0=stylesIt->StartGradientMatrix.y0*(1.0-gradratio)+stylesIt->EndGradientMatrix.y0*gradratio;
+					ratiomatrix.xx=stylesIt->StartGradientMatrix.xx*(1.0-gradratio)+stylesIt->EndGradientMatrix.xx*gradratio;
+					ratiomatrix.yx=stylesIt->StartGradientMatrix.yx*(1.0-gradratio)+stylesIt->EndGradientMatrix.yx*gradratio;
+					ratiomatrix.xy=stylesIt->StartGradientMatrix.xy*(1.0-gradratio)+stylesIt->EndGradientMatrix.xy*gradratio;
+					ratiomatrix.yy=stylesIt->StartGradientMatrix.yy*(1.0-gradratio)+stylesIt->EndGradientMatrix.yy*gradratio;
 
 					f.Matrix = ratiomatrix;
-					std::vector<GRADRECORD>& gradrecords = stylesIt->FillStyleType==FOCAL_RADIAL_GRADIENT ? f.FocalGradient.GradientRecords : f.Gradient.GradientRecords;
+					std::vector<GRADRECORD>& gradrecords = f.Gradient.GradientRecords;
 					gradrecords.reserve(stylesIt->StartColors.size());
 					GRADRECORD gr(0xff);
 					uint8_t compratio = ratio>>8;
@@ -492,9 +485,9 @@ void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::li
 								break;
 							}
 						}
-						f.FocalGradient.InterpolationMode=stylesIt->InterpolationMode;
-						f.FocalGradient.SpreadMode=stylesIt->SpreadMode;
-						f.FocalGradient.FocalPoint=stylesIt->StartFocalPoint + (stylesIt->EndFocalPoint-stylesIt->StartFocalPoint)*gradratio;
+						f.Gradient.InterpolationMode=stylesIt->InterpolationMode;
+						f.Gradient.SpreadMode=stylesIt->SpreadMode;
+						f.Gradient.FocalPoint=stylesIt->StartFocalPoint*256.0 + (stylesIt->EndFocalPoint-stylesIt->StartFocalPoint)*gradratio*256.0;
 						gradrecords.push_back(gr);
 						f.ShapeBounds = boundsrc;
 					}
@@ -520,6 +513,7 @@ void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::li
 							break;
 						}
 					}
+					f.ShapeBounds = boundsrc;
 					break;
 				}
 				default:
@@ -528,27 +522,27 @@ void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::li
 			}
 			itfs = stylesIt->fillstylecache.insert(make_pair(ratio,f)).first;
 		}
-		tokens.filltokens.push_back(GeomToken(SET_FILL).uval);
-		tokens.filltokens.push_back(GeomToken((*itfs).second).uval);
+		tokens.filltokens->tokens.push_back(GeomToken(SET_FILL).uval);
+		tokens.filltokens->tokens.push_back(GeomToken((*itfs).second).uval);
 		vector<ShapePathSegment>& segments = it->second;
 		for (size_t j = 0; j < segments.size(); ++j)
 		{
 			ShapePathSegment segment = segments[j];
-			if (j == 0 || segment.from != segments[j - 1].to) {
-				tokens.filltokens.push_back(GeomToken(MOVE).uval);
-				tokens.filltokens.push_back(GeomToken(segment.from,true).uval);
+			if (j == 0 || segment.from.key != segments[j - 1].to.key) {
+				tokens.filltokens->tokens.push_back(GeomToken(MOVE).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.from).uval);
 			}
-			if (segment.quadctrl == segment.from || segment.quadctrl == segment.to) {
-				tokens.filltokens.push_back(GeomToken(STRAIGHT).uval);
-				tokens.filltokens.push_back(GeomToken(segment.to,true).uval);
+			if (segment.quadctrl.key == segment.from.key || segment.quadctrl.key == segment.to.key) {
+				tokens.filltokens->tokens.push_back(GeomToken(STRAIGHT).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.to).uval);
 			}
 			else {
-				tokens.filltokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
-				tokens.filltokens.push_back(GeomToken(segment.quadctrl,true).uval);
-				tokens.filltokens.push_back(GeomToken(segment.to,true).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.quadctrl).uval);
+				tokens.filltokens->tokens.push_back(GeomToken(segment.to).uval);
 			}
 		}
-		tokens.filltokens.push_back(GeomToken(CLEAR_FILL).uval);
+		tokens.filltokens->tokens.push_back(GeomToken(CLEAR_FILL).uval);
 		// add tokens for strokes intertwined with current fill
 		int currentlinestyle=0;
 		for (size_t j = 0; j < segments.size(); ++j)
@@ -565,47 +559,43 @@ void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::li
 						++linestylesIt;
 						assert(linestylesIt!=linestyles.end());
 					}
-					if ((*linestylesIt).HasFillFlag) // TODO linestyles with fill flag not yet implemented for nanoGL
-						tokens.canRenderToGL=false;
 					auto itls = getStrokeLineStyle(linestylesIt,ratio,&linestylesIt->linestylecache,boundsrc);
 					//Set the line style for strokes inside filltokens
-					tokens.filltokens.push_back(GeomToken(SET_STROKE).uval);
-					tokens.filltokens.push_back(GeomToken((*itls).second).uval);
-					if (tokens.currentLineWidth < (*itls).second.Width)
-						tokens.currentLineWidth = (*itls).second.Width;
-					if ((*linestylesIt).HasFillFlag) // TODO linestyles with fill flag not yet implemented for nanoGL
-						tokens.canRenderToGL=false;
+					tokens.filltokens->tokens.push_back(GeomToken(SET_STROKE).uval);
+					tokens.filltokens->tokens.push_back(GeomToken((*itls).second).uval);
 					for (size_t k = j; k < segments.size(); ++k)
 					{
 						ShapePathSegment strokesegment = segments[k];
 						if (strokesegment.linestyleindex != currentlinestyle)
 						{
 							j=k;
-							tokens.filltokens.push_back(GeomToken(CLEAR_STROKE).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(CLEAR_STROKE).uval);
 							break;
 						}
-						if (k == j || strokesegment.from != segments[k-1].to) {
-							tokens.filltokens.push_back(GeomToken(MOVE).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.from,true).uval);
+						if (k == j || strokesegment.from.key != segments[k-1].to.key) {
+							tokens.filltokens->tokens.push_back(GeomToken(MOVE).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.from).uval);
 						}
-						if (strokesegment.quadctrl == strokesegment.from || strokesegment.quadctrl == strokesegment.to) {
-							tokens.filltokens.push_back(GeomToken(STRAIGHT).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.to,true).uval);
+						if (strokesegment.quadctrl.key == strokesegment.from.key || strokesegment.quadctrl.key == strokesegment.to.key) {
+							tokens.filltokens->tokens.push_back(GeomToken(STRAIGHT).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.to).uval);
 						}
 						else {
-							tokens.filltokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.quadctrl,true).uval);
-							tokens.filltokens.push_back(GeomToken(strokesegment.to,true).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.quadctrl).uval);
+							tokens.filltokens->tokens.push_back(GeomToken(strokesegment.to).uval);
 						}
 					}
 				}
 			}
 		}
 		if (currentlinestyle)
-			tokens.filltokens.push_back(GeomToken(CLEAR_STROKE).uval);
+			tokens.filltokens->tokens.push_back(GeomToken(CLEAR_STROKE).uval);
 	}
 	if (strokeShapesMap.size() > 0)
 	{
+		if (!tokens.stroketokens)
+			tokens.stroketokens=_MR(new tokenListRef());
 		it=strokeShapesMap.begin();
 		//For each stroke
 		for(;it!=strokeShapesMap.end();++it)
@@ -619,32 +609,84 @@ void ShapesBuilder::outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::li
 				++stylesIt;
 				assert(stylesIt!=linestyles.end());
 			}
-			if ((*stylesIt).HasFillFlag) // TODO linestyles with fill flag not yet implemented for nanoGL
-				tokens.canRenderToGL=false;
 			//Set the line style
 			auto itls = getStrokeLineStyle(stylesIt,ratio,&stylesIt->linestylecache,boundsrc);
 			vector<ShapePathSegment>& segments = it->second;
-			tokens.stroketokens.push_back(GeomToken(SET_STROKE).uval);
-			tokens.stroketokens.push_back(GeomToken((*itls).second).uval);
-			if (tokens.currentLineWidth < (*itls).second.Width)
-				tokens.currentLineWidth = (*itls).second.Width;
+			tokens.stroketokens->tokens.push_back(GeomToken(SET_STROKE).uval);
+			tokens.stroketokens->tokens.push_back(GeomToken((*itls).second).uval);
 			for (size_t j = 0; j < segments.size(); ++j)
 			{
 				ShapePathSegment segment = segments[j];
-				if (j == 0 || segment.from != segments[j - 1].to) {
-					tokens.stroketokens.push_back(GeomToken(MOVE).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.from,true).uval);
+				if (j == 0 || segment.from.key != segments[j - 1].to.key) {
+					tokens.stroketokens->tokens.push_back(GeomToken(MOVE).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.from).uval);
 				}
-				if (segment.quadctrl == segment.from || segment.quadctrl == segment.to) {
-					tokens.stroketokens.push_back(GeomToken(STRAIGHT).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.to,true).uval);
+				if (segment.quadctrl.key == segment.from.key || segment.quadctrl.key == segment.to.key) {
+					tokens.stroketokens->tokens.push_back(GeomToken(STRAIGHT).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.to).uval);
 				}
 				else {
-					tokens.stroketokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.quadctrl,true).uval);
-					tokens.stroketokens.push_back(GeomToken(segment.to,true).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(CURVE_QUADRATIC).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.quadctrl).uval);
+					tokens.stroketokens->tokens.push_back(GeomToken(segment.to).uval);
 				}
 			}
 		}
 	}
 }
+
+void tokensVector::clear()
+{
+	filltokens.reset();
+	stroketokens.reset();
+	boundsRect = RECT(INT32_MAX,INT32_MIN,INT32_MAX,INT32_MIN);
+}
+
+void tokensVector::destruct()
+{
+	if (next)
+		next->destruct();
+	filltokens.reset();
+	stroketokens.reset();
+}
+
+tokenListRef::~tokenListRef()
+{
+	if (fillStyles)
+		delete fillStyles;
+	if (lineStyles)
+		delete lineStyles;
+}
+
+FILLSTYLE& tokenListRef::addFillStyle(const FILLSTYLE& fs)
+{
+	fillstylecache* cacheentry = new fillstylecache(fs);
+	cacheentry->next=fillStyles;
+	fillStyles=cacheentry;
+	return cacheentry->style;
+}
+
+LINESTYLE2& tokenListRef::addLineStyle(const LINESTYLE2& ls)
+{
+	linestylecache* cacheentry = new linestylecache(ls);
+	cacheentry->next=lineStyles;
+	lineStyles=cacheentry;
+	return cacheentry->style;
+}
+
+void tokenListRef::clone(tokenListRef* source)
+{
+	tokens.assign(source->tokens.begin(),source->tokens.end());
+	fillstylecache* fs = source->fillStyles;
+	while (fs)
+	{
+		addFillStyle(fs->style);
+		fs = fs->next;
+	}
+	linestylecache* ls = source->lineStyles;
+	while (ls)
+	{
+		addLineStyle(ls->style);
+		ls = ls->next;
+	}
+}
