@@ -25,6 +25,8 @@
 #include <vector>
 #include <map>
 
+typedef std::vector<uint64_t> TokenList;
+
 namespace lightspark
 {
 
@@ -35,18 +37,27 @@ class Vector2Tmpl
 public:
 	T x,y;
 	Vector2Tmpl(T a=0, T b=0):x(a),y(b){}
+	Vector2Tmpl& operator=(const Vector2Tmpl&) = default;
 	/* conversion Vector2 -> Vector2f is implicit */
 	Vector2Tmpl(const Vector2Tmpl<int32_t>& o) : x(o.x),y(o.y) {}
+	template<typename U>
+	Vector2Tmpl(const Vector2Tmpl<U>& o) : x(o.x),y(o.y) {}
 	/* conversion Vector2f -> Vector2 is explicit */
 	Vector2Tmpl<int32_t> round() const { return Vector2Tmpl<int32_t>(x,y); }
 	bool operator==(const Vector2Tmpl<T>& v)const{return v.x==x && v.y==y;}
 	bool operator!=(const Vector2Tmpl<T>& v)const{return v.x!=x || v.y!=y;}
-	bool operator<(const Vector2Tmpl<T>& v) const {return (y==v.y)?(x < v.x):(y < v.y);}
+	bool operator<(const Vector2Tmpl<T>& v) const { return x < v.x || (x == v.x && y < v.y); }
+	bool operator>(const Vector2Tmpl<T>& v) const { return v < *this; }
+	bool operator<=(const Vector2Tmpl<T>& v) const { return !(*this > v); }
+	bool operator>=(const Vector2Tmpl<T>& v) const { return !(*this < v); }
 	Vector2Tmpl<T> operator-() const { return Vector2Tmpl<T>(-x,-y); }
 	Vector2Tmpl<T> operator-(const Vector2Tmpl<T>& v)const { return Vector2Tmpl<T>(x-v.x,y-v.y);}
 	Vector2Tmpl<T> operator+(const Vector2Tmpl<T>& v)const { return Vector2Tmpl<T>(x+v.x,y+v.y);}
+	Vector2Tmpl<T> operator*(const Vector2Tmpl<T>& v)const { return Vector2Tmpl<T>(x*v.x,y*v.y);}
+	Vector2Tmpl<T> operator/(const Vector2Tmpl<T>& v)const { return Vector2Tmpl<T>(x/v.x,y/v.y);}
+	Vector2Tmpl<T> operator/(const T& v)const { return Vector2Tmpl<T>(x/v,y/v);}
 	Vector2Tmpl<T>& operator+=(const Vector2Tmpl<T>& v){ x+=v.x; y+=v.y; return *this;}
-	Vector2Tmpl<T> operator*(int p)const { return Vector2Tmpl<T>(x*p,y*p);}
+	Vector2Tmpl<T> operator*(T p)const { return Vector2Tmpl<T>(x*p,y*p);}
 	Vector2Tmpl<T>& operator/=(T v) { x/=v; y/=v; return *this;}
 	int dot(const Vector2Tmpl<T>& r) const { return x*r.x+y*r.y;}
 	Vector2Tmpl<T> projectInto(const RECT& r) const
@@ -56,15 +67,27 @@ public:
 		out.y = maxTmpl<T>(minTmpl<T>(y,r.Ymax),r.Ymin);
 		return out;
 	}
-	std::ostream& operator<<(std::ostream& s)
-	{
-		s << "{ "<< x << ',' << y << " }";
-		return s;
-	}
 };
 
+template<class T>
+std::ostream& operator<<(std::ostream& s, const Vector2Tmpl<T>& v)
+{
+	s << "{ "<< v.x << ',' << v.y << " }";
+	return s;
+}
+
 enum GEOM_TOKEN_TYPE { STRAIGHT=0, CURVE_QUADRATIC, MOVE, SET_FILL, SET_STROKE, CLEAR_FILL, CLEAR_STROKE, CURVE_CUBIC, FILL_KEEP_SOURCE, FILL_TRANSFORM_TEXTURE };
 
+union floatVec
+{
+	uint64_t key;
+	struct
+	{
+		float x;
+		float y;
+	} vec;
+};
+
 struct GeomToken
 {
 	union
@@ -72,8 +95,8 @@ struct GeomToken
 		GEOM_TOKEN_TYPE type;
 		struct
 		{
-			int32_t x;
-			int32_t y;
+			float x;
+			float y;
 		} vec;
 		const FILLSTYLE*  fillStyle; // make sure the pointer is valid until rendering is done
 		const LINESTYLE2* lineStyle; // make sure the pointer is valid until rendering is done
@@ -81,15 +104,14 @@ struct GeomToken
 		uint64_t uval;// this is used to have direct access to the value as it is stored in a vector<uint64_t> for performance
 	};
 	GeomToken(GEOM_TOKEN_TYPE t):type(t) {}
-	GeomToken(uint64_t v, bool isvec)
+	GeomToken(floatVec& v)
 	{
-		if (isvec)
-		{
-			vec.x = (int32_t(v&0xffffffff));
-			vec.y = (int32_t(v>>32));
-		}
-		else
-			uval = v;
+		vec.x = v.vec.x;
+		vec.y = v.vec.y;
+	}
+	GeomToken(uint64_t v, bool /*uint64_indicator*/)
+	{
+		uval = v;
 	}
 	GeomToken(const FILLSTYLE& fs):fillStyle(&fs) {}
 	GeomToken(const LINESTYLE2& ls):lineStyle(&ls) {}
@@ -100,40 +122,86 @@ struct GeomToken
 		vec.y=_vec.y;
 	}
 };
+struct fillstylecache
+{
+	FILLSTYLE style;
+	fillstylecache* next;
+	fillstylecache(const FILLSTYLE& fs):style(fs),next(nullptr)
+	{
+	}
+	~fillstylecache()
+	{
+		if (next)
+			delete next;
+	}
+};
+struct linestylecache
+{
+	LINESTYLE2 style;
+	linestylecache* next;
+	linestylecache(const LINESTYLE2& ls):style(ls),next(nullptr)
+	{
+	}
+	~linestylecache()
+	{
+		if (next)
+			delete next;
+	}
+};
+class tokenListRef : public RefCountable
+{
+	fillstylecache* fillStyles;
+	linestylecache* lineStyles;
+public:
+	TokenList tokens;
+	tokenListRef():fillStyles(nullptr),lineStyles(nullptr)
+	{
+	}
+	~tokenListRef();
+	FILLSTYLE& addFillStyle(const FILLSTYLE& fs);
+	LINESTYLE2& addLineStyle(const LINESTYLE2& ls);
+	void clone(tokenListRef* source);
+};
 
 struct tokensVector
 {
-	std::vector<uint64_t> filltokens;
-	std::vector<uint64_t> stroketokens;
+	_NR<tokenListRef> filltokens;
+	_NR<tokenListRef> stroketokens;
+	tokensVector* next;
+	MATRIX startMatrix;
 	RECT boundsRect;
-	uint16_t currentLineWidth;
-	bool canRenderToGL;
-	tokensVector():currentLineWidth(0),canRenderToGL(true) {}
-	void clear()
+	RGBA color;
+	bool isGlyph;
+	bool isFilled; // indicates if this tokensVector was filled with tokens, even if no tokens were generated (e.g. glyph for "space" character)
+	tokensVector():	next(nullptr), boundsRect(INT32_MAX,INT32_MIN,INT32_MAX,INT32_MIN), isGlyph(false), isFilled(false)
 	{
-		filltokens.clear();
-		stroketokens.clear();
-		canRenderToGL=true;
-		currentLineWidth=0;
+	}
+	void clear();
+	void destruct();
+	bool empty() const
+	{
+		return (!filltokens || filltokens->tokens.empty()) && (!stroketokens || stroketokens->tokens.empty()) && (!next || next->empty());
 	}
 	uint32_t size() const
 	{
-		return filltokens.size()+stroketokens.size();
-	}
-	bool empty() const
-	{
-		return filltokens.empty() && stroketokens.empty();
+		return (filltokens ? filltokens->tokens.size() : 0) + (stroketokens ? stroketokens->tokens.size() : 0) + (next ? next->size() : 0);
 	}
 };
 
-
 class ShapePathSegment {
 public:
-	uint64_t from;
-	uint64_t quadctrl;
-	uint64_t to;
+	floatVec from;
+	floatVec quadctrl;
+	floatVec to;
 	int linestyleindex;
-	ShapePathSegment(uint64_t from, uint64_t quadctrl, uint64_t to, int linestyleindex): from(from), quadctrl(quadctrl), to(to),linestyleindex(linestyleindex) {}
+	ShapePathSegment(floatVec from, floatVec quadctrl, floatVec to, int linestyleindex): from(from), quadctrl(quadctrl), to(to),linestyleindex(linestyleindex) {}
+	ShapePathSegment()
+	{
+		from.key=0;
+		quadctrl.key=0;
+		to.key=0;
+		linestyleindex=0;
+	}
 	ShapePathSegment reverse() const {return ShapePathSegment(to, quadctrl, from, linestyleindex);}
 };
 
@@ -145,18 +213,24 @@ private:
 	std::map< unsigned int, std::vector<ShapePathSegment> > strokeShapesMap;
 
 	static bool isOutlineEmpty(const std::vector<ShapePathSegment>& outline);
-	static uint64_t makeVertex(const Vector2& v) { return (uint64_t(v.y)<<32) | (uint64_t(v.x)&0xffffffff); }
+	static floatVec makeVertex(const Vector2f& v) 
+	{ 
+		floatVec ret;
+		ret.vec.x = v.x;
+		ret.vec.y = v.y;
+		return ret;
+	}
 	std::map<uint16_t,LINESTYLE2>::iterator getStrokeLineStyle(const std::list<MORPHLINESTYLE2>::iterator& stylesIt, uint16_t ratio, std::map<uint16_t, LINESTYLE2>* linestylecache, const RECT& boundsrc);
 public:
-	void extendOutline(const Vector2& v1, const Vector2& v2, int linestyleindex);
-	void extendOutlineCurve(const Vector2& start, const Vector2& control, const Vector2& end, int linestyleindex);
+	void extendOutline(const Vector2f& v1, const Vector2f& v2, int linestyleindex);
+	void extendOutlineCurve(const Vector2f& start, const Vector2f& control, const Vector2f& end, int linestyleindex);
 	void endSubpathForStyles(unsigned fill0, unsigned fill1, unsigned stroke, bool formorphing);
 	/**
 		Generate a sequence of cachable tokens that defines the geomtries
 		@param styles This list is supposed to survive until as long as the returned tokens array
 		@param tokens A vector that will be filled with tokens
 	*/
-	void outputTokens(const std::list<FILLSTYLE>& styles, const std::list<LINESTYLE2>& linestyles, tokensVector& tokens);
+	void outputTokens(const std::list<FILLSTYLE>& styles, const std::list<LINESTYLE2>& linestyles, tokensVector& tokens, bool isGlyph);
 	void outputMorphTokens(std::list<MORPHFILLSTYLE>& styles, std::list<MORPHLINESTYLE2>& linestyles, tokensVector& tokens, uint16_t ratio, const RECT& boundsrc);
 	void clear();
 };
