@@ -22,18 +22,24 @@
 
 #include "compat.h"
 #include "backends/geometry.h"
+#include "events.h"
 #include "threading.h"
 #include "platforms/engineutils.h"
 #include "swftypes.h"
 #include "smartrefs.h"
+#include "forwards/scripting/flash/events/flashevents.h"
 #include "scripting/flash/ui/keycodes.h"
+#include "forwards/scripting/flash/events/flashevents.h"
 #include <vector>
+#include <deque>
+#include <set>
+
+#include <SDL2/SDL_keyboard.h>
 
 namespace lightspark
 {
 
 class SystemState;
-class DisplayObject;
 class InteractiveObject;
 class Sprite;
 class MouseEvent;
@@ -44,23 +50,30 @@ friend class Stage;
 private:
 	SystemState* m_sys;
 	EngineData* engineData;
+	SDL_Thread* t;
+	enum STATUS { CREATED=0, STARTED, TERMINATED };
+	volatile STATUS status;
 	bool terminated;
 	// this is called from mainloopthread
-	bool worker(SDL_Event *event);
+	//bool worker(SDL_Event *event);
+	static int worker(void* d);
 
+	std::deque<LSEventStorage> inputEventQueue;
 	std::vector<InteractiveObject* > listeners;
 	Mutex mutexListeners;
 	Mutex mutexDragged;
+	Mutex mutexQueue;
+	Cond eventCond;
 
 	_NR<Sprite> curDragged;
 	_NR<InteractiveObject> currentMouseOver;
 	_NR<InteractiveObject> lastMouseDownTarget;
 	_NR<InteractiveObject> lastMouseUpTarget;
 	_NR<InteractiveObject> lastRolledOver;
-	SDL_Keymod lastKeymod;
-	set<AS3KeyCode> keyDownSet;
-	SDL_Keycode lastKeyDown;
-	SDL_Keycode lastKeyUp;
+	LSModifier lastKeymod;
+	std::set<AS3KeyCode> keyDownSet;
+	AS3KeyCode lastKeyDown;
+	AS3KeyCode lastKeyUp;
 	const RECT* dragLimit;
 	Vector2f dragOffset;
 	class MaskData
@@ -70,20 +83,23 @@ private:
 		MATRIX m;
 		MaskData(DisplayObject* _d, const MATRIX& _m):d(_d),m(_m){}
 	};
-	_NR<InteractiveObject> getMouseTarget(uint32_t x, uint32_t y, DisplayObject::HIT_TYPE type);
-	void handleMouseDown(uint32_t x, uint32_t y, SDL_Keymod buttonState,bool pressed);
-	void handleMouseDoubleClick(uint32_t x, uint32_t y, SDL_Keymod buttonState,bool pressed);
-	void handleMouseUp(uint32_t x, uint32_t y, SDL_Keymod buttonState, bool pressed, uint8_t button);
-	void handleMouseMove(uint32_t x, uint32_t y, SDL_Keymod buttonState,bool pressed);
-	void handleScrollEvent(uint32_t x, uint32_t y, uint32_t direction, SDL_Keymod buttonState,bool pressed);
+	_NR<InteractiveObject> getMouseTarget(const Vector2f& point, HIT_TYPE type);
+	_NR<InteractiveObject> getMouseTarget(uint32_t x, uint32_t y, HIT_TYPE type);
+	void handleMouseDown(const LSMouseButtonEvent& event);
+	void handleMouseDoubleClick(const LSMouseButtonEvent& event);
+	void handleMouseUp(const LSMouseButtonEvent& event);
+	void handleMouseMove(const LSMouseMoveEvent& event);
+	void handleScrollEvent(const LSMouseWheelEvent& event);
 	void handleMouseLeave();
 
-	bool handleKeyboardShortcuts(const SDL_KeyboardEvent *keyevent);
-	void sendKeyEvent(const SDL_KeyboardEvent *keyevent);
+	bool handleKeyboardShortcuts(const LSKeyEvent& event);
+	void sendKeyEvent(const LSKeyEvent& event, bool pressed);
 
-	bool handleContextMenuEvent(SDL_Event* event);
+	int handleEvent(const LSEvent& event);
+	bool handleContextMenuEvent(const LSEvent& event);
 	Mutex inputDataSpinlock;
 	Vector2 mousePos;
+	Vector2 mousePosStart;
 	bool button1pressed;
 public:
 	InputThread(SystemState* s);
@@ -108,18 +124,24 @@ public:
 		}
 		return false;
 	}
-	bool handleEvent(SDL_Event *event)
-	{
-		return worker(event);
-	}
+	
+	bool isCreated() const { return status == CREATED; }
+	bool isStarted() const { return status == STARTED; }
+	bool isTerminated() const { return status == TERMINATED; }
+	bool queueEvent(const LSEvent& event);
+
 	AS3KeyCode getLastKeyDown();
 	AS3KeyCode getLastKeyUp();
-	SDL_Keycode getLastKeyCode();
-	SDL_Keymod getLastKeyMod();
+	AS3KeyCode getLastKeyCode();
+	LSModifier getLastKeyMod();
 	bool isKeyDown(AS3KeyCode key);
 	void setLastKeyDown(KeyboardEvent* e);
 	void setLastKeyUp(KeyboardEvent* e);
 };
 
+InputThread* getInputThread();
+AS3KeyCode getAS3KeyCode(SDL_Keycode sdlkey);
+AS3KeyCode getAS3KeyCodeFromScanCode(SDL_Scancode sdlscancode);
+
 }
 #endif /* BACKENDS_INPUT_H */
