@@ -18,10 +18,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
+#include "version.h"
 #include "backends/security.h"
 #include "scripting/abc.h"
 #include "scripting/class.h"
 #include "scripting/flash/net/flashnet.h"
+#include "scripting/flash/display/RootMovieClip.h"
 #include "swf.h"
 #include "backends/config.h"
 #include "backends/netutils.h"
@@ -608,7 +610,7 @@ void CurlDownloader::execute()
 		//Its probably a good idea to limit redirections, 100 should be more than enough
 		curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 100);
 		// TODO use same useragent as adobe
-		//curl_easy_setopt(curl, CURLOPT_USERAGENT, "Mozilla/5.0");
+		curl_easy_setopt(curl, CURLOPT_USERAGENT, "Lightspark " VERSION);
 		// Empty string means that CURL will decompress if the
 		// server send a compressed file. (This has been
 		// renamed to CURLOPT_ACCEPT_ENCODING in newer CURL,
@@ -757,17 +759,30 @@ void LocalDownloader::execute()
 		FileStreamCache *fileCache = dynamic_cast<FileStreamCache *>(cache.getPtr());
 		if (fileCache)
 		{
-			fileCache->useExistingFile(url);
+			try
+			{
+				fileCache->useExistingFile(url);
+				//Report that we've downloaded everything already
+				length = fileCache->getReceivedLength();
+				notifyOwnerAboutBytesLoaded();
+				notifyOwnerAboutBytesTotal();
+			}
+			catch(exception& e)
+			{
+				LOG(LOG_ERROR, "Exception when using existing file: "<<url<<" "<<e.what());
+				setFailed();
+				return;
+			}
 
-			//Report that we've downloaded everything already
-			length = fileCache->getReceivedLength();
-			notifyOwnerAboutBytesLoaded();
-			notifyOwnerAboutBytesTotal();
 		}
 		//Otherwise we follow the normal procedure
 		else {
+			tiny_string s("file://");
+			s += URLInfo::decode(url, URLInfo::ENCODE_ESCAPE);
+			char* filepath =g_filename_from_uri(s.raw_buf(),nullptr,nullptr);
 			std::ifstream file;
-			file.open(url.raw_buf(), std::ios::in|std::ios::binary);
+			file.open(filepath, std::ios::in|std::ios::binary);
+			free(filepath);
 
 			if(file.is_open())
 			{
@@ -829,7 +844,7 @@ bool DownloaderThreadBase::createDownloader(_R<StreamCache> cache,
 {
 	if(checkPolicyFile)
 	{
-		SecurityManager::EVALUATIONRESULT evaluationResult = \
+		SecurityManager::EVALUATIONRESULT evaluationResult = 
 			dispatcher->getSystemState()->securityManager->evaluatePoliciesURL(url, true);
 		if(threadAborting)
 			return false;
