@@ -25,6 +25,7 @@
 #include "scripting/class.h"
 #include "scripting/toplevel/Error.h"
 #include "scripting/flash/net/XMLSocket.h"
+#include "scripting/flash/display/RootMovieClip.h"
 #include <sstream>
 #include <string>
 #include <algorithm>
@@ -363,6 +364,17 @@ URLPFileList* SecurityManager::searchURLPolicyFiles(const URLInfo& url, bool loa
 						pendingURLPFiles, loadedURLPFiles);
 }
 
+SocketPolicyFile* SecurityManager::getSocketPolicyFile(const URLInfo& url, bool loadPendingPolicies)
+{
+	SocketPolicyFile* policy = getSocketPolicyFileByURL(url);
+
+	if(policy == nullptr)
+		policy = addSocketPolicyFile(url);
+	if(loadPendingPolicies)
+		getSys()->securityManager->loadSocketPolicyFile(policy);
+	return policy;
+}
+
 /**
  * \brief Search for socket policy files relevant to a given URL
  *
@@ -377,18 +389,23 @@ URLPFileList* SecurityManager::searchURLPolicyFiles(const URLInfo& url, bool loa
  */
 SocketPFileList* SecurityManager::searchSocketPolicyFiles(const URLInfo& url, bool loadPendingPolicies)
 {
+	SocketPFileList* result;
+
 	//Get or create the master policy file object
-	URLInfo masterURL = url.goToURL(SocketPolicyFile::MASTER_PORT_URL);
-	SocketPolicyFile* master = getSocketPolicyFileByURL(masterURL);
+	SocketPolicyFile* master = getSocketPolicyFile(url.goToURL(SocketPolicyFile::MASTER_PORT_URL), loadPendingPolicies);
 
-	if(master == nullptr)
-		master = addSocketPolicyFile(masterURL);
-
-	if(loadPendingPolicies)
-		getSys()->securityManager->loadSocketPolicyFile(master);
+	bool useFallback = master->isLoaded() && !master->isValid();
+	//Fallback to the target port, if the master port failed.
+	if (useFallback)
+	{
+		LOG(LOG_INFO, "SECURITY: Master port failed, falling back to target port.");
+		//master = getSocketPolicyFile(url);
+		result = new SocketPFileList;
+		result->push_back(getSocketPolicyFile(url, loadPendingPolicies));
+		return result;
+	}
 
 	//Get applicable policy files
-	SocketPFileList *result;
 	result = searchPolicyFiles<SocketPolicyFile>(url, master, loadPendingPolicies,
 						     pendingSocketPFiles, loadedSocketPFiles);
 
@@ -531,7 +548,10 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateLocalDirectoryURL(con
 {
 	//The URL is local and points to a directory above the origin
 	if(url.getProtocol() == "file" && !url.isSubOf(getSys()->mainClip->getOrigin()))
+	{
+		LOG(LOG_ERROR,"evaluateLocalDirectoryURL failed:"<<url<<" "<<getSys()->mainClip->getOrigin());
 		return NA_RESTRICT_LOCAL_DIRECTORY;
+	}
 
 	return ALLOWED;
 }
@@ -1307,6 +1327,7 @@ bool SocketPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 	ssize_t nbytes = sock.sendAll(socket_policy_cmd, socket_policy_cmd_len);
 	if (nbytes != (int)socket_policy_cmd_len)
 	{
+		LOG(LOG_ERROR, "SECURITY: policy command size isn't the same as the total sent size.");
 		return false;
 	}
 
@@ -1322,6 +1343,7 @@ bool SocketPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 	if (nbytes < 0 && outData.size() == 0)
 	{
 		// error reading from socket
+		LOG(LOG_ERROR, "SECURITY: Failed to read from socket.");
 		return false;
 	}
 
@@ -1329,6 +1351,7 @@ bool SocketPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 	// is not '\0' or '>' or '\n'
 	if (outData.size() == 0 || (outData[outData.size()-1] != '\0' && outData[outData.size()-1] != '>' && outData[outData.size()-1] != '\n'))
 	{
+		LOG(LOG_ERROR, "SECURITY: Last character of policy file is invalid.");
 		return false;
 	}
 
