@@ -28,10 +28,9 @@ using namespace std;
 
 const pugi::xml_node XMLBase::buildFromString(const tiny_string& str,
 										unsigned int xmlparsemode,
-										const tiny_string& default_ns,
 										pugi::xml_parse_result* parseresult)
 {
-	tiny_string buf = quirkEncodeNull(removeWhitespace(str));
+	tiny_string buf = str.removeWhitespace().encodeNull();
 	if (buf.numBytes() > 0 && buf.charAt(0) == '<')
 	{
 		pugi::xml_parse_result res = xmldoc.load_buffer((void*)buf.raw_buf(),buf.numBytes(),xmlparsemode);
@@ -45,6 +44,9 @@ const pugi::xml_node XMLBase::buildFromString(const tiny_string& str,
 		{
 			case pugi::status_ok:
 				break;
+			case pugi::status_bad_start_element:
+				createError<TypeError>(getWorker(),kXMLBadQName);
+				break;
 			case pugi::status_end_element_mismatch:
 				createError<TypeError>(getWorker(),kXMLUnterminatedElementTag);
 				break;
@@ -52,6 +54,9 @@ const pugi::xml_node XMLBase::buildFromString(const tiny_string& str,
 				createError<TypeError>(getWorker(),kXMLMalformedElement);
 				break;
 			case pugi::status_bad_pi:
+				createError<TypeError>(getWorker(),kXMLUnterminatedProcessingInstruction);
+				break;
+			case pugi::status_bad_declaration:
 				createError<TypeError>(getWorker(),kXMLUnterminatedXMLDecl);
 				break;
 			case pugi::status_bad_attribute:
@@ -66,6 +71,9 @@ const pugi::xml_node XMLBase::buildFromString(const tiny_string& str,
 			case pugi::status_bad_comment:
 				createError<TypeError>(getWorker(),kXMLUnterminatedComment);
 				break;
+			case pugi::status_bad_end_element:
+				createError<TypeError>(getWorker(),kXMLMalformedElement);
+				break;
 			default:
 				LOG(LOG_ERROR,"xml parser error:"<<buf<<" "<<res.status<<" "<<res.description());
 				break;
@@ -116,78 +124,3 @@ const tiny_string XMLBase::encodeToXML(const tiny_string value, bool bIsAttribut
 	}
 	return res;
 }
-
-// Adobe player's XML parser accepts many strings which are not valid
-// XML according to the specs. This function attempts to massage
-// invalid-but-accepted-by-Adobe strings into valid XML so that
-// libxml++ parser doesn't throw an error.
-string XMLBase::parserQuirks(const string& str)
-{
-	string buf = quirkEncodeNull(str);
-	buf = quirkXMLDeclarationInMiddle(buf);
-	return buf;
-}
-
-string XMLBase::quirkXMLDeclarationInMiddle(const string& str) {
-	string buf(str);
-
-	// Adobe player ignores XML declarations in the middle of a
-	// string.
-	while (true)
-	{
-		size_t start = buf.find("<?xml ", 1);
-		if (start == buf.npos)
-			break;
-		
-		size_t end = buf.find("?>", start+5);
-		if (end == buf.npos)
-			break;
-		end += 2;
-		
-		buf.erase(start, end-start);
-	}
-
-	return buf;
-}
-string XMLBase::quirkEncodeNull(const string value)
-{
-	string res;
-	auto it = value.cbegin();
-	while (it != value.cend())
-	{
-		switch (*it)
-		{
-			case '\0':
-				res += "&#x0;";
-				break;
-			default:
-				res += *it;
-				break;
-		}
-		it++;
-	}
-	return res;
-}
-
-tiny_string XMLBase::removeWhitespace(tiny_string val)
-{
-	bool bwhite = true;
-	uint32_t start = 0;
-	CharIterator it = val.begin();
-	CharIterator itend = val.begin();
-	while (it != val.end())
-	{
-		if (!g_unichar_isspace(*it))
-		{
-			itend=it;
-			itend++;
-			bwhite = false;
-		}
-		else if (bwhite)
-			start++;
-		it++;
-	}
-	if (bwhite)
-		return "";
-	return val.substr(start,itend);
-}
