@@ -21,6 +21,7 @@
 #define LOGGER_H 1
 
 #include "compat.h"
+#include "interfaces/logger.h"
 #include <iostream>
 #include <sstream>
 #include <vector>
@@ -28,6 +29,21 @@
 
 enum LOG_LEVEL { LOG_ERROR=0, LOG_INFO=1, LOG_NOT_IMPLEMENTED=2,LOG_CALLS=3,LOG_TRACE=4};
 
+// LOG_CALL will only generate output in DEBUG builds
+// this is done because the LOG(LOG_CALLS...) macro creates measurable perfomance loss 
+// when used inside the ABCVm::executeFunction loop even on lower log levels
+#ifndef NDEBUG
+#define LOG_CALL(esp)					\
+do {							\
+	if(LOG_CALLS<=Log::getLevel())			\
+	{						\
+		Log l(LOG_CALLS);			\
+		l() << esp << std::endl;		\
+	}						\
+} while(0)
+#else
+#define LOG_CALL(esp)
+#endif
 #define LOG(level,esp)					\
 do {							\
 	if(level<=Log::getLevel())			\
@@ -46,19 +62,38 @@ private:
 	LOG_LEVEL cur_level;
 	bool valid;
 public:
-	static void print(const std::string& s);
 	Log(LOG_LEVEL l) DLL_PUBLIC;
 	~Log() DLL_PUBLIC;
 	std::ostream& operator()() DLL_PUBLIC;
 	operator bool() { return valid; }
-	static void setLogLevel(LOG_LEVEL l) { log_level = l; };
+	static void setLogLevel(LOG_LEVEL l) { log_level = l; }
 	static LOG_LEVEL getLevel() {return log_level;}
 	static int calls_indent;
 	/* redirect logging and print() to that file */
 	static void redirect(std::string filename) DLL_PUBLIC;
+	static void setLogStream(std::ostream* stream) DLL_PUBLIC;
 
 };
 
+namespace lightspark
+{
+
+// Generic logger.
+class DLL_PUBLIC Logger : public ILogger
+{
+private:
+	std::ostream* outStream;
+public:
+	Logger() : ILogger(), outStream(&std::cout) {}
+
+	// Traces/Prints the supplied string.
+	void trace(const tiny_string& str) override;
+
+	void setStream(std::ostream* stream);
+};
+
+}
+
 template<typename T>
 std::ostream& printContainer(std::ostream& os, const T& v)
 {
