@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -22,12 +22,19 @@
 
 #include "compat.h"
 #include "tiny_string.h"
+#ifdef __APPLE__
+#include <stdlib.h>
+#else
 #include <malloc.h>
+#endif
 
 namespace lightspark
 {
 
 #ifdef MEMORY_USAGE_PROFILING
+class MemoryAccount;
+DLL_PUBLIC MemoryAccount* getUnaccountedMemoryAccount();
+
 class MemoryAccount
 {
 public:
@@ -73,6 +80,8 @@ public:
 		//Adding the data to the object itself would not work
 		//since it can be reset by the constructors
 		objData* ret=reinterpret_cast<objData*>(malloc(size+sizeof(objData)));
+		if(!m)
+			m = getUnaccountedMemoryAccount();
 		m->addBytes(size);
 		ret->objSize = size;
 		ret->memoryAccount = m;
@@ -87,8 +96,6 @@ public:
 	}
 };
 
-DLL_PUBLIC MemoryAccount* getUnaccountedMemoryAccount();
-
 template<class T>
 class reporter_allocator: protected std::allocator<T>
 {
