@@ -24,7 +24,6 @@
 #include <istream>
 #include <map>
 #include "swftypes.h"
-#include "backends/decoder.h"
 
 namespace lightspark
 {
@@ -65,6 +64,7 @@ public:
 	tiny_string methodName;
 	std::list<asAtom> dataobjectlist;
 	ScriptDataTag() {}
+	~ScriptDataTag();
 	ScriptDataTag(std::istream& s);
 };
 
