@@ -20,19 +20,23 @@
 
 #include "platforms/engineutils.h"
 #include "swf.h"
-#include <SDL2/SDL.h>
-#include <SDL2/SDL_mouse.h>
-#include "backends/input.h"
-#include "backends/rendering.h"
-#include "backends/lsopengl.h"
 #include "backends/audio.h"
+#include "backends/decoder.h"
+#include "backends/input.h"
+#include "backends/lsopengl.h"
+#include "backends/rendering.h"
+#include "backends/sdl/event_loop.h"
 #include <pango/pangocairo.h>
 #include "version.h"
 #include "abc.h"
 #include "class.h"
 #include "scripting/flash/events/flashevents.h"
+#include "scripting/flash/display/Stage.h"
+#include "scripting/flash/geom/Rectangle.h"
 #include "flash/display/NativeMenuItem.h"
 #include "flash/utils/ByteArray.h"
+#include "flash/geom/flashgeom.h"
+#include "launcher.h"
 #include <glib/gstdio.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -60,13 +64,32 @@ using namespace lightspark;
 uint32_t EngineData::userevent = (uint32_t)-1;
 SDL_Thread* EngineData::mainLoopThread = nullptr;
 bool EngineData::mainthread_running = false;
-bool EngineData::sdl_needinit = true;
+bool EngineData::needinit = true;
 bool EngineData::enablerendering = true;
 SDL_Cursor* EngineData::handCursor = nullptr;
+SDL_Cursor* EngineData::arrowCursor = nullptr;
+SDL_Cursor* EngineData::ibeamCursor = nullptr;
 Semaphore EngineData::mainthread_initialized(0);
-EngineData::EngineData() : contextmenu(nullptr),contextmenurenderer(nullptr),sdleventtickjob(nullptr),incontextmenu(false),incontextmenupreparing(false),widget(nullptr),nvgcontext(nullptr), width(0), height(0),needrenderthread(true),supportPackedDepthStencil(false),hasExternalFontRenderer(false),
+
+bool lightspark::isEventLoopThread()
+{
+	auto thread = EngineData::mainLoopThread;
+	if (thread != nullptr)
+		return SDL_GetThreadID(thread) == SDL_ThreadID();
+	else
+		return isMainThread();
+}
+
+EngineData::EngineData() : contextmenu(nullptr),contextmenurenderer(nullptr),sdleventtickjob(nullptr),incontextmenu(false),incontextmenupreparing(false),widget(nullptr),
+	nvgcontext(nullptr),
+	width(0), height(0),needrenderthread(true),supportPackedDepthStencil(false),hasExternalFontRenderer(false),
 	startInFullScreenMode(false),startscalefactor(1.0)
 {
+#ifdef _WIN32
+	platformOS="Windows";
+#else
+	platformOS="Linux";
+#endif
 }
 
 EngineData::~EngineData()
@@ -81,100 +104,168 @@ EngineData::~EngineData()
 	if (nvgcontext)
 		nvgDeleteGL2(nvgcontext);
 #endif
+	handleQuit();
 }
-bool EngineData::mainloop_handleevent(SDL_Event* event,SystemState* sys)
+
+void EngineData::runInTrueMainThread(SystemState* sys, MainThreadCallback func)
 {
-	if (sys && sys->getEngineData())
-		sys->getEngineData()->renderContextMenu();
-	if (event->type == LS_USEREVENT_INIT)
-	{
-		sys = (SystemState*)event->user.data1;
-		setTLSSys(sys);
-		setTLSWorker(sys->worker);
-	}
-	else if (event->type == LS_USEREVENT_EXEC)
-	{
-		if (event->user.data1)
-			((void (*)(SystemState*))event->user.data1)(sys);
-	}
-	else if (event->type == LS_USEREVENT_QUIT)
-	{
-		setTLSSys(nullptr);
-		SDL_Quit();
-		return true;
-	}
-	else if (event->type == LS_USEREVENT_OPEN_CONTEXTMENU)
-	{
-		if (sys && sys->getEngineData())
-			sys->getEngineData()->openContextMenuIntern((InteractiveObject*)event->user.data1);
-	}
-	else if (event->type == LS_USEREVENT_UPDATE_CONTEXTMENU)
-	{
-		if (sys && sys->getEngineData())
-		{
-			int pos = *(int*)event->user.data1;
-			delete (int*)event->user.data1;
-			sys->getEngineData()->updateContextMenu(pos);
-		}
-	}
-	else if (event->type == LS_USEREVENT_SELECTITEM_CONTEXTMENU)
-	{
-		if (sys && sys->getEngineData())
-		{
-			sys->getEngineData()->selectContextMenuItemIntern();
-		}
-	}
+	runInMainThread(sys, func);
+}
+
+void EngineData::runInMainThread(SystemState* sys, MainThreadCallback func)
+{
+	if (!isEventLoopThread())
+		getSys()->pushEvent(LSExecEvent(func));
 	else
-	{
-		if (sys && sys->getInputThread() && sys->getInputThread()->handleEvent(event))
-			return false;
-		switch (event->type)
+		func(sys);
+}
+
+void EngineData::handleQuit()
+{
+	if (!SDL_WasInit(0))
+		SDL_Quit ();
+}
+
+bool EngineData::mainloop_handleevent(const LSEvent& event, SystemState* sys)
+{
+	using FocusType = LSWindowFocusEvent::FocusType;
+
+	if (event.isInvalid())
+		return false;
+	bool hasSys = sys != nullptr;
+	bool hasEngineData = hasSys && sys->getEngineData() != nullptr;
+	if (hasEngineData)
+		sys->getEngineData()->renderContextMenu();
+	bool isMiscType = true;
+	bool quit = event.visit(makeVisitor
+	(
+		[&](const LSInitEvent& init)
 		{
-			case SDL_WINDOWEVENT:
+			sys = init.sys;
+			setTLSSys(sys);
+			setTLSWorker(sys->worker);
+			return false;
+		},
+		[&](const LSExecEvent& exec)
+		{
+			if (exec.callback)
+				exec.callback(sys);
+			return false;
+		},
+		[&](const LSOpenContextMenuEvent& open)
+		{
+			if (hasEngineData)
+				sys->getEngineData()->openContextMenuIntern(open.obj);
+			return false;
+		},
+		[&](const LSUpdateContextMenuEvent& update)
+		{
+			if (hasEngineData)
+				sys->getEngineData()->updateContextMenu(update.selectedItem);
+			return false;
+		},
+		[&](const LSSelectItemContextMenuEvent&)
+		{
+			if (hasEngineData)
+				sys->getEngineData()->selectContextMenuItemIntern();
+			return false;
+		},
+		[&](const LSQuitEvent& quit)
+		{
+			if (quit.quitType == LSQuitEvent::QuitType::User)
 			{
-				switch (event->window.event)
-				{
-					case SDL_WINDOWEVENT_RESIZED:
-					case SDL_WINDOWEVENT_SIZE_CHANGED:
-						if (sys && (!sys->getEngineData() || !sys->getEngineData()->inFullScreenMode()) &&  sys->getRenderThread())
-							sys->getRenderThread()->requestResize(event->window.data1,event->window.data2,false);
-						break;
-					case SDL_WINDOWEVENT_EXPOSED:
-					{
-						//Signal the renderThread
-						if (sys && sys->getRenderThread())
-						{
-							sys->getRenderThread()->draw(true);
-							// it seems that sometimes the systemstate thread stops ticking when in background, this ensures it starts ticking again...
-							sys->sendMainSignal();
-						}
-						break;
-					}
-					case SDL_WINDOWEVENT_FOCUS_LOST:
-						sys->getEngineData()->closeContextMenu();
-						break;
-					case SDL_WINDOWEVENT_ENTER:
-						sys->addBroadcastEvent("activate");
-						break;
-					case SDL_WINDOWEVENT_LEAVE:
-						sys->addBroadcastEvent("deactivate");
-						break;
-					default:
-						break;
-				}
-				break;
+				assert(hasEngineData);
+				setTLSSys(nullptr);
+				sys->getEngineData()->handleQuit();
+				return true;
 			}
-			case SDL_QUIT:
-				sys->setShutdownFlag();
-				break;
-		}
-	}
+			isMiscType = false;
+			return false;
+		},
+		[&](const LSEvent&) { isMiscType = false; return false; }
+	));
+
+	if (quit || isMiscType)
+		return quit;
+	if (hasSys && sys->getInputThread() != nullptr && sys->getInputThread()->queueEvent(event))
+		return false;
+
+	event.visit(makeVisitor
+	(
+		[&](const LSWindowResizedEvent& resize)
+		{
+			if (hasSys && (!hasEngineData || !sys->getEngineData()->inFullScreenMode()) && sys->getRenderThread() != nullptr)
+				sys->getRenderThread()->requestResize(resize.size,false);
+		},
+		[&](const LSWindowMovedEvent& move)
+		{
+			// NOTE: `TODO` is specific to SDL.
+			// TODO there doesn't seem to be a way to detect the starting of window movement in SDL, so for now we emit "moving" and "moved" events for every SDL_WINDOWEVENT_MOVED
+			sys->getEngineData()->x = move.pos.x;
+			sys->getEngineData()->y = move.pos.y;
+			if (hasSys && getVm(sys) != nullptr && !sys->stage->nativeWindow.isNull())
+			{
+				Rectangle *rectBefore=Class<Rectangle>::getInstanceS(sys->worker);
+				rectBefore->x = sys->getEngineData()->old_x;
+				rectBefore->y = sys->getEngineData()->old_y;
+				rectBefore->width = sys->getEngineData()->width;
+				rectBefore->height = sys->getEngineData()->height;
+				Rectangle *rectAfter=Class<Rectangle>::getInstanceS(sys->worker);
+				rectAfter->x = sys->getEngineData()->x;
+				rectAfter->y = sys->getEngineData()->y;
+				rectAfter->width = sys->getEngineData()->width;
+				rectAfter->height = sys->getEngineData()->height;
+				getVm(sys)->addEvent(_MR(sys->stage->nativeWindow),_MR(Class<NativeWindowBoundsEvent>::getInstanceS(sys->worker,"moving",_MR(rectBefore),_MR(rectAfter))));
+			}
+			if (hasSys && getVm(sys) != nullptr && !sys->stage->nativeWindow.isNull())
+			{
+				Rectangle *rectBefore=Class<Rectangle>::getInstanceS(sys->worker);
+				rectBefore->x = sys->getEngineData()->old_x;
+				rectBefore->y = sys->getEngineData()->old_y;
+				rectBefore->width = sys->getEngineData()->width;
+				rectBefore->height = sys->getEngineData()->height;
+				Rectangle *rectAfter=Class<Rectangle>::getInstanceS(sys->worker);
+				rectAfter->x = sys->getEngineData()->x;
+				rectAfter->y = sys->getEngineData()->y;
+				rectAfter->width = sys->getEngineData()->width;
+				rectAfter->height = sys->getEngineData()->height;
+				getVm(sys)->addEvent(_MR(sys->stage->nativeWindow),_MR(Class<NativeWindowBoundsEvent>::getInstanceS(sys->worker,"move",_MR(rectBefore),_MR(rectAfter))));
+			}
+			sys->getEngineData()->old_x = move.pos.x;
+			sys->getEngineData()->old_y = move.pos.y;
+		},
+		[&](const LSWindowExposedEvent&)
+		{
+			//Signal the renderThread
+			if (hasSys && sys->getRenderThread() != nullptr)
+			{
+				sys->getRenderThread()->draw(true);
+				// it seems that sometimes the systemstate thread stops ticking when in background, this ensures it starts ticking again...
+				sys->sendMainSignal();
+			}
+		},
+		[&](const LSWindowFocusEvent& focus)
+		{
+			if (focus.focusType == FocusType::Keyboard)
+			{
+				if (focus.focused)
+					sys->addBroadcastEvent("activate");
+				else
+				{
+					sys->getEngineData()->closeContextMenu();
+					sys->addBroadcastEvent("deactivate");
+				}
+			}
+		},
+		[&](const LSQuitEvent&) { sys->setShutdownFlag(); },
+		[](const LSEvent&) {}
+	));
 	return false;
 }
 bool initSDL()
 {
-	bool sdl_available = !EngineData::sdl_needinit;
-	if (EngineData::sdl_needinit)
+	bool sdl_available = !EngineData::needinit;
+	if (EngineData::needinit)
 	{
 		if (!EngineData::enablerendering)
 		{
@@ -190,6 +281,7 @@ bool initSDL()
 			else
 				sdl_available = !SDL_Init ( SDL_INIT_VIDEO );
 			SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8); // needed for nanovg
+			SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24 );
 			SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);// needed for nanovg
 			SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, 4);// needed for nanovg
 #ifdef ENABLE_GLES2
@@ -202,8 +294,9 @@ bool initSDL()
 	return sdl_available;
 }
 /* main loop handling */
-static int mainloop_runner(void*)
+static int mainloop_runner(void* d)
 {
+	IEventLoop* th = (IEventLoop*)d;
 	if (!initSDL())
 	{
 		LOG(LOG_ERROR,"Unable to initialize SDL:"<<SDL_GetError());
@@ -214,13 +307,14 @@ static int mainloop_runner(void*)
 	{
 		EngineData::mainthread_running = true;
 		EngineData::mainthread_initialized.signal();
-		SDL_Event event;
-		while (SDL_WaitEvent(&event))
+		Optional<LSEventStorage> event;
+		while (event = th->waitEvent(getSys()), event.hasValue())
 		{
 			SystemState* sys = getSys();
-			
-			if (EngineData::mainloop_handleevent(&event,sys))
+
+			if (EngineData::mainloop_handleevent(*event, sys))
 			{
+				delete th;
 				EngineData::mainthread_running = false;
 				return 0;
 			}
@@ -228,16 +322,25 @@ static int mainloop_runner(void*)
 	}
 	return 0;
 }
-void EngineData::mainloop_from_plugin(SystemState* sys)
+void EngineData::mainloop_from_plugin(SystemState* sys, IEventLoop* eventLoop)
 {
 	initSDL();
-	EngineData::sdl_needinit = false;
+	EngineData::needinit = false;
+	Optional<LSEventStorage> event;
+	setTLSSys(sys);
+	while (event = eventLoop->waitEvent(sys), event.hasValue())
+		mainloop_handleevent(*event, sys);
+	setTLSSys(nullptr);
+}
+
+void EngineData::sdl_mainloop_from_plugin(SystemState* sys)
+{
+	initSDL();
+	EngineData::needinit = false;
 	SDL_Event event;
 	setTLSSys(sys);
 	while (SDL_PollEvent(&event))
-	{
-		mainloop_handleevent(&event,sys);
-	}
+		mainloop_handleevent(SDLEventLoop::toLSEvent(sys, event),sys);
 	setTLSSys(nullptr);
 }
 
@@ -249,7 +352,7 @@ public:
 	SDLEventTicker(EngineData* engine,SystemState* sys):m_engine(engine),m_sys(sys) {}
 	void tick() override
 	{
-		m_engine->runInMainThread(m_sys,EngineData::mainloop_from_plugin);
+		m_engine->runInMainThread(m_sys,EngineData::sdl_mainloop_from_plugin);
 		if (!m_engine->inFullScreenMode() && !m_engine->inContextMenu() && !m_engine->inContextMenuPreparing() )
 			stopMe=true;
 	}
@@ -272,10 +375,10 @@ void EngineData::startSDLEventTicker(SystemState* sys)
 /* This is not run in the linux plugin, as firefox
  * runs its own gtk_main, which we must not interfere with.
  */
-bool EngineData::startSDLMain()
+bool EngineData::startSDLMain(EventLoop* eventLoop)
 {
 	assert(!mainLoopThread);
-	mainLoopThread = SDL_CreateThread(mainloop_runner,"mainloop",nullptr);
+	mainLoopThread = SDL_CreateThread(mainloop_runner,"mainloop",eventLoop);
 	mainthread_initialized.wait();
 	return mainthread_running;
 }
@@ -285,6 +388,16 @@ bool EngineData::FileExists(SystemState* sys, const tiny_string &filename, bool
 	LOG(LOG_ERROR,"FileExists not implemented");
 	return false;
 }
+bool EngineData::FileIsHidden(SystemState* sys, const tiny_string &filename, bool isfullpath)
+{
+	LOG(LOG_ERROR,"FileIsHidden not implemented");
+	return false;
+}
+bool EngineData::FileIsDirectory(SystemState* sys, const tiny_string &filename, bool isfullpath)
+{
+	LOG(LOG_ERROR,"FileIsDirectory not implemented");
+	return false;
+}
 
 uint32_t EngineData::FileSize(SystemState* sys, const tiny_string& filename, bool isfullpath)
 {
@@ -298,6 +411,12 @@ tiny_string EngineData::FileFullPath(SystemState* sys, const tiny_string& filena
 	return "";
 }
 
+tiny_string EngineData::FileBasename(SystemState* sys, const tiny_string& filename, bool isfullpath)
+{
+	LOG(LOG_ERROR,"FileBasename not implemented");
+	return "";
+}
+
 tiny_string EngineData::FileRead(SystemState* sys,const tiny_string &filename, bool isfullpath)
 {
 	LOG(LOG_ERROR,"FileRead not implemented");
@@ -330,6 +449,11 @@ bool EngineData::FileCreateDirectory(SystemState* sys, const tiny_string &filena
 	LOG(LOG_ERROR,"FileCreateDirectory not implemented");
 	return false;
 }
+bool EngineData::FilGetDirectoryListing(SystemState* sys, const tiny_string &filename, bool isfullpath, std::vector<tiny_string>& filelist)
+{
+	LOG(LOG_ERROR,"FilGetDirectoryListing not implemented");
+	return false;
+}
 
 bool EngineData::FilePathIsAbsolute(const tiny_string& filename)
 {
@@ -346,7 +470,11 @@ tiny_string EngineData::FileGetApplicationStorageDir()
 void EngineData::initGLEW()
 {
 //For now GLEW does not work with GLES2
-#ifndef ENABLE_GLES2
+#ifdef ENABLE_GLES2
+#ifdef GL_DEPTH24_STENCIL8_OES
+	supportPackedDepthStencil=true;
+#endif
+#else
 	//Now we can initialize GLEW
 	GLenum err = glewInit();
 	if (GLEW_OK != err)
@@ -386,6 +514,37 @@ void EngineData::initNanoVG()
 	if (nvgcontext == nullptr)
 		LOG(LOG_ERROR,"couldn't initialize nanovg");
 }
+SDL_Window* EngineData::createMainSDLWidget(uint32_t w, uint32_t h)
+{
+	SDL_Window* window = SDL_CreateWindow("Lightspark",
+										  SDL_WINDOWPOS_UNDEFINED,
+										  SDL_WINDOWPOS_UNDEFINED,
+										  w,
+										  h,
+										  SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE);
+	if (window == 0) {
+		// try creation of window without opengl support
+		window = SDL_CreateWindow("Lightspark",
+								  SDL_WINDOWPOS_UNDEFINED,
+								  SDL_WINDOWPOS_UNDEFINED,
+								  w,
+								  h,
+								  SDL_WINDOW_RESIZABLE);
+		if (window == 0)
+			LOG(LOG_ERROR, "createWidget failed:" << SDL_GetError());
+	}
+	return window;
+}
+
+SDL_GLContext EngineData::createSDLGLContext(SDL_Window* widget)
+{
+	return SDL_GL_CreateContext(widget);
+}
+
+void EngineData::deleteSDLGLContext(SDL_GLContext ctx)
+{
+	SDL_GL_DeleteContext(ctx);
+}
 
 void EngineData::showWindow(uint32_t w, uint32_t h)
 {
@@ -398,7 +557,15 @@ void EngineData::showWindow(uint32_t w, uint32_t h)
 		this->width = w;
 	if (this->height == 0)
 		this->height = h;
+	x=0;
+	y=0;
 	widget = createWidget(this->width,this->height);
+	if (widget)
+		SDL_GetWindowPosition(widget,&x,&y);
+	old_x =x;
+	old_y =y;
+	old_width = this->width;
+	old_height = this->height;
 	// plugins create a hidden window that should not be shown
 	if (widget && !(SDL_GetWindowFlags(widget) & SDL_WINDOW_HIDDEN))
 		SDL_ShowWindow(widget);
@@ -406,6 +573,94 @@ void EngineData::showWindow(uint32_t w, uint32_t h)
 	
 }
 
+void EngineData::checkForNativeAIRExtensions(std::vector<tiny_string>& extensions, char* fileName)
+{
+	tiny_string p = g_path_get_dirname(fileName);
+	p += G_DIR_SEPARATOR_S;
+	p += "META-INF";
+	p += G_DIR_SEPARATOR_S;
+	p += "AIR";
+	p += G_DIR_SEPARATOR_S;
+	p += "extensions";
+	GDir* dir = g_dir_open(p.raw_buf(),0,nullptr);
+	if (dir)
+	{
+		while (true)
+		{
+			const char* subpath = g_dir_read_name(dir);
+			if (!subpath)
+				break;
+			tiny_string extensionpath=p;
+			extensionpath += G_DIR_SEPARATOR_S;
+			extensionpath += subpath;
+			extensionpath += G_DIR_SEPARATOR_S;
+			extensionpath += "library.swf";
+			if (g_file_test(p.raw_buf(),G_FILE_TEST_EXISTS))
+			{
+				LOG(LOG_INFO,"native extension found:"<<extensionpath);
+				extensions.push_back(extensionpath);
+			}
+		}
+		g_dir_close(dir);
+	}
+	if (!extensions.empty())
+	{
+		// try to load additional libraries that may be needed for the extensions
+		tiny_string basedir = g_path_get_dirname(fileName);
+		GDir* dir = g_dir_open(basedir.raw_buf(),0,nullptr);
+		if (dir)
+		{
+			while (true)
+			{
+				const char* file = g_dir_read_name(dir);
+				if (!file)
+					break;
+				if (g_file_test(file,G_FILE_TEST_IS_DIR))
+					continue;
+				tiny_string s=file;
+#ifdef _WIN32
+				const char* suffix = ".dll";
+#else
+				const char* suffix = ".so";
+#endif
+				if (!s.endsWith(suffix))
+					continue;
+				tiny_string libpath=basedir;
+				libpath += G_DIR_SEPARATOR_S;
+				libpath += s;
+				void* lib = SDL_LoadObject(libpath.raw_buf());
+				if (lib==nullptr)
+					LOG(LOG_ERROR,"loading additional lib failed:"<<SDL_GetError()<<" "<<libpath);
+				else
+					LOG(LOG_INFO,"additional lib loaded:"<<lib<<" "<<libpath);
+			}
+			g_dir_close(dir);
+		}
+	}
+}
+
+void EngineData::addQuitEvent()
+{
+	if (getSys()->runSingleThreaded || !isEventLoopThread())
+	{
+		getSys()->pushEvent(LSQuitEvent(LSQuitEvent::QuitType::User));
+		SDL_WaitThread(EngineData::mainLoopThread,nullptr);
+	}
+	else
+	{
+		setTLSSys(nullptr);
+		handleQuit();
+	}
+}
+
+void EngineData::notifyTimer()
+{
+	SDL_Event event;
+	SDL_zero(event);
+	event.type = LS_USEREVENT_NEW_TIMER;
+	SDL_PushEvent(&event);
+}
+
 std::string EngineData::getsharedobjectfilename(const tiny_string& name)
 {
 	tiny_string subdir = sharedObjectDatapath + G_DIR_SEPARATOR_S;
@@ -564,27 +819,18 @@ void EngineData::updateContextMenuFromMouse(uint32_t windowID, int mousey)
 			}
 		}
 	}
-	SDL_Event event;
-	SDL_zero(event);
-	event.type = LS_USEREVENT_UPDATE_CONTEXTMENU;
-	event.user.data1 = (void*)new int(newselecteditem);
-	SDL_PushEvent(&event);
+	getSys()->pushEvent(LSUpdateContextMenuEvent(newselecteditem));
 }
 
 void EngineData::selectContextMenuItem()
 {
-	SDL_Event event;
-	SDL_zero(event);
-	event.type = LS_USEREVENT_SELECTITEM_CONTEXTMENU;
-	SDL_PushEvent(&event);
+	getSys()->pushEvent(LSSelectItemContextMenuEvent());
 }
 void EngineData::InteractiveObjectRemovedFromStage()
 {
-	SDL_Event event;
-	SDL_zero(event);
-	event.type = LS_USEREVENT_INTERACTIVEOBJECT_REMOVED_FOM_STAGE;
-	SDL_PushEvent(&event);
+	getSys()->pushEvent(LSRemovedFromStageEvent());
 }
+
 void EngineData::selectContextMenuItemIntern()
 {
 	if (contextmenucurrentitem >=0)
@@ -632,6 +878,14 @@ void EngineData::selectContextMenuItemIntern()
 	}
 	closeContextMenu();
 }
+
+SDL_Window* EngineData::createWidget(uint32_t w, uint32_t h)
+{
+	// Setup SDL
+	SDL_Window* window = createMainSDLWidget(w, h);
+	Launcher::setWindowIcon(window);
+	return window;
+}
 void EngineData::updateContextMenu(int newselecteditem)
 {
 	float bordercolor = 0.3;
@@ -742,7 +996,45 @@ void EngineData::resetMouseHandCursor(SystemState* /*sys*/)
 {
 	SDL_SetCursor(SDL_GetDefaultCursor());
 }
-
+void EngineData::setMouseCursor(SystemState* /*sys*/, const tiny_string& name)
+{
+	if (name=="hand")
+	{
+		if (!handCursor)
+			handCursor = SDL_CreateSystemCursor(SDL_SystemCursor::SDL_SYSTEM_CURSOR_HAND);
+		SDL_SetCursor(handCursor);
+	}
+	else if (name=="arrow")
+	{
+		if (!arrowCursor)
+			arrowCursor = SDL_CreateSystemCursor(SDL_SystemCursor::SDL_SYSTEM_CURSOR_ARROW);
+		SDL_SetCursor(arrowCursor);
+	}
+	else if (name=="ibeam")
+	{
+		if (!ibeamCursor)
+			ibeamCursor = SDL_CreateSystemCursor(SDL_SystemCursor::SDL_SYSTEM_CURSOR_IBEAM);
+		SDL_SetCursor(ibeamCursor);
+	}
+	else if (name=="auto")
+		SDL_SetCursor(SDL_GetDefaultCursor());
+	else
+	{
+		LOG(LOG_NOT_IMPLEMENTED,"showCursor for name:"<<name);
+		SDL_SetCursor(SDL_GetDefaultCursor());
+	}		
+}
+tiny_string EngineData::getMouseCursor(SystemState *sys)
+{
+	SDL_Cursor* cursor = SDL_GetCursor();
+	if (cursor == handCursor)
+		return "hand";
+	if (cursor == arrowCursor)
+		return "arrow";
+	if (cursor == ibeamCursor)
+		return "ibeam";
+	return "auto";
+}
 void EngineData::setClipboardText(const std::string txt)
 {
 	int ret = SDL_SetClipboardText(txt.c_str());
@@ -752,11 +1044,89 @@ void EngineData::setClipboardText(const std::string txt)
 		LOG(LOG_ERROR, "copying text to clipboard failed:"<<SDL_GetError());
 }
 
+bool EngineData::getScreenData(SDL_DisplayMode* screen)
+{
+	if (SDL_GetDesktopDisplayMode(0, screen) != 0) {
+		LOG(LOG_ERROR,"Capabilities: SDL_GetDesktopDisplayMode failed:"<<SDL_GetError());
+		return false;
+	}
+	return true;
+}
+
+double EngineData::getScreenDPI()
+{
+#if SDL_VERSION_ATLEAST(2, 0, 4)
+	float ddpi;
+	float hdpi;
+	float vdpi;
+	SDL_GetDisplayDPI(SDL_GetWindowDisplayIndex(widget),&ddpi,&hdpi,&vdpi);
+	return ddpi;
+#else
+	LOG(LOG_NOT_IMPLEMENTED,"getScreenDPI needs SDL version >= 2.0.4");
+	return 96.0;
+#endif
+}
+
+void EngineData::setWindowPosition(int x, int y, uint32_t width, uint32_t height)
+{
+	if (widget)
+	{
+		SDL_SetWindowPosition(widget,x,y);
+		SDL_SetWindowSize(widget,width,height);
+	}
+}
+
+void EngineData::setWindowPosition(const Vector2& pos, const Vector2& size)
+{
+	setWindowPosition(pos.x, pos.y, size.x, size.y);
+}
+
+void EngineData::getWindowPosition(int* x, int* y)
+{
+	if (widget)
+	{
+		SDL_GetWindowPosition(widget,x,y);
+	}
+	else
+	{
+		*x=0;
+		*y=0;
+	}
+}
+
+Vector2 EngineData::getWindowPosition()
+{
+	Vector2 pos;
+	getWindowPosition(&pos.x, &pos.y);
+	return pos;
+}
+
 StreamCache *EngineData::createFileStreamCache(SystemState* sys)
 {
 	return new FileStreamCache(sys);
 }
 
+void EngineData::DoSwapBuffers()
+{
+	uint32_t err;
+	if (getGLError(err))
+		LOG(LOG_ERROR,"swapbuffers:"<<widget<<" "<<err);
+	SDL_GL_SwapWindow(widget);
+}
+
+void EngineData::InitOpenGL()
+{
+	mSDLContext = createSDLGLContext(widget);
+	if (!mSDLContext)
+		LOG(LOG_ERROR,"failed to create openGL context:"<<SDL_GetError());
+	initGLEW();
+}
+
+void EngineData::DeinitOpenGL()
+{
+	deleteSDLGLContext(mSDLContext);
+}
+
 bool EngineData::getGLError(uint32_t &errorCode) const
 {
 	errorCode=glGetError();
@@ -789,6 +1159,8 @@ void EngineData::getGlCompressedTextureFormats()
 		LOG(LOG_INFO,"OpenGL supported compressed texture format:"<<hex<<formats[i]);
 		if (formats[i] == GL_COMPRESSED_RGBA_S3TC_DXT5_EXT)
 			compressed_texture_formats.push_back(TEXTUREFORMAT_COMPRESSED::DXT5);
+		if (formats[i] == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT)
+			compressed_texture_formats.push_back(TEXTUREFORMAT_COMPRESSED::DXT1);
 	}
 	delete [] formats;
 }
@@ -807,6 +1179,11 @@ void EngineData::exec_glUniform4f(int location,float v0,float v1,float v2,float
 	glUniform4f(location,v0,v1,v2,v3);
 }
 
+void EngineData::exec_glUniform1fv(int location, uint32_t size, float* v)
+{
+	glUniform1fv(location,size,v);
+}
+
 void EngineData::exec_glBindTexture_GL_TEXTURE_2D(uint32_t id)
 {
 	glBindTexture(GL_TEXTURE_2D, id);
@@ -890,32 +1267,32 @@ void EngineData::exec_glDisable_GL_STENCIL_TEST()
 {
 	glDisable(GL_STENCIL_TEST);
 }
-void EngineData::exec_glDepthFunc(DEPTH_FUNCTION depthfunc)
+void EngineData::exec_glDepthFunc(DEPTHSTENCIL_FUNCTION depthfunc)
 {
 	switch (depthfunc)
 	{
-		case ALWAYS:
+		case DEPTHSTENCIL_ALWAYS:
 			glDepthFunc(GL_ALWAYS);
 			break;
-		case EQUAL:
+		case DEPTHSTENCIL_EQUAL:
 			glDepthFunc(GL_EQUAL);
 			break;
-		case GREATER:
+		case DEPTHSTENCIL_GREATER:
 			glDepthFunc(GL_GREATER);
 			break;
-		case GREATER_EQUAL:
+		case DEPTHSTENCIL_GREATER_EQUAL:
 			glDepthFunc(GL_GEQUAL);
 			break;
-		case LESS:
+		case DEPTHSTENCIL_LESS:
 			glDepthFunc(GL_LESS);
 			break;
-		case LESS_EQUAL:
+		case DEPTHSTENCIL_LESS_EQUAL:
 			glDepthFunc(GL_LEQUAL);
 			break;
-		case NEVER:
+		case DEPTHSTENCIL_NEVER:
 			glDepthFunc(GL_NEVER);
 			break;
-		case NOT_EQUAL:
+		case DEPTHSTENCIL_NOT_EQUAL:
 			glDepthFunc(GL_NOTEQUAL);
 			break;
 	}
@@ -941,6 +1318,10 @@ void EngineData::exec_glFlush()
 {
 	glFlush();
 }
+void EngineData::exec_glFinish()
+{
+	glFinish();
+}
 
 uint32_t EngineData::exec_glCreateShader_GL_FRAGMENT_SHADER()
 {
@@ -1051,17 +1432,22 @@ void EngineData::exec_glBindRenderbuffer(uint32_t renderBuffer)
 	glBindRenderbuffer(GL_RENDERBUFFER,renderBuffer);
 }
 
-void EngineData::exec_glRenderbufferStorage_GL_RENDERBUFFER_GL_DEPTH_STENCIL(uint32_t width, uint32_t height)
+void EngineData::
+	exec_glRenderbufferStorage_GL_RENDERBUFFER_GL_DEPTH_STENCIL(uint32_t width, uint32_t height)
 {
-#ifndef ENABLE_GLES2
-	glRenderbufferStorage(GL_RENDERBUFFER,GL_DEPTH_STENCIL,width,height);
+#ifdef ENABLE_GLES2
+#ifdef GL_DEPTH24_STENCIL8_OES
+	glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH24_STENCIL8_OES,width,height);
+#endif
+#else
+	glRenderbufferStorage(GL_RENDERBUFFER,GL_DEPTH24_STENCIL8,width,height);
 #endif
 }
 
 void EngineData::exec_glFramebufferRenderbuffer_GL_FRAMEBUFFER_GL_DEPTH_STENCIL_ATTACHMENT(uint32_t depthStencilRenderBuffer)
 {
 #ifndef ENABLE_GLES2
-	glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH_STENCIL_ATTACHMENT, GL_RENDERBUFFER,depthStencilRenderBuffer);
+	glFramebufferRenderbuffer(GL_FRAMEBUFFER,GL_DEPTH_STENCIL_ATTACHMENT, GL_RENDERBUFFER,depthStencilRenderBuffer);	
 #endif
 }
 
@@ -1095,6 +1481,16 @@ void EngineData::exec_glDeleteBuffers(uint32_t size, uint32_t* buffers)
 	glDeleteBuffers(size,buffers);
 }
 
+void EngineData::exec_glDeleteFramebuffers(uint32_t size, uint32_t* buffers)
+{
+	glDeleteFramebuffers(size,buffers);
+}
+
+void EngineData::exec_glDeleteRenderbuffers(uint32_t size, uint32_t* buffers)
+{
+	glDeleteRenderbuffers(size,buffers);
+}
+
 void EngineData::exec_glBlendFunc(BLEND_FACTOR src, BLEND_FACTOR dst)
 {
 	GLenum glsrc, gldst;
@@ -1269,12 +1665,12 @@ void EngineData::exec_glTexImage2D_GL_TEXTURE_2D_GL_UNSIGNED_INT_8_8_8_8_HOST(in
 {
 	glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_HOST, pixels);
 }
-void EngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level,int32_t width, int32_t height,int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat,uint32_t compressedImageSize)
+void EngineData::glTexImage2Dintern(uint32_t type,int32_t level,int32_t width, int32_t height,int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat,uint32_t compressedImageSize)
 {
 	switch (format)
 	{
 		case TEXTUREFORMAT::BGRA:
-			glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_BYTE, pixels);
+			glTexImage2D(type, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_BYTE, pixels);
 			break;
 		case TEXTUREFORMAT::BGR:
 #if ENABLE_GLES2
@@ -1283,20 +1679,19 @@ void EngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level,int32_t width, in
 				((uint8_t*)pixels)[i] = ((uint8_t*)pixels)[i+2];
 				((uint8_t*)pixels)[i+2] = t;
 			}
-			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_BYTE, pixels);
+			glTexImage2D(type, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_BYTE, pixels);
 #else
-			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_BGR, GL_UNSIGNED_BYTE, pixels);
+			glTexImage2D(type, level, GL_RGB, width, height, border, GL_BGR, GL_UNSIGNED_BYTE, pixels);
 #endif
 			break;
 		case TEXTUREFORMAT::BGRA_PACKED:
-			glTexImage2D(GL_TEXTURE_2D, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_SHORT_4_4_4_4, pixels);
+			glTexImage2D(type, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_SHORT_4_4_4_4, pixels);
 			break;
 		case TEXTUREFORMAT::BGR_PACKED:
 #if ENABLE_GLES2
 			LOG(LOG_NOT_IMPLEMENTED,"textureformat BGR_PACKED for opengl es");
-			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, pixels);
 #else
-			glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, border, GL_BGR, GL_UNSIGNED_SHORT_5_6_5, pixels);
+			glTexImage2D(type, level, GL_RGB, width, height, border, GL_BGR, GL_UNSIGNED_SHORT_5_6_5, pixels);
 #endif
 			break;
 		case TEXTUREFORMAT::COMPRESSED:
@@ -1305,7 +1700,10 @@ void EngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level,int32_t width, in
 			switch (compressedformat)
 			{
 				case TEXTUREFORMAT_COMPRESSED::DXT5:
-					glCompressedTexImage2D(GL_TEXTURE_2D, level, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, width, height, border, compressedImageSize, pixels);
+					glCompressedTexImage2D(type, level, GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, width, height, border, compressedImageSize, pixels);
+					break;
+				case TEXTUREFORMAT_COMPRESSED::DXT1:
+					glCompressedTexImage2D(type, level, GL_COMPRESSED_RGB_S3TC_DXT1_EXT, width, height, border, compressedImageSize, pixels);
 					break;
 				default:
 					LOG(LOG_NOT_IMPLEMENTED,"upload texture in compressed format "<<compressedformat);
@@ -1321,7 +1719,14 @@ void EngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level,int32_t width, in
 			break;
 	}
 }
-
+void EngineData::exec_glTexImage2D_GL_TEXTURE_2D(int32_t level,int32_t width, int32_t height,int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat,uint32_t compressedImageSize, bool isRectangleTexture)
+{
+#if ENABLE_GLES2
+	glTexImage2Dintern(GL_TEXTURE_2D ,level, width, height, border, pixels, format, compressedformat,compressedImageSize);
+#else
+	glTexImage2Dintern(isRectangleTexture ? GL_TEXTURE_RECTANGLE : GL_TEXTURE_2D,level, width, height, border, pixels, format, compressedformat,compressedImageSize);
+#endif
+}
 void EngineData::exec_glDrawBuffer_GL_BACK()
 {
 	glDrawBuffer(GL_BACK);
@@ -1375,12 +1780,21 @@ void EngineData::exec_glGenerateMipmap_GL_TEXTURE_2D()
 {
 	glGenerateMipmap(GL_TEXTURE_2D);
 }
+void EngineData::exec_glGenerateMipmap_GL_TEXTURE_CUBE_MAP()
+{
+	glGenerateMipmap(GL_TEXTURE_CUBE_MAP);
+}
 
 void EngineData::exec_glReadPixels(int32_t width, int32_t height, void *buf)
 {
 	glPixelStorei(GL_PACK_ALIGNMENT, 1);
 	glReadPixels(0,0,width, height, GL_RGB, GL_UNSIGNED_BYTE, buf);
 }
+void EngineData::exec_glReadPixels_GL_BGRA(int32_t width, int32_t height,void *buf)
+{
+	glPixelStorei(GL_PACK_ALIGNMENT, 1);
+	glReadPixels(0,0,width, height, GL_BGRA, GL_UNSIGNED_BYTE, buf);
+}
 void EngineData::exec_glBindTexture_GL_TEXTURE_CUBE_MAP(uint32_t id)
 {
 	glBindTexture(GL_TEXTURE_CUBE_MAP, id);
@@ -1393,9 +1807,9 @@ void EngineData::exec_glTexParameteri_GL_TEXTURE_CUBE_MAP_GL_TEXTURE_MAG_FILTER_
 {
 	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 }
-void EngineData::exec_glTexImage2D_GL_TEXTURE_CUBE_MAP_POSITIVE_X_GL_UNSIGNED_BYTE(uint32_t side, int32_t level,int32_t width, int32_t height,int32_t border, const void* pixels)
+void EngineData::exec_glTexImage2D_GL_TEXTURE_CUBE_MAP_POSITIVE_X_GL_UNSIGNED_BYTE(uint32_t side, int32_t level, int32_t width, int32_t height, int32_t border, void* pixels, TEXTUREFORMAT format, TEXTUREFORMAT_COMPRESSED compressedformat, uint32_t compressedImageSize)
 {
-	glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X+side, level, GL_RGBA8, width, height, border, GL_BGRA, GL_UNSIGNED_BYTE, pixels);
+	glTexImage2Dintern(GL_TEXTURE_CUBE_MAP_POSITIVE_X+side, level, width, height, border, pixels, format, compressedformat,compressedImageSize);
 }
 
 void EngineData::exec_glScissor(int32_t x, int32_t y, int32_t width, int32_t height)
@@ -1418,11 +1832,179 @@ void EngineData::exec_glStencilFunc_GL_ALWAYS()
 	glStencilFunc(GL_ALWAYS, 0, 0xff);
 }
 
+void EngineData::exec_glStencilFunc_GL_NEVER()
+{
+	glStencilFunc(GL_NEVER, 0, 0xff);
+}
+
+void EngineData::exec_glStencilFunc_GL_EQUAL(int32_t ref, uint32_t mask)
+{
+	glStencilFunc(GL_EQUAL, ref, mask);
+}
+
+void EngineData::exec_glStencilOp_GL_INCR()
+{
+	glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
+}
+
+void EngineData::exec_glStencilOp_GL_DECR()
+{
+	glStencilOp(GL_KEEP, GL_KEEP, GL_DECR);
+}
+
+void EngineData::exec_glStencilOp_GL_KEEP()
+{
+	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
+}
+
+void EngineData::exec_glStencilOpSeparate(TRIANGLE_FACE face, DEPTHSTENCIL_OP sfail, DEPTHSTENCIL_OP dpfail, DEPTHSTENCIL_OP dppass)
+{
+	GLenum glface=GL_FRONT;
+	switch (face)
+	{
+		case FACE_BACK:
+			glface=GL_BACK;
+			break;
+		case FACE_FRONT:
+			glface=GL_FRONT;
+			break;
+		case FACE_FRONT_AND_BACK:
+			glface=GL_FRONT_AND_BACK;
+			break;
+		case FACE_NONE:
+			glface=GL_NONE;
+			break;
+	}
+	GLenum glsfail=GL_KEEP;
+	switch (sfail)
+	{
+		case DEPTHSTENCIL_KEEP: 
+			glsfail=GL_KEEP;
+			break;
+		case DEPTHSTENCIL_ZERO:
+			glsfail=GL_ZERO;
+			break;
+		case DEPTHSTENCIL_REPLACE:
+			glsfail=GL_REPLACE;
+			break;
+		case DEPTHSTENCIL_INCR:
+			glsfail=GL_INCR;
+			break;
+		case DEPTHSTENCIL_INCR_WRAP:
+			glsfail=GL_INCR_WRAP;
+			break;
+		case DEPTHSTENCIL_DECR:
+			glsfail=GL_DECR;
+			break;
+		case DEPTHSTENCIL_DECR_WRAP:
+			glsfail=GL_DECR_WRAP;
+			break;
+		case DEPTHSTENCIL_INVERT:
+			glsfail=GL_INVERT;
+			break;
+	}
+	GLenum gldpfail=GL_KEEP;
+	switch (dpfail)
+	{
+		case DEPTHSTENCIL_KEEP: 
+			gldpfail=GL_KEEP;
+			break;
+		case DEPTHSTENCIL_ZERO:
+			gldpfail=GL_ZERO;
+			break;
+		case DEPTHSTENCIL_REPLACE:
+			gldpfail=GL_REPLACE;
+			break;
+		case DEPTHSTENCIL_INCR:
+			gldpfail=GL_INCR;
+			break;
+		case DEPTHSTENCIL_INCR_WRAP:
+			gldpfail=GL_INCR_WRAP;
+			break;
+		case DEPTHSTENCIL_DECR:
+			gldpfail=GL_DECR;
+			break;
+		case DEPTHSTENCIL_DECR_WRAP:
+			gldpfail=GL_DECR_WRAP;
+			break;
+		case DEPTHSTENCIL_INVERT:
+			gldpfail=GL_INVERT;
+			break;
+	}
+	GLenum gldppass=GL_KEEP;
+	switch (dppass)
+	{
+		case DEPTHSTENCIL_KEEP: 
+			gldppass=GL_KEEP;
+			break;
+		case DEPTHSTENCIL_ZERO:
+			gldppass=GL_ZERO;
+			break;
+		case DEPTHSTENCIL_REPLACE:
+			gldppass=GL_REPLACE;
+			break;
+		case DEPTHSTENCIL_INCR:
+			gldppass=GL_INCR;
+			break;
+		case DEPTHSTENCIL_INCR_WRAP:
+			gldppass=GL_INCR_WRAP;
+			break;
+		case DEPTHSTENCIL_DECR:
+			gldppass=GL_DECR;
+			break;
+		case DEPTHSTENCIL_DECR_WRAP:
+			gldppass=GL_DECR_WRAP;
+			break;
+		case DEPTHSTENCIL_INVERT:
+			gldppass=GL_INVERT;
+			break;
+	}
+	glStencilOpSeparate(glface, glsfail, gldpfail, gldppass);
+}
+
+void EngineData::exec_glStencilMask(uint32_t mask)
+{
+	glStencilMask(mask);
+}
+
+void EngineData::exec_glStencilFunc(DEPTHSTENCIL_FUNCTION func, uint32_t ref, uint32_t mask)
+{
+	uint32_t f = GL_ALWAYS;
+	switch (func)
+	{
+		case DEPTHSTENCIL_ALWAYS:
+			f = GL_ALWAYS;
+			break;
+		case DEPTHSTENCIL_EQUAL:
+			f = GL_EQUAL;
+			break;
+		case DEPTHSTENCIL_GREATER:
+			f = GL_GREATER;
+			break;
+		case DEPTHSTENCIL_GREATER_EQUAL:
+			f = GL_GEQUAL;
+			break;
+		case DEPTHSTENCIL_LESS:
+			f = GL_LESS;
+			break;
+		case DEPTHSTENCIL_LESS_EQUAL:
+			f = GL_LEQUAL;
+			break;
+		case DEPTHSTENCIL_NEVER:
+			f = GL_NEVER;
+			break;
+		case DEPTHSTENCIL_NOT_EQUAL:
+			f = GL_NOTEQUAL;
+			break;
+	}
+	glStencilFunc(f,ref,mask);
+}
+
 void audioCallback(void * userdata, uint8_t * stream, int len)
 {
 	AudioManager* manager = (AudioManager*)userdata;
 
-    SDL_memset(stream, 0, len);
+	SDL_memset(stream, 0, len);
 
 	{
 		Locker l(manager->streamMutex);
@@ -1445,19 +2027,22 @@ void audioCallback(void * userdata, uint8_t * stream, int len)
 				float* dst32=(float *)(stream+readcount);
 				readcount += ret;
 
-				float src1, src2;
+				float src1left, src1right, src2;
 				double dst_sample;
 
 				const double max_audioval = 3.402823466e+38F;
 				const double min_audioval = -3.402823466e+38F;
+				float* panning = s->getPanning();
 				int curpanning=0;
 				while (ret)
 				{
-					src1 = *src32 * fvolume * fmaxvolume * s->getPanning()[curpanning];
-					curpanning = 1-curpanning;
+					src1left = *src32 * fvolume * fmaxvolume * panning[curpanning];
+					src1right = *(src32+1) * fvolume * fmaxvolume * panning[curpanning+1];
+					curpanning = 2-curpanning;
+					if (!curpanning)
+						src32+=2;
 					src2 = *dst32;
-					src32++;
-					dst_sample = ((double)src1) + ((double)src2);
+					dst_sample = double(src1left) + double(src1right) + double(src2);
 					if (dst_sample > max_audioval) {
 						dst_sample = max_audioval;
 					} else if (dst_sample < min_audioval) {
@@ -1536,23 +2121,18 @@ int EngineData::audio_getSampleRate()
 {
 	return 44100;
 }
-IDrawable *EngineData::getTextRenderDrawable(const TextData &_textData, const MATRIX &_m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r, float _xs, float _ys, bool _im, _NR<DisplayObject> _mask, float _s, float _a, const std::vector<IDrawable::MaskData> &_ms, float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier, float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset, SMOOTH_MODE smoothing)
+IDrawable *EngineData::getTextRenderDrawable(const TextData &_textData, const MATRIX &_m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _xs, float _ys, bool _isMask, bool _cacheAsBitmap, float _s, float _a, const ColorTransformBase& _colortransform, SMOOTH_MODE smoothing,AS_BLENDMODE _blendmode)
 {
 	if (hasExternalFontRenderer)
-		return new externalFontRenderer(_textData,this, _x, _y, _w, _h, _rx,_ry,_rw,_rh,_r,_xs,_ys,_im, _mask, _a, _ms,
-										_redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier,
-										_redOffset,_greenOffset,_blueOffset,_alphaOffset,
-										smoothing,_m);
+		return new externalFontRenderer(_textData,this, _x, _y, _w, _h,_xs,_ys,_isMask, _cacheAsBitmap, _a,
+										_colortransform, smoothing,_blendmode,_m);
 	return nullptr;
 }
 
-externalFontRenderer::externalFontRenderer(const TextData &_textData, EngineData *engine, int32_t x, int32_t y, int32_t w, int32_t h, int32_t rx, int32_t ry, int32_t rw, int32_t rh, float r, float xs, float ys, bool im, _NR<DisplayObject> _mask, float a, const std::vector<IDrawable::MaskData> &m,
-										   float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
-										   float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset,
-										   SMOOTH_MODE smoothing, const MATRIX &_m)
-	: IDrawable(w, h, x, y,rw,rh,rx,ry,r,xs,ys, 1, 1, im, _mask, a, m,
-				_redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier,
-				_redOffset,_greenOffset,_blueOffset,_alphaOffset,smoothing,_m),m_engine(engine)
+externalFontRenderer::externalFontRenderer(const TextData &_textData, EngineData *engine, int32_t x, int32_t y, int32_t w, int32_t h, float xs, float ys, bool _isMask, bool _cacheAsBitmap, float a,
+										   const ColorTransformBase& _colortransform, SMOOTH_MODE smoothing,AS_BLENDMODE _blendmode, const MATRIX &_m)
+	: IDrawable(w, h, x, y,xs,ys, 1, 1, _isMask, _cacheAsBitmap, 1.0, a,
+				_colortransform,smoothing,_blendmode,_m),m_engine(engine)
 {
 	externalressource = engine->setupFontRenderer(_textData,a,smoothing);
 }
