@@ -18,6 +18,7 @@
 **************************************************************************/
 
 #include "scripting/flash/display/flashdisplay.h"
+#include "scripting/flash/display/Bitmap.h"
 #include "scripting/flash/display/BitmapData.h"
 #include "scripting/flash/display/bitmapencodingcolorspace.h"
 #include "scripting/flash/display/colorcorrection.h"
@@ -41,9 +42,19 @@
 #include "scripting/flash/display/shaderparametertype.h"
 #include "scripting/flash/display/shaderprecision.h"
 #include "scripting/flash/display/swfversion.h"
+#include "scripting/flash/display/StageAspectRatio.h"
+#include "scripting/flash/display/Stage.h"
 #include "scripting/flash/display/triangleculling.h"
+#include "scripting/flash/display/Loader.h"
+#include "scripting/flash/display/LoaderInfo.h"
+#include "scripting/flash/display/MorphShape.h"
+#include "scripting/flash/display/MovieClip.h"
 #include "scripting/flash/display/NativeWindow.h"
+#include "scripting/flash/display/Shape.h"
+#include "scripting/flash/display/SimpleButton.h"
+#include "scripting/flash/display/Stage3D.h"
 
+#include "scripting/toplevel/Global.h"
 #include "scripting/class.h"
 #include "scripting/abc.h"
 
@@ -111,5 +122,8 @@ void ABCVm::registerClassesFlashDisplay(Global* builtin)
 	builtin->registerBuiltin("JointStyle","flash.display",Class<JointStyle>::getRef(m_sys));
 	
 	if(m_sys->flashMode==SystemState::AIR)
+	{
 		builtin->registerBuiltin("NativeWindow","flash.display",Class<NativeWindow>::getRef(m_sys));
+		builtin->registerBuiltin("StageAspectRatio","flash.display",Class<StageAspectRatio>::getRef(m_sys));
+	}
 }
