@@ -28,6 +28,7 @@
 #include "scripting/toplevel/Integer.h"
 #include "scripting/toplevel/XML.h"
 
+#include "scripting/toplevel/Global.h"
 #include "scripting/class.h"
 #include "scripting/abc.h"
 using namespace lightspark;
@@ -40,17 +41,17 @@ void ABCVm::registerClassesFlashUtils(Global* builtin)
 	builtin->registerBuiltin("Dictionary","flash.utils",Class<Dictionary>::getRef(m_sys));
 	builtin->registerBuiltin("Proxy","flash.utils",Class<Proxy>::getRef(m_sys));
 	builtin->registerBuiltin("Timer","flash.utils",Class<Timer>::getRef(m_sys));
-	builtin->registerBuiltin("getQualifiedClassName","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,getQualifiedClassName,1,Class<ASString>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("getQualifiedSuperclassName","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,getQualifiedSuperclassName,1,Class<ASString>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("getDefinitionByName","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,getDefinitionByName,1,Class<ASObject>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("getTimer","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,getTimer,0,Class<Integer>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("setInterval","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,setInterval,2,Class<Integer>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("setTimeout","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,setTimeout,2,Class<Integer>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("clearInterval","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,clearInterval)));
-	builtin->registerBuiltin("clearTimeout","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,clearTimeout)));
-	builtin->registerBuiltin("describeType","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,describeType,1,Class<XML>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("escapeMultiByte","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,escapeMultiByte,1,Class<ASString>::getRef(m_sys).getPtr())));
-	builtin->registerBuiltin("unescapeMultiByte","flash.utils",_MR(Class<IFunction>::getFunction(m_sys,unescapeMultiByte,1,Class<ASString>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("getQualifiedClassName","flash.utils",_MR(m_sys->getBuiltinFunction(getQualifiedClassName,1,Class<ASString>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("getQualifiedSuperclassName","flash.utils",_MR(m_sys->getBuiltinFunction(getQualifiedSuperclassName,1,Class<ASString>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("getDefinitionByName","flash.utils",_MR(m_sys->getBuiltinFunction(getDefinitionByName,1,Class<ASObject>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("getTimer","flash.utils",_MR(m_sys->getBuiltinFunction(getTimer,0,Class<Integer>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("setInterval","flash.utils",_MR(m_sys->getBuiltinFunction(setInterval,2,Class<Integer>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("setTimeout","flash.utils",_MR(m_sys->getBuiltinFunction(setTimeout,2,Class<Integer>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("clearInterval","flash.utils",_MR(m_sys->getBuiltinFunction(clearInterval)));
+	builtin->registerBuiltin("clearTimeout","flash.utils",_MR(m_sys->getBuiltinFunction(clearTimeout)));
+	builtin->registerBuiltin("describeType","flash.utils",_MR(m_sys->getBuiltinFunction(describeType,1,Class<XML>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("escapeMultiByte","flash.utils",_MR(m_sys->getBuiltinFunction(escapeMultiByte,1,Class<ASString>::getRef(m_sys).getPtr())));
+	builtin->registerBuiltin("unescapeMultiByte","flash.utils",_MR(m_sys->getBuiltinFunction(unescapeMultiByte,1,Class<ASString>::getRef(m_sys).getPtr())));
 	builtin->registerBuiltin("IExternalizable","flash.utils",InterfaceClass<IExternalizable>::getRef(m_sys));
 	builtin->registerBuiltin("IDataInput","flash.utils",InterfaceClass<IDataInput>::getRef(m_sys));
 	builtin->registerBuiltin("IDataOutput","flash.utils",InterfaceClass<IDataOutput>::getRef(m_sys));
