@@ -19,6 +19,7 @@
 
 #include "scripting/flash/xml/flashxml.h"
 
+#include "scripting/toplevel/Global.h"
 #include "scripting/class.h"
 #include "scripting/abc.h"
 using namespace lightspark;
@@ -27,4 +28,5 @@ void ABCVm::registerClassesFlashXML(Global* builtin)
 {
 	builtin->registerBuiltin("XMLDocument","flash.xml",Class<XMLDocument>::getRef(m_sys));
 	builtin->registerBuiltin("XMLNode","flash.xml",Class<XMLNode>::getRef(m_sys));
+	builtin->registerBuiltin("XMLNodeType","flash.xml",Class<XMLNodeType>::getRef(m_sys));
 }
