@@ -20,13 +20,15 @@
 #include "abc.h"
 #include "compat.h"
 #include "abcutils.h"
-#include "toplevel/toplevel.h"
+#include "scripting/toplevel/toplevel.h"
 #include "scripting/toplevel/Boolean.h"
 #include "scripting/toplevel/ASString.h"
 #include "scripting/toplevel/Number.h"
+#include "scripting/toplevel/Null.h"
 #include "scripting/toplevel/Integer.h"
 #include "scripting/toplevel/UInteger.h"
 #include "scripting/flash/system/flashsystem.h"
+#include "scripting/flash/display/RootMovieClip.h"
 #include <string>
 #include <sstream>
 
@@ -37,10 +39,10 @@ enum SPECIAL_OPCODES { SET_SLOT_NO_COERCE = 0xfb, COERCE_EARLY = 0xfc, GET_SCOPE
 
 struct lightspark::InferenceData
 {
-	const Type* type;
+	Type* type;
 	const ASObject* obj;
 	InferenceData():type(NULL),obj(NULL){}
-	InferenceData(const Type* t):type(t),obj(NULL){}
+	InferenceData(Type* t):type(t),obj(NULL){}
 	InferenceData(const ASObject* o):type(NULL),obj(o){}
 	bool isValid() const { return type!=NULL || obj!=NULL; }
 	/* Method to understand if the passed InferenceData is of the type of this InferenceData
@@ -50,7 +52,7 @@ struct lightspark::InferenceData
 	{
 		if(this->type)
 		{
-			const Class_base* classType=dynamic_cast<const Class_base*>(this->type);
+			Class_base* classType=dynamic_cast<Class_base*>((Type*)this->type);
 			if(classType && classType->isSubClass(c))
 				return true;
 		}
@@ -111,7 +113,7 @@ struct lightspark::BasicBlock
 		for(uint32_t i=0;i<n;i++)
 			stackTypes.pop_back();
 	}
-	void pushStack(const Type* t)
+	void pushStack(Type* t)
 	{
 		stackTypes.push_back(InferenceData(t));
 	}
@@ -125,7 +127,7 @@ struct lightspark::BasicBlock
 			throw ParseException("Invalid code in optimizer");
 		scopeStackTypes.pop_back();
 	}
-	void pushScopeStack(const Type* t)
+	void pushScopeStack(Type* t)
 	{
 		scopeStackTypes.push_back(InferenceData(t));
 	}
@@ -211,7 +213,7 @@ InferenceData ABCVm::earlyBindFindPropStrict(ostream& out, const SyntheticFuncti
 		return ret;
 	//Look on the application domain
 	ASObject* target;
-	bool found = f->mi->context->root->applicationDomain->findTargetByMultiname(*name, target,f->mi->context->root->getInstanceWorker());
+	bool found = f->mi->context->applicationDomain->findTargetByMultiname(*name, target,f->mi->context->applicationDomain->getInstanceWorker());
 	if(found)
 	{
 		//If we found the property on the application domain we can safely use the target verbatim
@@ -257,7 +259,7 @@ InferenceData ABCVm::earlyBindGetLex(ostream& out, const SyntheticFunction* f, c
 	//About custom domains. We can't resolve the object now. But we can output a special getLex opcode that will
 	//rewrite itself to a PUSH_EARLY when it's executed.
 	//NOTE: We use findVariableByMultiname because we don't want to actually run the init scripts now
-	bool found = f->mi->context->root->applicationDomain->findTargetByMultiname(*name, target,f->mi->context->root->getInstanceWorker());
+	bool found = f->mi->context->applicationDomain->findTargetByMultiname(*name, target,f->mi->context->applicationDomain->getInstanceWorker());
 	if(found)
 	{
 		out << (uint8_t)GET_LEX_ONCE;
@@ -270,7 +272,7 @@ InferenceData ABCVm::earlyBindGetLex(ostream& out, const SyntheticFunction* f, c
 	return ret;
 }
 
-const Type* ABCVm::getLocalType(const SyntheticFunction* f, unsigned localIndex)
+Type* ABCVm::getLocalType(const SyntheticFunction* f, unsigned localIndex)
 {
 	if(localIndex==0 && f->isMethod())
 		return f->inClass;
@@ -1216,7 +1218,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 				writeInt32(out,i);
 
-				const Type* t=getLocalType(function, i);
+				Type* t=getLocalType(function, i);
 				curBlock->pushStack(t);
 				break;
 			}
@@ -1315,7 +1317,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 					const multiname* slotType = objData.type->resolveSlotTypeName(t);
 					if(slotType)
 					{
-						ASObject* ret=mi->context->root->applicationDomain->getVariableByMultinameOpportunistic(*slotType,mi->context->root->getInstanceWorker());
+						ASObject* ret=mi->context->applicationDomain->getVariableByMultinameOpportunistic(*slotType,mi->context->applicationDomain->getInstanceWorker());
 						if(ret && ret->getObjectType()==T_CLASS)
 						{
 							Class_base* c=static_cast<Class_base*>(ret);
@@ -1414,7 +1416,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				InferenceData inferredData;
 
 				//Try to resolve the type is it is already defined
-				ASObject* ret=mi->context->root->applicationDomain->getVariableByMultinameOpportunistic(*name,mi->context->root->getInstanceWorker());
+				ASObject* ret=mi->context->applicationDomain->getVariableByMultinameOpportunistic(*name,mi->context->applicationDomain->getInstanceWorker());
 				if(ret && ret->getObjectType()==T_CLASS)
 				{
 					coerceToClass=static_cast<Class_base*>(ret);
@@ -1627,7 +1629,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//TODO: collapse on getlocal
 				out << (uint8_t)opcode;
 				//Infer the type of the object when possible
-				const Type* t=getLocalType(function, opcode-0xd0);
+				Type* t=getLocalType(function, opcode-0xd0);
 				curBlock->pushStack(t);
 				break;
 			}
