@@ -34,10 +34,11 @@ private:
 	bool loading;
 	bool isStreaming;
 public:
-	AVM1Sound(ASWorker* wrk,Class_base* c):Sound(wrk,c),clip(nullptr),loading(false),isStreaming(false){}
+	AVM1Sound(ASWorker* wrk,Class_base* c):Sound(wrk,c),clip(nullptr),loading(false),isStreaming(false){ subtype = SUBTYPE_AVM1SOUND; }
 	AVM1Sound(ASWorker* wrk,Class_base* c, _R<StreamCache> soundData, AudioFormat format, number_t duration_in_ms):Sound(wrk,c,soundData,format,duration_in_ms),
 		clip(nullptr),loading(false),isStreaming(false)
 	{
+		subtype = SUBTYPE_AVM1SOUND;
 	}
 	static void sinit(Class_base* c);
 	void AVM1HandleEvent(EventDispatcher* dispatcher, Event* e) override;
@@ -48,8 +49,11 @@ public:
 	ASFUNCTION_ATOM(setVolume);
 	ASFUNCTION_ATOM(getPan);
 	ASFUNCTION_ATOM(setPan);
+	ASFUNCTION_ATOM(getTransform);
+	ASFUNCTION_ATOM(setTransform);
 	ASFUNCTION_ATOM(stop);
 	ASFUNCTION_ATOM(getPosition);
+	ASFUNCTION_ATOM(AVM1_duration);
 	ASFUNCTION_ATOM(loadSound);
 };
 
