@@ -26,10 +26,10 @@ using namespace lightspark;
 
 void AVM1TextField::sinit(Class_base* c)
 {
-	DisplayObject::AVM1SetupMethods(c);
+	InteractiveObject::AVM1SetupMethods(c);
 	TextField::sinit(c);
 	c->isSealed = false;
-	c->setDeclaredMethodByQName("setNewTextFormat","",Class<IFunction>::getFunction(c->getSystemState(),TextField::_setDefaultTextFormat),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("setNewTextFormat","",c->getSystemState()->getBuiltinFunction(TextField::_setDefaultTextFormat),NORMAL_METHOD,true);
 }
 void AVM1TextFormat::sinit(Class_base* c)
 {
