@@ -10,8 +10,7 @@ multiname* AVM1ContextMenuItem::setVariableByMultiname(multiname& name, asAtom&
 	{
 		if (asAtomHandler::isFunction(o))
 		{
-			asAtomHandler::as<IFunction>(o)->incRef();
-			this->callbackfunction = _MR(asAtomHandler::as<IFunction>(o));
+			this->callbackfunction = _MR(asAtomHandler::getObjectNoCheck(o));
 		}
 		else
 			LOG(LOG_ERROR,"ContextMenuItem.onSelect value is no function:"<<asAtomHandler::toDebugString(o));
@@ -22,8 +21,8 @@ multiname* AVM1ContextMenuItem::setVariableByMultiname(multiname& name, asAtom&
 void AVM1ContextMenuItem::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, NativeMenuItem, _constructor, CLASS_DYNAMIC_NOT_FINAL);
-	c->setDeclaredMethodByQName("caption","",Class<IFunction>::getFunction(c->getSystemState(),NativeMenuItem::_getter_label),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("caption","",Class<IFunction>::getFunction(c->getSystemState(),NativeMenuItem::_setter_label),SETTER_METHOD,true);
+	c->setDeclaredMethodByQName("caption","",c->getSystemState()->getBuiltinFunction(NativeMenuItem::_getter_label),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("caption","",c->getSystemState()->getBuiltinFunction(NativeMenuItem::_setter_label),SETTER_METHOD,true);
 	REGISTER_GETTER_SETTER(c,separatorBefore);
 	REGISTER_GETTER_SETTER(c,visible);
 	REGISTER_GETTER_SETTER(c,enabled);
