@@ -18,7 +18,9 @@
 **************************************************************************/
 
 #include "scripting/abc.h"
+#include "scripting/toplevel/toplevel.h"
 #include "scripting/toplevel/ASString.h"
+#include "scripting/toplevel/Global.h"
 #include "scripting/toplevel/Vector.h"
 #include "scripting/class.h"
 #include "parsing/tags.h"
@@ -27,7 +29,19 @@ using namespace lightspark;
 
 ASObject* lightspark::new_asobject(ASWorker* wrk)
 {
-	return Class<ASObject>::getInstanceS(wrk);
+	Class_base* c=Class<ASObject>::getClass(wrk->getSystemState());
+	ASObject* ret = wrk->freelist_asobject.getObjectFromFreeList()->as<Activation_object>();
+	if (!ret)
+	{
+		ret=new (c->memoryAccount) ASObject(wrk,c);
+		ret->objfreelist = &wrk->freelist_asobject;
+		assert_and_throw(ret);
+	}
+	ret->resetCached();
+	ret->setIsInitialized();
+	ret->constructionComplete();
+	ret->setConstructIndicator();
+	return ret;
 }
 
 Prototype* lightspark::new_objectPrototype(ASWorker* wrk)
@@ -59,7 +73,17 @@ ObjectConstructor* lightspark::new_objectConstructor(Class_base* cls,uint32_t le
 Activation_object* lightspark::new_activationObject(ASWorker* wrk)
 {
 	Class_base* c=Class<ASObject>::getClass(wrk->getSystemState());
-	return new (c->memoryAccount) Activation_object(wrk,c);
+	Activation_object* ret = wrk->freelist_activationobject.getObjectFromFreeList()->as<Activation_object>();
+	if (!ret)
+	{
+		ret=new (c->memoryAccount) Activation_object(wrk,c);
+		assert_and_throw(ret);
+	}
+	ret->resetCached();
+	ret->setIsInitialized();
+	ret->constructionComplete();
+	ret->setConstructIndicator();
+	return ret;
 }
 
 
@@ -71,6 +95,19 @@ Class_inherit::Class_inherit(const QName& name, MemoryAccount* m, const traits_i
 	subtype = SUBTYPE_INHERIT;
 }
 
+bool Class_inherit::checkScriptInit()
+{
+	if (global)
+	{
+		if (inScriptInit)
+			return false;
+		inScriptInit=true;
+		global->checkScriptInit();
+		inScriptInit=false;
+	}
+	return true;
+}
+
 void Class_inherit::finalize()
 {
 	instancefactory.reset();
@@ -103,7 +140,7 @@ void Class_inherit::prepareShutdown()
 	}
 }
 
-void Class_inherit::getInstance(ASWorker* worker, asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+void Class_inherit::getInstance(ASWorker* worker, asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass, bool callSyntheticConstructor)
 {
 	checkScriptInit();
 	//We override the classdef
@@ -138,7 +175,7 @@ void Class_inherit::getInstance(ASWorker* worker, asAtom& ret, bool construct, a
 		}
 	}
 	if(construct)
-		handleConstruction(ret,args,argslen,true);
+		handleConstruction(ret,args,argslen,true,worker->isExplicitlyConstructed(), callSyntheticConstructor);
 }
 void Class_inherit::recursiveBuild(ASObject* target) const
 {
@@ -237,6 +274,11 @@ variable* Class_inherit::findVariableByMultiname(const multiname& name, Class_ba
 	checkScriptInit();
 	return Class_base::findVariableByMultiname(name, cls, nsRealID, isborrowed, considerdynamic, wrk);
 }
+multiname* Class_inherit::setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset, ASWorker* wrk)
+{
+	checkScriptInit();
+	return Class_base::setVariableByMultiname(name, o, allowConst, alreadyset, wrk);
+}
 string Class_inherit::toDebugString() const
 {
 	std::string res = Class_base::toDebugString();
@@ -247,7 +289,7 @@ string Class_inherit::toDebugString() const
 	return res;
 }
 template<>
-void Class<Global>::getInstance(ASWorker* worker,asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+void Class<Global>::getInstance(ASWorker* worker,asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass, bool callSyntheticConstructor)
 {
 	createError<TypeError>(worker,kConstructOfNonFunctionError);
 }
@@ -318,7 +360,68 @@ void lightspark::lookupAndLink(Class_base* c, uint32_t nameID, uint32_t interfac
 	}
 }
 
-void Class<ASObject>::getInstance(ASWorker* worker, asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass)
+void Class<ASObject>::AVM1generator(ASWorker* wrk, asAtom& ret, asAtom* args, const unsigned int argslen)
+{
+	bool canConvert =
+	(
+		argslen != 0 &&
+		asAtomHandler::isValid(args[0]) &&
+		!asAtomHandler::isNullOrUndefined(args[0])
+	);
+
+	if (!canConvert)
+	{
+		ret = asAtomHandler::fromObject(new_asobject(wrk));
+		return;
+	}
+
+	// NOTE: For a non null/undefined argument, AVM1's `Object`
+	// constructor converts the argument into an `Object`, rather than
+	// returning the argument itself.
+	ASObject* obj = nullptr;
+
+	switch (asAtomHandler::getObjectType(args[0]))
+	{
+		case T_BOOLEAN:
+			obj = abstract_b
+			(
+				getSystemState(),
+				args[0].uintval & 0x80
+			);
+			break;
+		case T_NUMBER:
+		{
+			auto num = asAtomHandler::toNumber(args[0]);
+			obj = abstract_d(wrk, num);
+			break;
+		}
+		case T_INTEGER:
+		case T_UINTEGER:
+		{
+			auto _int = asAtomHandler::toInt64(args[0]);
+			obj = abstract_di(wrk, _int);
+			break;
+		}
+		case T_STRING:
+		{
+			auto str = asAtomHandler::toString(args[0], wrk);
+			obj = abstract_s(wrk, str);
+			break;
+		}
+		default:
+			ASATOM_INCREF(args[0]);
+			if (asAtomHandler::isObject(args[0]))
+				obj = asAtomHandler::getObjectNoCheck(args[0]);
+			else
+				ret = args[0];
+			break;
+	}
+
+	if (obj != nullptr)
+		ret.uintval = (LIGHTSPARK_ATOM_VALTYPE)obj | ATOM_OBJECTPTR;
+}
+
+void Class<ASObject>::getInstance(ASWorker* worker, asAtom& ret, bool construct, asAtom* args, const unsigned int argslen, Class_base* realClass, bool callSyntheticConstructor)
 {
 	if (construct && args && argslen == 1 && this == Class<ASObject>::getClass(this->getSystemState()))
 	{
@@ -344,7 +447,7 @@ void Class<ASObject>::getInstance(ASWorker* worker, asAtom& ret, bool construct,
 		realClass=this;
 	ret=asAtomHandler::fromObjectNoPrimitive(new (realClass->memoryAccount) ASObject(worker,realClass));
 	if(construct)
-		handleConstruction(ret,args,argslen,true);
+		handleConstruction(ret,args,argslen,true,worker->isExplicitlyConstructed(),callSyntheticConstructor);
 }
 Class<ASObject>* Class<ASObject>::getClass(SystemState* sys)
 {
