@@ -31,10 +31,11 @@ class NativeApplication: public EventDispatcher
 public:
 	NativeApplication(ASWorker* wrk,Class_base* c):EventDispatcher(wrk,c){}
 	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION_ATOM(_constructor);
-	ASFUNCTION_ATOM(_getNativeApplication);
+	ASPROPERTY_GETTER(_NR<NativeApplication>,nativeApplication);
+	ASFUNCTION_ATOM(_getApplicationDescriptor);
 	ASFUNCTION_ATOM(addEventListener);
+	ASFUNCTION_ATOM(_exit);
 };
 
 class NativeDragManager: public ASObject
