@@ -19,12 +19,12 @@
 
 #include <stack>
 #include "scripting/flash/display/BitmapContainer.h"
-#include "scripting/flash/display/flashdisplay.h"
 #include "scripting/flash/filters/flashfilters.h"
 #include "scripting/flash/geom/flashgeom.h"
 #include "backends/rendering.h"
 #include "backends/image.h"
 #include "backends/decoder.h"
+#include "backends/streamcache.h"
 #include "swf.h"
 
 using namespace std;
@@ -32,10 +32,8 @@ using namespace lightspark;
 
 extern void nanoVGDeleteImage(int image);
 BitmapContainer::BitmapContainer(MemoryAccount* m):stride(0),width(0),height(0),
-	data(reporter_allocator<uint8_t>(m)),
-	redMultiplier(1.0),greenMultiplier(1.0),blueMultiplier(1.0),alphaMultiplier(1.0),
-	redOffset(0),greenOffset(0),blueOffset(0),alphaOffset(0),
-	nanoVGImageHandle(-1),cachedCairoPattern(nullptr)
+	data(reporter_allocator<uint8_t>(m)),renderevent(0),
+	nanoVGImageHandle(-1),cachedCairoPattern(nullptr),nanoVGGradientPattern(nullptr)
 {
 }
 
@@ -44,13 +42,15 @@ BitmapContainer::~BitmapContainer()
 	if (bitmaptexture.isValid())
 	{
 		RenderThread* rt = getSys()->getRenderThread();
-		if (rt)
+		if (rt && rt->isStarted())
 		{
 			getSys()->getRenderThread()->releaseTexture(bitmaptexture);
 		}
 	}
 	if (nanoVGImageHandle != -1)
 		nanoVGDeleteImage(nanoVGImageHandle);
+	if (nanoVGGradientPattern)
+		delete[] nanoVGGradientPattern;
 	if (cachedCairoPattern)
 		cairo_pattern_destroy(cachedCairoPattern);
 		
@@ -58,32 +58,11 @@ BitmapContainer::~BitmapContainer()
 
 uint8_t* BitmapContainer::applyColorTransform(ColorTransform *ctransform)
 {
-	if (ctransform->redMultiplier==1.0 &&
-		ctransform->greenMultiplier==1.0 &&
-		ctransform->blueMultiplier==1.0 &&
-		ctransform->alphaMultiplier==1.0 &&
-		ctransform->redOffset==0.0 &&
-		ctransform->greenOffset==0.0 &&
-		ctransform->blueOffset==0.0 &&
-		ctransform->alphaOffset==0.0)
-		return getData();
-	if (ctransform->redMultiplier==this->redMultiplier &&
-		ctransform->greenMultiplier==this->greenMultiplier &&
-		ctransform->blueMultiplier==this->blueMultiplier &&
-		ctransform->alphaMultiplier==this->alphaMultiplier &&
-		ctransform->redOffset==this->redOffset &&
-		ctransform->greenOffset==this->greenOffset &&
-		ctransform->blueOffset==this->blueOffset &&
-		ctransform->alphaOffset==this->alphaOffset)
+	if (ctransform->isIdentity() && currentcolortransform.isIdentity())
+		return getOriginalData();
+	if (*ctransform==currentcolortransform)
 		return getDataColorTransformed();
-	redMultiplier=ctransform->redMultiplier;
-	greenMultiplier=ctransform->greenMultiplier;
-	blueMultiplier=ctransform->blueMultiplier;
-	alphaMultiplier=ctransform->alphaMultiplier;
-	redOffset=ctransform->redOffset;
-	greenOffset=ctransform->greenOffset;
-	blueOffset=ctransform->blueOffset;
-	alphaOffset=ctransform->alphaOffset;
+	currentcolortransform=*ctransform;
 	return ctransform->applyTransformation(this);
 }
 
@@ -98,32 +77,32 @@ uint8_t* BitmapContainer::applyColorTransform(number_t redMulti, number_t greenM
 		blueOff==0.0 &&
 		alphaOff==0.0)
 		return getData();
-	if (redMulti==this->redMultiplier &&
-		greenMulti==this->greenMultiplier &&
-		blueMulti==this->blueMultiplier &&
-		alphaMulti==this->alphaMultiplier &&
-		redOff==this->redOffset &&
-		greenOff==this->greenOffset &&
-		blueOff==this->blueOffset &&
-		alphaOff==this->alphaOffset)
+	if (redMulti==currentcolortransform.redMultiplier &&
+		greenMulti==currentcolortransform.greenMultiplier &&
+		blueMulti==currentcolortransform.blueMultiplier &&
+		alphaMulti==currentcolortransform.alphaMultiplier &&
+		redOff==currentcolortransform.redOffset &&
+		greenOff==currentcolortransform.greenOffset &&
+		blueOff==currentcolortransform.blueOffset &&
+		alphaOff==currentcolortransform.alphaOffset)
 		return getDataColorTransformed();
-	redMultiplier=redMulti;
-	greenMultiplier=greenMulti;
-	blueMultiplier=blueMulti;
-	alphaMultiplier=alphaMulti;
-	redOffset=redOff;
-	greenOffset=greenOff;
-	blueOffset=blueOff;
-	alphaOffset=alphaOff;
+	currentcolortransform.redMultiplier=redMulti;
+	currentcolortransform.greenMultiplier=greenMulti;
+	currentcolortransform.blueMultiplier=blueMulti;
+	currentcolortransform.alphaMultiplier=alphaMulti;
+	currentcolortransform.redOffset=redOff;
+	currentcolortransform.greenOffset=greenOff;
+	currentcolortransform.blueOffset=blueOff;
+	currentcolortransform.alphaOffset=alphaOff;
 	uint8_t* src = getData();
 	uint8_t* dst = getDataColorTransformed();
 	uint32_t size = getWidth()*getHeight()*4;
 	for (uint32_t i = 0; i < size; i+=4)
 	{
-		dst[i+3] = max(0,min(255,int(((number_t(src[i+3]) * alphaMultiplier) + alphaOffset))));
-		dst[i+2] = max(0,min(255,int(((number_t(src[i+2]) *  blueMultiplier) +  blueOffset)*(number_t(dst[i+3])/255.0))));
-		dst[i+1] = max(0,min(255,int(((number_t(src[i+1]) * greenMultiplier) + greenOffset)*(number_t(dst[i+3])/255.0))));
-		dst[i  ] = max(0,min(255,int(((number_t(src[i  ]) *   redMultiplier) +   redOffset)*(number_t(dst[i+3])/255.0))));
+		dst[i+3] = max(0,min(255,int(((number_t(src[i+3]) * currentcolortransform.alphaMultiplier) + currentcolortransform.alphaOffset))));
+		dst[i+2] = max(0,min(255,int(((number_t(src[i+2]) * currentcolortransform. blueMultiplier) + currentcolortransform. blueOffset)*(number_t(dst[i+3])/255.0))));
+		dst[i+1] = max(0,min(255,int(((number_t(src[i+1]) * currentcolortransform.greenMultiplier) + currentcolortransform.greenOffset)*(number_t(dst[i+3])/255.0))));
+		dst[i  ] = max(0,min(255,int(((number_t(src[i  ]) * currentcolortransform.  redMultiplier) + currentcolortransform.  redOffset)*(number_t(dst[i+3])/255.0))));
 	}
 	return getDataColorTransformed();
 }
@@ -276,41 +255,35 @@ void BitmapContainer::clear()
 	bitmaptexture.makeEmpty();
 }
 
-uint8_t* BitmapContainer::upload(bool refresh)
-{
-	return getData();
-}
-
-TextureChunk& BitmapContainer::getTexture()
-{
-	return bitmaptexture;
-}
-
-void BitmapContainer::uploadFence()
-{
-	ITextureUploadable::uploadFence();
-	decRef();// is increffed in checkTexture
-}
-bool BitmapContainer::checkTexture()
+// needs to be called in renderThread
+bool BitmapContainer::checkTextureForUpload(SystemState* sys)
 {
 	if (isEmpty()) {
 		return false;
 	}
 
 	if (!bitmaptexture.isValid())
-	{
-		bitmaptexture=getSys()->getRenderThread()->allocateTexture(width, height, true);
-	}
-	incRef();// is decreffed in uploadFence
+		bitmaptexture=sys->getRenderThread()->allocateTexture(width, height, true,true);
+	sys->getRenderThread()->loadChunkBGRA(bitmaptexture,width, height,currentcolortransform.isIdentity() ? getData() : getDataColorTransformed());
 	return true;
 }
 
+void BitmapContainer::clone(BitmapContainer* c)
+{
+	memcpy (c->getOriginalData(),getOriginalData(),getWidth()*getHeight()*4);
+	if (!currentcolortransform.isIdentity())
+	{
+		c->currentcolortransform = currentcolortransform;
+		memcpy (c->getDataColorTransformed(),getDataColorTransformed(),getWidth()*getHeight()*4);
+	}
+}
+
 void BitmapContainer::setAlpha(int32_t x, int32_t y, uint8_t alpha)
 {
 	if (x < 0 || x >= width || y < 0 || y >= height)
 		return;
-	resetColorTransform();
-	uint32_t *p=reinterpret_cast<uint32_t *>(&data[y*stride + 4*x]);
+	uint8_t* d = getCurrentData();
+	uint32_t *p=reinterpret_cast<uint32_t *>(&d[y*stride + 4*x]);
 	*p = ((uint32_t)alpha << 24) + (*p & 0xFFFFFF);
 }
 
@@ -318,48 +291,72 @@ void BitmapContainer::setPixel(int32_t x, int32_t y, uint32_t color, bool setAlp
 {
 	if (x < 0 || x >= width || y < 0 || y >= height)
 		return;
-
-	resetColorTransform();
-	uint32_t *p=reinterpret_cast<uint32_t *>(&data[y*stride + 4*x]);
-	if(setAlpha)
-	{
-		if (ispremultiplied || (((*p)&0xff000000) == 0xff000000))
-			*p=color;
-		else
-		{
-			uint32_t res = 0;
-			uint32_t alpha = ((color >> 24)&0xff);
-			res |= ((((color >> 0) &0xff) * alpha)&0xff) << 0;
-			res |= ((((color >> 8) &0xff) * alpha)&0xff) << 8;
-			res |= ((((color >> 16) &0xff) * alpha)&0xff) << 16;
-			res |= alpha<<24;
-			*p=res;
-		}
-	}
+	uint8_t* d = getCurrentData();
+	uint32_t *p=reinterpret_cast<uint32_t *>(&d[y*stride + 4*x]);
+	if (ispremultiplied || ((color&0xff000000) == 0xff000000))
+		*p=color;
 	else
-		*p=(*p & 0xff000000) | (color & 0x00ffffff);
+	{
+		uint32_t res = 0;
+		uint32_t alpha = (((setAlpha ? color : *p) >> 24)&0xff);
+		res |= ((((color >> 0 ) &0xff) * alpha +0x7f)/0xff) << 0;
+		res |= ((((color >> 8 ) &0xff) * alpha +0x7f)/0xff) << 8;
+		res |= ((((color >> 16) &0xff) * alpha +0x7f)/0xff) << 16;
+		res |= alpha<<24;
+		*p=res;
+	}
 }
 
+// values taken from ruffle, see https://github.com/ruffle-rs/ruffle/blob/master/core/src/bitmap/bitmap_data.rs
+uint32_t FLASH_PREMUL_FACTOR[256] = {
+	0, 16678912, 8339456, 5559638, 4169728, 3335783, 2779819, 2386603, 2086230, 1855488,
+	1667892, 1518251, 1391151, 1285234, 1193302, 1111928, 1043895, 981113, 927744, 879275,
+	834621, 795535, 759126, 726358, 695839, 668183, 642538, 618737, 596651, 576171, 555964,
+	538706, 522104, 506319, 490557, 477321, 464038, 451353, 439544, 428244, 417582, 407500,
+	397768, 388535, 379630, 371117, 363179, 355235, 348050, 340965, 334052, 327038, 321269,
+	315077, 309159, 303586, 298189, 293092, 287981, 283080, 278251, 273892, 269268, 265179,
+	261087, 256971, 253160, 249322, 245508, 242164, 238575, 235245, 231859, 228848, 225785,
+	222712, 219616, 216827, 213985, 211432, 208835, 206075, 203750, 201196, 198895, 196223,
+	194301, 191987, 189686, 187636, 185559, 183426, 181453, 179444, 177638, 175855, 174054,
+	171948, 170489, 168695, 166889, 165365, 163519, 162045, 160508, 158970, 157429, 156150,
+	154610, 153081, 151803, 150511, 148986, 147709, 146420, 145116, 143868, 142586, 141545,
+	140277, 139194, 137957, 136954, 135676, 134652, 133621, 132604, 131577, 130552, 129527,
+	128508, 127476, 126451, 125432, 124670, 123645, 122818, 121847, 121082, 120060, 119288,
+	118263, 117502, 116720, 115967, 115195, 114424, 113655, 112893, 112125, 111356, 110563,
+	109811, 109048, 108287, 107766, 107004, 106236, 105724, 104953, 104434, 103676, 102904,
+	102375, 101879, 101119, 100604, 99834, 99321, 98813, 98112, 97533, 97019, 96509, 95994,
+	95486, 94713, 94185, 93689, 93179, 92667, 92149, 91643, 91129, 90621, 90068, 89597,
+	89342, 88829, 88318, 87804, 87294, 87034, 86523, 85994, 85499, 85245, 84732, 84222,
+	83956, 83450, 82937, 82685, 82173, 81840, 81405, 80889, 80638, 80127, 79862, 79354,
+	79103, 78590, 78332, 78077, 77565, 77308, 76795, 76541, 76284, 75766, 75518, 75262,
+	74748, 74493, 74238, 73691, 73470, 73214, 72959, 72447, 72189, 71935, 71671, 71166,
+	70911, 70651, 70399, 70140, 69886, 69615, 69116, 68861, 68603, 68350, 68093, 67839,
+	67576, 67326, 67070, 66813, 66556, 66302, 66046, 65791, 65408,
+};
+
 uint32_t BitmapContainer::getPixel(int32_t x, int32_t y,bool premultiplied) const
 {
 	if (x < 0 || x >= width || y < 0 || y >= height)
 		return 0;
-
-	const uint32_t *p=reinterpret_cast<const uint32_t *>(&data[y*stride + 4*x]);
+	
+	uint8_t* d = getCurrentData();
+	const uint32_t *p=reinterpret_cast<const uint32_t *>(&d[y*stride + 4*x]);
 	if (!premultiplied)
 	{
 		uint32_t res = 0;
 		uint32_t alpha = ((*p >> 24)&0xff);
 		if (alpha && alpha != 0xff)
 		{
-			// return value with "un-multiplied" alpha: ceiling(value*255/alpha)
+			// return value with "un-multiplied" alpha: algorithm taken from ruffle
 			uint32_t b = ((*p) >> 0) &0xff;
 			uint32_t g = ((*p) >> 8) &0xff;
 			uint32_t r = ((*p) >> 16) &0xff;
-			//x/y + (x % y != 0);
-			res |= (((b*0xff)/alpha+((b*0xff)%alpha ? 1:0))&0xff) << 0;
-			res |= (((g*0xff)/alpha+((g*0xff)%alpha ? 1:0))&0xff) << 8;
-			res |= (((r*0xff)/alpha+((r*0xff)%alpha ? 1:0))&0xff) << 16;
+
+			uint32_t alpha_factor = FLASH_PREMUL_FACTOR[alpha];
+
+			res |= ((b*alpha_factor+0x8000)>>16)<<0;
+			res |= ((g*alpha_factor+0x8000)>>16)<<8;
+			res |= ((r*alpha_factor+0x8000)>>16)<<16;
 			res |= alpha<<24;
 			return res;
 		}
@@ -382,35 +379,47 @@ void BitmapContainer::copyRectangle(_R<BitmapContainer> source,
 
 	if (copyWidth <= 0 || copyHeight <= 0)
 		return;
-	resetColorTransform();
 	int sx = clippedSourceRect.Xmin;
 	int sy = clippedSourceRect.Ymin;
+	uint8_t *p = getCurrentData();
 	if (mergeAlpha==false)
 	{
 		//Fast path using memmove
 		for (int i=0; i<copyHeight; i++)
 		{
-			memmove(&data[(clippedY+i)*stride + 4*clippedX],
+			memmove(&p[(clippedY+i)*stride + 4*clippedX],
 				&source->data[(sy+i)*source->stride + 4*sx],
 				4*copyWidth);
 		}
 	}
 	else
 	{
-		uint8_t* sourcedata = &source->data[0];
+		uint8_t* sourcedata = source->getCurrentData();
 		bool needsdeletion = false;
-		if (sourcedata == &data[0])
+		if (sourcedata == p)
 		{
-			// cairo doesn't work if source and destination buffers are the same 
+			// source and destination are the same BitmapContainer, so we operate on a copy
+			// TODO check if it is really necessary (source/destination rectangles overlap)
 			sourcedata = new uint8_t[data.size()];
-			memcpy (sourcedata,&data[0],data.size());
+			memcpy (sourcedata,p,data.size());
 			needsdeletion = true;
 		}
-		//Slow path using Cairo
-		CairoRenderContext ctxt(&data[0], width, height,false);
-		ctxt.simpleBlit(clippedX, clippedY, sourcedata,
-				source->getWidth(), source->getHeight(),
-				sx, sy, copyWidth, copyHeight);
+		// TODO check if there is a faster algorithm for this
+		for (int i=0; i<copyHeight; i++)
+		{
+			for (int j=0; j<copyWidth; j++)
+			{
+				uint32_t* pdst = reinterpret_cast<uint32_t *>(&p[(clippedY+i)*stride+4*(j+clippedX)]);
+				uint32_t* psrc = reinterpret_cast<uint32_t *>(&sourcedata[(sy+i)*source->stride+4*(j+sx)]);
+				uint32_t srcalpha = ((*psrc >> 24)&0xff);
+				uint32_t dstalpha = 0xff-srcalpha;
+				uint32_t b = (((*psrc)     ) &0xff) + ((((*pdst)     ) &0xff) * dstalpha) / 0xff;
+				uint32_t g = (((*psrc) >> 8) &0xff) + ((((*pdst) >> 8) &0xff) * dstalpha) / 0xff;
+				uint32_t r = (((*psrc) >>16) &0xff) + ((((*pdst) >>16) &0xff) * dstalpha) / 0xff;
+				uint32_t a = (((*psrc) >>24) &0xff) + ((((*pdst) >>24) &0xff) * dstalpha) / 0xff;
+				*pdst = (b & 0xff) | ((g&0xff)<<8) | ((r&0xff)<<16) | (a<<24);
+			}
+		}
 		if (needsdeletion)
 			delete[] sourcedata;
 	}
@@ -418,7 +427,7 @@ void BitmapContainer::copyRectangle(_R<BitmapContainer> source,
 
 void BitmapContainer::applyFilter(_R<BitmapContainer> source,
 				    const RECT& sourceRect,
-				    int32_t destX, int32_t destY,
+				    number_t destX, number_t destY,
 				    BitmapFilter* filter)
 {
 	RECT clippedSourceRect;
@@ -435,20 +444,19 @@ void BitmapContainer::fillRectangle(const RECT& inputRect, uint32_t color, bool
 	if (clippedRect.Ymin>=clippedRect.Ymax || clippedRect.Xmin>=clippedRect.Xmax)
 		return;
 	
-	resetColorTransform();
 	uint32_t realcolor = useAlpha ? color : (0xFF000000 | (color & 0xFFFFFF));
 	// fill first line
 	for(int32_t x=clippedRect.Xmin;x<clippedRect.Xmax;x++)
 	{
 		uint32_t offset=clippedRect.Ymin*stride + x*4;
-		uint32_t* ptr=(uint32_t*)(getData()+offset);
+		uint32_t* ptr=(uint32_t*)(getCurrentData()+offset);
 		*ptr = realcolor;
 	}
 	// use memcpy to fill all other lines
 	for(int32_t y=clippedRect.Ymin+1;y<clippedRect.Ymax;y++)
 	{
 		uint32_t offset=y*stride + clippedRect.Xmin*4;
-		memcpy(getData()+offset,getData()+clippedRect.Ymin*stride+clippedRect.Xmin*4,(clippedRect.Xmax-clippedRect.Xmin)*4);
+		memcpy(getCurrentData()+offset,getData()+clippedRect.Ymin*stride+clippedRect.Xmin*4,(clippedRect.Xmax-clippedRect.Xmin)*4);
 	}
 }
 
@@ -487,18 +495,13 @@ bool BitmapContainer::scroll(int32_t x, int32_t y)
 
 inline uint32_t *BitmapContainer::getDataNoBoundsChecking(int32_t x, int32_t y) const
 {
-	return (uint32_t*)&data[y*stride + 4*x];
+	uint8_t* d = getCurrentData();
+	return (uint32_t*)&d[y*stride + 4*x];
 }
-void BitmapContainer::resetColorTransform()
+
+uint8_t* BitmapContainer::getCurrentData() const
 {
-	redMultiplier=1.0;
-	greenMultiplier=1.0;
-	blueMultiplier=1.0;
-	alphaMultiplier=1.0;
-	redOffset=0.0;
-	greenOffset=0.0;
-	blueOffset=0.0;
-	alphaOffset=0.0;
+	return currentcolortransform.isIdentity() ? (uint8_t*)data.data() : (uint8_t*)data_colortransformed.data();
 }
 
 /*
@@ -523,7 +526,6 @@ void BitmapContainer::floodFill(int32_t startX, int32_t startY, uint32_t color)
 	if (startX < 0 || startX >= width || startY < 0 || startY >= height)
 		return;
 
-	resetColorTransform();
 	uint32_t seedColor = getPixel(startX, startY);
 
 	// Comment on the codeproject.com: "needed in some cases" ???
@@ -660,7 +662,7 @@ void BitmapContainer::clipRect(_R<BitmapContainer> source, const RECT& sourceRec
 	outputY = dTop;
 }
 
-std::vector<uint32_t> BitmapContainer::getPixelVector(const RECT& inputRect) const
+std::vector<uint32_t> BitmapContainer::getPixelVector(const RECT& inputRect, bool premultiplied) const
 {
 	RECT rect;
 	clipRect(inputRect, rect);
@@ -674,7 +676,10 @@ std::vector<uint32_t> BitmapContainer::getPixelVector(const RECT& inputRect) con
 	{
 		for (int32_t x=rect.Xmin; x<rect.Xmax; x++)
 		{
-			result.push_back(*getDataNoBoundsChecking(x, y));
+			if (premultiplied)
+				result.push_back(*getDataNoBoundsChecking(x, y));
+			else
+				result.push_back(getPixel(x, y,false));
 		}
 	}
 
