@@ -20,11 +20,12 @@
 #ifndef SCRIPTING_FLASH_DISPLAY_DISPLAYOBJECT_H
 #define SCRIPTING_FLASH_DISPLAY_DISPLAYOBJECT_H 1
 
+#include "forwards/backends/graphics.h"
+#include "forwards/swftypes.h"
 #include "smartrefs.h"
 #include "scripting/flash/display/IBitmapDrawable.h"
 #include "asobject.h"
 #include "scripting/flash/events/flashevents.h"
-#include "backends/graphics.h"
 
 namespace lightspark
 {
@@ -37,6 +38,9 @@ class Stage;
 class Transform;
 class Rectangle;
 class KeyboardEvent;
+class InvalidateQueue;
+class CachedSurface;
+struct RenderDisplayObjectToBitmapContainer;
 
 class DisplayObject: public EventDispatcher, public IBitmapDrawable
 {
@@ -52,12 +56,6 @@ friend class Bitmap;
 friend class CairoRenderer;
 friend class Graphics;
 friend std::ostream& operator<<(std::ostream& s, const DisplayObject& r);
-public:
-	enum HIT_TYPE { GENERIC_HIT, // point is over the object
-					GENERIC_HIT_INVISIBLE, // ...even if the object is invisible
-					MOUSE_CLICK, // point over the object and mouseEnabled
-					DOUBLE_CLICK // point over the object and doubleClickEnabled
-				  };
 private:
 	ASPROPERTY_GETTER_SETTER(_NR<AccessibilityProperties>,accessibilityProperties);
 	static ATOMIC_INT32(instanceCount);
@@ -69,12 +67,19 @@ private:
 	AS_BLENDMODE blendMode;
 	// if true, this displayobject is the root object of a loaded file (swf or image)
 	bool isLoadedRoot;
-	bool ismask;
+	bool filterlistHasChanged;
+	uint32_t ismaskCount; // number of DisplayObjects this is mask of
+	
+	number_t maxfilterborder;
 public:
 	UI16_SWF Ratio;
 	int ClipDepth;
+	// list of objects that are currently not on stage, but need to be handled in frame events
+	DisplayObject* hiddenPrevDisplayObject;
+	DisplayObject* hiddenNextDisplayObject;
 	DisplayObject* avm1PrevDisplayObject;
 	DisplayObject* avm1NextDisplayObject;
+	std::map<uint32_t,asAtom > avm1locals;
 private:
 	// the parent is not handled as a _NR<DisplayObjectContainer> because that will lead to circular dependencies in refcounting
 	// and the parent can never be destructed
@@ -85,7 +90,7 @@ private:
 	/* cachedSurface may only be read/written from within the render thread
 	 * It is the cached version of the object for fast draw on the Stage
 	 */
-	CachedSurface cachedSurface;
+	_R<CachedSurface> cachedSurface;
 	/*
 	 * Utility function to set internal MATRIX
 	 * Also used by Transform
@@ -95,15 +100,14 @@ private:
 	ACQUIRE_RELEASE_FLAG(constructed);
 	bool useLegacyMatrix;
 	bool needsTextureRecalculation;
-	bool needsNeedsCachedBitmapRecalculation;
 	bool textureRecalculationSkippable;
-	void gatherMaskIDrawables(std::vector<IDrawable::MaskData>& masks);
 	std::map<uint32_t,asAtom> avm1variables;
 	uint32_t avm1mouselistenercount;
 	uint32_t avm1framelistenercount;
+	void onSetScrollRect(_NR<Rectangle> oldValue);
 protected:
-	_NR<Bitmap> cachedBitmap;
 	_NR<Rectangle> scalingGrid;
+	MATRIX currentrendermatrix;
 	std::multimap<uint32_t,_NR<DisplayObject>> variablebindings;
 	bool onStage;
 	bool visible;
@@ -111,7 +115,8 @@ protected:
 	/**
 	  	The object that masks us, if any
 	*/
-	_NR<DisplayObject> mask;
+	DisplayObject* mask;
+	DisplayObject* clipMask;
 	mutable Mutex spinlock;
 	void computeBoundsForTransformedRect(number_t xmin, number_t xmax, number_t ymin, number_t ymax,
 			number_t& outXMin, number_t& outYMin, number_t& outWidth, number_t& outHeight,
@@ -126,68 +131,71 @@ protected:
 	bool skipRender() const;
 
 	bool defaultRender(RenderContext& ctxt);
-	virtual bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax)
+	virtual bool boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, bool visibleOnly)
 	{
 		throw RunTimeException("DisplayObject::boundsRect: Derived class must implement this!");
 	}
-	virtual bool boundsRectWithoutChildren(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax)
-	{
-		return boundsRect(xmin, xmax, ymin, ymax);
-	}
-	bool boundsRectGlobal(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax);
-	virtual bool renderImpl(RenderContext& ctxt)
-	{
-		throw RunTimeException("DisplayObject::renderImpl: Derived class must implement this!");
-	}
-	virtual _NR<DisplayObject> hitTestImpl(number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly)
+	bool boundsRectGlobal(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, bool fromcurrentrendering=true);
+	virtual _NR<DisplayObject> hitTestImpl(const Vector2f& globalPoint, const Vector2f& localPoint, HIT_TYPE type,bool interactiveObjectsOnly)
 	{
 		throw RunTimeException("DisplayObject::hitTestImpl: Derived class must implement this!");
 	}
 	virtual void afterSetLegacyMatrix() {}
+	bool skipCountCylicMemberReferences(garbagecollectorstate& gcstate);
 public:
+	bool boundsRectGlobal(RectF& rect, bool fromcurrentrendering=true);
+	virtual bool boundsRectWithoutChildren(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, bool visibleOnly)
+	{
+		return boundsRect(xmin, xmax, ymin, ymax, visibleOnly);
+	}
+	virtual void fillGraphicsData(Vector* v, bool recursive) {}
+	void updatedRect(); // scrollrect was changed
 	void setMask(_NR<DisplayObject> m);
+	void setClipMask(_NR<DisplayObject> m);
 	void setBlendMode(UI8 blendmode);
 	AS_BLENDMODE getBlendMode() const { return blendMode; }
-	void constructionComplete() override;
-	void afterConstruction() override;
+	static bool isShaderBlendMode(AS_BLENDMODE bl);
+	void constructionComplete(bool _explicit = false) override;
+	void beforeConstruction(bool _explicit = false) override;
+	void afterConstruction(bool _explicit = false) override;
 	void prepareDestruction()
 	{
 		destroyContents();
 		setParent(nullptr);
 		removeAVM1Listeners();
 	}
-	void applyFilters(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, int xpos, int ypos,number_t scalex,number_t scaley);
+	void applyFilters(BitmapContainer* target, BitmapContainer* source, const RECT& sourceRect, number_t xpos, number_t ypos, number_t scalex, number_t scaley);
 	_NR<DisplayObject> invalidateQueueNext;
-	_NR<LoaderInfo> loaderInfo;
+	LoaderInfo* loaderInfo;
 	ASPROPERTY_GETTER_SETTER(_NR<Array>,filters);
 	ASPROPERTY_GETTER_SETTER(_NR<Rectangle>,scrollRect);
 	_NR<ColorTransform> colorTransform;
-	// pointer to the ancestor of this DisplayObject that is cached as Bitmap
-	DisplayObject* cachedAsBitmapOf;
-	void invalidateCachedAsBitmapOf();
 	void setNeedsTextureRecalculation(bool skippable=false);
 	void resetNeedsTextureRecalculation() { needsTextureRecalculation=false; }
 	bool getNeedsTextureRecalculation() const { return needsTextureRecalculation; }
-	void setNeedsCachedBitmapRecalculation()  { needsNeedsCachedBitmapRecalculation=true; }
-	void resetNeedsCachedBitmapRecalculation() { needsNeedsCachedBitmapRecalculation=false; }
 	bool getTextureRecalculationSkippable() const { return textureRecalculationSkippable; }
 	// this may differ from the main clip if this instance was generated from a loaded swf, not from the main clip
-	RootMovieClip* loadedFrom;
+	ApplicationDomain* loadedFrom;
 	// this is reset after the drawjob is done to ensure a changed DisplayObject is only rendered once
 	bool hasChanged;
 	// this is set to true for DisplayObjects that are placed from a tag
 	bool legacy;
+	// The frame that this clip was placed on.
+	unsigned int placeFrame;
 	bool markedForLegacyDeletion;
-	/**
-	 * cacheAsBitmap is true also if any filter is used
-	 */
-	bool computeCacheAsBitmap(bool checksize=true);
-	bool requestInvalidationForCacheAsBitmap(InvalidateQueue* q);
-	void computeMasksAndMatrix(const DisplayObject *target, std::vector<IDrawable::MaskData>& masks, MATRIX& totalMatrix, bool includeRotation, bool &isMask, _NR<DisplayObject>& mask) const;
+	_R<CachedSurface>& getCachedSurface() { return cachedSurface; }
+	bool needsCacheAsBitmap() const;
+	bool hasFilters() const;
+	void requestInvalidationFilterParent(InvalidateQueue* q=nullptr);
+	virtual void requestInvalidationIncludingChildren(InvalidateQueue* q);
 	ASPROPERTY_GETTER_SETTER(bool,cacheAsBitmap);
-	IDrawable* getCachedBitmapDrawable(DisplayObject* target, const MATRIX& initialMatrix, _NR<DisplayObject>* pcachedBitmap);
-	_NR<DisplayObject> getCachedBitmap() const { return cachedBitmap; }
+	IDrawable* getFilterDrawable(bool smoothing);
 	DisplayObjectContainer* getParent() const { return parent; }
+	int getParentDepth() const;
+	int findParentDepth(DisplayObject* d) const;
+	DisplayObjectContainer* getAncestor(int depth) const;
+	DisplayObjectContainer* findCommonAncestor(DisplayObject* d, int& depth, bool init = true) const;
+	DisplayObjectContainer* findCommonAncestor(DisplayObject* d) const { int dummy; return findCommonAncestor(d, dummy); }
 	bool findParent(DisplayObject* d) const;
 	void setParent(DisplayObjectContainer* p);
 	void setScalingGrid();
@@ -201,24 +209,24 @@ public:
 	void prepareShutdown() override;
 	bool countCylicMemberReferences(garbagecollectorstate& gcstate) override;
 	MATRIX getMatrix(bool includeRotation = true) const;
-	bool isConstructed() const override { return ACQUIRE_READ(constructed); }
+	bool isConstructed() const override;
 	/**
 	 * Generate a new IDrawable instance for this object
 	 * @param target The topmost object in the hierarchy that is being drawn. Such object
 	 * _must_ be on the parent chain of this
 	 * @param initialMatrix A matrix that will be prepended to all transformations
 	 */
-	virtual IDrawable* invalidate(DisplayObject* target, const MATRIX& initialMatrix, bool smoothing, InvalidateQueue* q, _NR<DisplayObject>* cachedBitmap);
+	virtual IDrawable* invalidate(bool smoothing);
+	virtual void invalidateForRenderToBitmap(RenderDisplayObjectToBitmapContainer* container);
 	virtual void requestInvalidation(InvalidateQueue* q, bool forceTextureRefresh=false);
 	void updateCachedSurface(IDrawable* d);
-	MATRIX getConcatenatedMatrix(bool includeRoot=false) const;
-	void localToGlobal(number_t xin, number_t yin, number_t& xout, number_t& yout) const;
-	void globalToLocal(number_t xin, number_t yin, number_t& xout, number_t& yout) const;
+	MATRIX getConcatenatedMatrix(bool includeRoot=false, bool fromcurrentrendering=true) const;
+	Vector2f localToGlobal(const Vector2f& point, bool fromcurrentrendering=true) const;
+	Vector2f globalToLocal(const Vector2f& point, bool fromcurrentrendering=true) const;
+	void localToGlobal(number_t xin, number_t yin, number_t& xout, number_t& yout, bool fromcurrentrendering=true) const;
+	void globalToLocal(number_t xin, number_t yin, number_t& xout, number_t& yout, bool fromcurrentrendering=true) const;
 	float getConcatenatedAlpha() const;
-	virtual float getScaleFactor() const
-	{
-		throw RunTimeException("DisplayObject::getScaleFactor");
-	}
+	virtual float getScaleFactor() const;
 	multiname* setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset, ASWorker* wrk) override;
 	bool deleteVariableByMultiname(const multiname& name, ASWorker* wrk) override;
 	virtual void removeAVM1Listeners();
@@ -229,37 +237,47 @@ public:
 	void onNewEvent(Event *ev) override;
 	void afterHandleEvent(Event* ev) override;
 	
+	void onSetName(uint32_t oldName);
+
 	virtual void UpdateVariableBinding(asAtom v) {}
 	
 	tiny_string AVM1GetPath();
 	virtual void afterLegacyInsert();
-	virtual void afterLegacyDelete(DisplayObjectContainer* parent,bool inskipping) {}
+	virtual void afterLegacyDelete(bool inskipping) {}
 	virtual uint32_t getTagID() const { return UINT32_MAX;}
-	virtual void startDrawJob() {}
-	virtual void endDrawJob() {}
 	
-	bool Render(RenderContext& ctxt,bool force=false);
-	bool getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, const MATRIX& m);
-	_NR<DisplayObject> hitTest(number_t x, number_t y, HIT_TYPE type,bool interactiveObjectsOnly);
+	bool getBounds(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax, const MATRIX& m, bool visibleOnly=false);
+	_NR<DisplayObject> hitTest(const Vector2f& globalPoint, const Vector2f& localPoint, HIT_TYPE type,bool interactiveObjectsOnly);
 	virtual void setOnStage(bool staged, bool force, bool inskipping=false);
 	bool isOnStage() const { return onStage; }
-	bool isMask() const { return ismask; }
+	bool isMask() const { return ismaskCount; }
 	// checks for visibility depending on parent visibility 
 	bool isVisible() const;
 	bool isLoadedRootObject() const { return isLoadedRoot; }
 	float clippedAlpha() const;
 	float getRotation() const { return rotation; }
+	int getRawDepth();
+	int getDepth();
+	int getClipDepth() const;
+	number_t getMaxFilterBorder() const { return maxfilterborder; }
 	virtual _NR<RootMovieClip> getRoot();
 	virtual _NR<Stage> getStage();
 	void setLegacyMatrix(const MATRIX& m);
 	void setFilters(const FILTERLIST& filterlist);
+	virtual void refreshSurfaceState();
+	void setupSurfaceState(IDrawable* d);
+
+	bool placedByActionScript;
+	// If set, skip the next call to enterFrame().
+	bool skipFrame;
+	virtual void enterFrame(bool implicit) {}
 	virtual void advanceFrame(bool implicit) {}
 	virtual void declareFrame(bool implicit) {}
 	virtual void initFrame();
 	virtual void executeFrameScript();
 	virtual bool needsActionScript3() const;
 	virtual void handleMouseCursor(bool rollover) {}
-	virtual bool hasGraphics() const { return false; }
+	virtual bool allowAsMask() const { return true; }
 	Vector2f getLocalMousePos();
 	Vector2f getXY();
 	void setX(number_t x);
@@ -268,11 +286,16 @@ public:
 	void setScaleX(number_t val);
 	void setScaleY(number_t val);
 	void setScaleZ(number_t val);
-	inline void setVisible(bool v) { this->visible = v; }
+	void setVisible(bool v);
+	void setLoaderInfo(LoaderInfo* li);
 	// Nominal width and heigt are the size before scaling and rotation
 	number_t getNominalWidth();
 	number_t getNominalHeight();
-	DisplayObject* getMask() const { return mask.getPtr(); }
+	DisplayObject* getMask() const { return mask; }
+	DisplayObject* getClipMask() const { return clipMask; }
+	bool inMask() const;
+	bool belongsToMask() const;
+	
 	static void sinit(Class_base* c);
 	ASFUNCTION_ATOM(_getVisible);
 	ASFUNCTION_ATOM(_setVisible);
@@ -318,7 +341,7 @@ public:
 	ASFUNCTION_ATOM(hitTestPoint);
 	ASPROPERTY_GETTER_SETTER(number_t, rotationX);
 	ASPROPERTY_GETTER_SETTER(number_t, rotationY);
-	ASPROPERTY_GETTER_SETTER(_NR<ASObject>, opaqueBackground);
+	ASPROPERTY_GETTER_SETTER(asAtom, opaqueBackground);
 	ASPROPERTY_GETTER_SETTER(_NR<ASObject>, metaData);
 	
 	ASFUNCTION_ATOM(AVM1_getScaleX);
@@ -340,8 +363,9 @@ public:
 	ASFUNCTION_ATOM(AVM1_getBounds);
 	ASFUNCTION_ATOM(AVM1_swapDepths);
 	ASFUNCTION_ATOM(AVM1_getDepth);
+	ASFUNCTION_ATOM(AVM1_toString);
 	static void AVM1SetupMethods(Class_base* c);
-	DisplayObject* AVM1GetClipFromPath(tiny_string& path);
+	DisplayObject* AVM1GetClipFromPath(tiny_string& path, lightspark::asAtom* member=nullptr);
 	void AVM1SetVariable(tiny_string& name, asAtom v, bool setMember=true);
 	void AVM1SetVariableDirect(uint32_t nameId, asAtom v);
 	asAtom AVM1GetVariable(const tiny_string &name, bool checkrootvars=true);
@@ -351,7 +375,7 @@ public:
 	void AVM1SetFunction(const tiny_string& name, _NR<AVM1Function> obj);
 	AVM1Function *AVM1GetFunction(uint32_t nameID);
 	virtual void AVM1AfterAdvance() {}
-	void DrawToBitmap(BitmapData* bm, const MATRIX& initialMatrix, bool smoothing, bool forcachedbitmap, AS_BLENDMODE blendMode, ColorTransform* ct);
+	DisplayObject* AVM1getRoot();
 	std::string toDebugString() const override;
 };
 }
