@@ -32,8 +32,8 @@ class GraphicsEndFill: public ASObject, public IGraphicsFill, public IGraphicsDa
 public:
 	GraphicsEndFill(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base* c);
-	FILLSTYLE toFillStyle();
-	void appendToTokens(std::vector<uint64_t>& tokens,Graphics* graphics);
+	FILLSTYLE toFillStyle() override;
+	void appendToTokens(tokensVector& tokens, Graphics* graphics) override;
 };
 
 }
