@@ -24,7 +24,6 @@
 #include "swftypes.h"
 #include "scripting/flash/events/flashevents.h"
 #include "scripting/flash/display/BitmapContainer.h"
-#include "platforms/engineutils.h"
 
 namespace lightspark
 {
@@ -36,20 +35,35 @@ protected:
 	uint32_t textureID;
 	uint32_t width;
 	uint32_t height;
+	uint32_t maxmiplevel;
+	uint32_t textureframebuffer;
+	uint32_t depthRenderBuffer;
+	uint32_t stencilRenderBuffer;
+	bool async;
 	TEXTUREFORMAT format;
 	TEXTUREFORMAT_COMPRESSED compressedformat;
 	vector<vector<uint8_t>> bitmaparray;
 	Context3D* context;
+	void fillFromDXT1(bool hasrgbdata, uint32_t level, uint32_t w, uint32_t h, std::vector<uint8_t>& rgbdata, std::vector<uint8_t>& rgbimagedata);
+	void fillFromDXT5(bool hasalphadata, bool hasrgbdata, uint32_t level, uint32_t w, uint32_t h, std::vector<uint8_t>& alphadata, std::vector<uint8_t>& alphaimagedata, std::vector<uint8_t>& rgbdata, std::vector<uint8_t>& rgbimagedata);
 	void parseAdobeTextureFormat(ByteArray* data, int32_t byteArrayOffset, bool forCubeTexture);
 	void setFormat(const tiny_string& f);
+	uint32_t getBytesNeeded(uint32_t miplevel);
+	void uploadFromBitmapDataIntern(BitmapData* source, uint32_t miplevel, uint32_t side=0, uint32_t max_miplevel=0);
+	void uploadFromByteArrayIntern(ByteArray* source, uint32_t offset, uint32_t miplevel);
 public:
 	TextureBase(ASWorker* wrk,Class_base* c):EventDispatcher(wrk,c)
-	  ,textureID(UINT32_MAX),width(0),height(0),format(BGRA),compressedformat(UNCOMPRESSED),context(nullptr)
+		,textureID(UINT32_MAX),width(0),height(0),maxmiplevel(0)
+		,textureframebuffer(UINT32_MAX),depthRenderBuffer(UINT32_MAX),stencilRenderBuffer(UINT32_MAX)
+		,async(false),format(BGRA),compressedformat(UNCOMPRESSED),context(nullptr)
 	{ subtype = SUBTYPE_TEXTUREBASE;}
 	TextureBase(ASWorker* wrk,Class_base* c,Context3D* _context):EventDispatcher(wrk,c)
-	  ,textureID(UINT32_MAX),width(0),height(0),format(BGRA),compressedformat(UNCOMPRESSED),context(_context)
+		,textureID(UINT32_MAX),width(0),height(0),maxmiplevel(0)
+		,textureframebuffer(UINT32_MAX),depthRenderBuffer(UINT32_MAX),stencilRenderBuffer(UINT32_MAX)
+		,async(false),format(BGRA),compressedformat(UNCOMPRESSED),context(_context)
 	{ subtype = SUBTYPE_TEXTUREBASE;}
 	static void sinit(Class_base* c);
+	bool destruct() override;
 	ASFUNCTION_ATOM(dispose);
 };
 
