@@ -31,15 +31,20 @@ class Array;
 class ContextMenu : public EventDispatcher
 {
 public:
-	ContextMenu(Class_base* c);
+	ContextMenu(ASWorker* wrk,Class_base* c);
 	static void sinit(Class_base* c);
+	bool destruct() override;
+	void prepareShutdown() override;
+	bool countCylicMemberReferences(garbagecollectorstate& gcstate) override;
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(hideBuiltInItems);
+	ASFUNCTION_ATOM(clone);
 	ASPROPERTY_GETTER(bool,isSupported);
 	ASPROPERTY_GETTER_SETTER(_NR<Array>,customItems);
 	ASPROPERTY_GETTER_SETTER(_NR<ContextMenuBuiltInItems>,builtInItems);
 	void getCurrentContextMenuItems(std::vector<_R<NativeMenuItem>>& items);
-	static void getVisibleBuiltinContextMenuItems(ContextMenu* m, std::vector<Ref<NativeMenuItem> > &items, SystemState *sys);
+	InteractiveObject* owner;
+	static void getVisibleBuiltinContextMenuItems(ContextMenu* m, std::vector<Ref<NativeMenuItem> > &items, ASWorker* worker);
 };
 
 }
