@@ -24,13 +24,16 @@
 #include "compat.h"
 #include "parsing/amf3_generator.h"
 #include "scripting/argconv.h"
+#include "scripting/toplevel/Array.h"
 #include "scripting/toplevel/Number.h"
 #include "scripting/toplevel/Integer.h"
 #include "scripting/toplevel/UInteger.h"
+#include "scripting/toplevel/Undefined.h"
 #include "scripting/flash/errors/flasherrors.h"
 #include <sstream>
 #include <zlib.h>
 #include <glib.h>
+#include <lzma.h>
 
 using namespace std;
 using namespace lightspark;
@@ -41,7 +44,8 @@ using namespace lightspark;
 // maybe we should set this smaller
 #define BA_MAX_SIZE 0x40000000
 
-ByteArray::ByteArray(ASWorker* wrk, Class_base* c, uint8_t* b, uint32_t l):ASObject(wrk,c,T_OBJECT,SUBTYPE_BYTEARRAY),littleEndian(false),objectEncoding(OBJECT_ENCODING::AMF3),currentObjectEncoding(OBJECT_ENCODING::AMF3),
+ByteArray::ByteArray(ASWorker* wrk, Class_base* c, uint8_t* b, uint32_t l):ASObject(wrk,c,T_OBJECT,SUBTYPE_BYTEARRAY),littleEndian(false),
+	objectEncoding(OBJECT_ENCODING::AMF3),currentObjectEncoding(OBJECT_ENCODING::AMF3),
 	position(0),bytes(b),real_len(l),len(l),shareable(false)
 {
 #ifdef MEMORY_USAGE_PROFILING
@@ -72,6 +76,7 @@ bool ByteArray::destruct()
 		bytes = nullptr;
 	}
 	currentObjectEncoding = OBJECT_ENCODING::AMF3;
+	objectEncoding = OBJECT_ENCODING::AMF3;
 	position = 0;
 	real_len = 0;
 	len = 0;
@@ -96,55 +101,55 @@ void ByteArray::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_SEALED);
 	c->isReusable=true;
-	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_getLength,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_setLength),SETTER_METHOD,true);
-	c->setDeclaredMethodByQName("bytesAvailable","",Class<IFunction>::getFunction(c->getSystemState(),_getBytesAvailable,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("position","",Class<IFunction>::getFunction(c->getSystemState(),_getPosition,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("position","",Class<IFunction>::getFunction(c->getSystemState(),_setPosition),SETTER_METHOD,true);
-	c->setDeclaredMethodByQName("endian","",Class<IFunction>::getFunction(c->getSystemState(),_getEndian,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("endian","",Class<IFunction>::getFunction(c->getSystemState(),_setEndian),SETTER_METHOD,true);
-	c->setDeclaredMethodByQName("objectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_getObjectEncoding,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("objectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_setObjectEncoding),SETTER_METHOD,true);
-	c->setDeclaredMethodByQName("defaultObjectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_getDefaultObjectEncoding,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,false);
-	c->setDeclaredMethodByQName("defaultObjectEncoding","",Class<IFunction>::getFunction(c->getSystemState(),_setDefaultObjectEncoding),SETTER_METHOD,false);
+	c->setDeclaredMethodByQName("length","",c->getSystemState()->getBuiltinFunction(_getLength,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("length","",c->getSystemState()->getBuiltinFunction(_setLength),SETTER_METHOD,true);
+	c->setDeclaredMethodByQName("bytesAvailable","",c->getSystemState()->getBuiltinFunction(_getBytesAvailable,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("position","",c->getSystemState()->getBuiltinFunction(_getPosition,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("position","",c->getSystemState()->getBuiltinFunction(_setPosition),SETTER_METHOD,true);
+	c->setDeclaredMethodByQName("endian","",c->getSystemState()->getBuiltinFunction(_getEndian,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("endian","",c->getSystemState()->getBuiltinFunction(_setEndian),SETTER_METHOD,true);
+	c->setDeclaredMethodByQName("objectEncoding","",c->getSystemState()->getBuiltinFunction(_getObjectEncoding,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("objectEncoding","",c->getSystemState()->getBuiltinFunction(_setObjectEncoding),SETTER_METHOD,true);
+	c->setDeclaredMethodByQName("defaultObjectEncoding","",c->getSystemState()->getBuiltinFunction(_getDefaultObjectEncoding,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,false);
+	c->setDeclaredMethodByQName("defaultObjectEncoding","",c->getSystemState()->getBuiltinFunction(_setDefaultObjectEncoding),SETTER_METHOD,false);
 
 	c->getSystemState()->staticByteArrayDefaultObjectEncoding = OBJECT_ENCODING::DEFAULT;
-	c->setDeclaredMethodByQName("clear","",Class<IFunction>::getFunction(c->getSystemState(),clear),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("compress","",Class<IFunction>::getFunction(c->getSystemState(),_compress),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("uncompress","",Class<IFunction>::getFunction(c->getSystemState(),_uncompress),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("deflate","",Class<IFunction>::getFunction(c->getSystemState(),_deflate),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("inflate","",Class<IFunction>::getFunction(c->getSystemState(),_inflate),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readBoolean","",Class<IFunction>::getFunction(c->getSystemState(),readBoolean,0,Class<Boolean>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readBytes","",Class<IFunction>::getFunction(c->getSystemState(),readBytes),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readByte","",Class<IFunction>::getFunction(c->getSystemState(),readByte,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readDouble","",Class<IFunction>::getFunction(c->getSystemState(),readDouble,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readFloat","",Class<IFunction>::getFunction(c->getSystemState(),readFloat,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readInt","",Class<IFunction>::getFunction(c->getSystemState(),readInt,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readMultiByte","",Class<IFunction>::getFunction(c->getSystemState(),readMultiByte,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readShort","",Class<IFunction>::getFunction(c->getSystemState(),readShort,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readUnsignedByte","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedByte,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedInt,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readUnsignedShort","",Class<IFunction>::getFunction(c->getSystemState(),readUnsignedShort,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readObject","",Class<IFunction>::getFunction(c->getSystemState(),readObject,0,Class<ASObject>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readUTF","",Class<IFunction>::getFunction(c->getSystemState(),readUTF,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("readUTFBytes","",Class<IFunction>::getFunction(c->getSystemState(),readUTFBytes,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeBoolean","",Class<IFunction>::getFunction(c->getSystemState(),writeBoolean),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeUTF","",Class<IFunction>::getFunction(c->getSystemState(),writeUTF),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeUTFBytes","",Class<IFunction>::getFunction(c->getSystemState(),writeUTFBytes),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeBytes","",Class<IFunction>::getFunction(c->getSystemState(),writeBytes),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeByte","",Class<IFunction>::getFunction(c->getSystemState(),writeByte),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeDouble","",Class<IFunction>::getFunction(c->getSystemState(),writeDouble),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeFloat","",Class<IFunction>::getFunction(c->getSystemState(),writeFloat),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeInt","",Class<IFunction>::getFunction(c->getSystemState(),writeInt),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeMultiByte","",Class<IFunction>::getFunction(c->getSystemState(),writeMultiByte),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeUnsignedInt","",Class<IFunction>::getFunction(c->getSystemState(),writeUnsignedInt),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeObject","",Class<IFunction>::getFunction(c->getSystemState(),writeObject),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("writeShort","",Class<IFunction>::getFunction(c->getSystemState(),writeShort),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),ByteArray::_toString),DYNAMIC_TRAIT);
+	c->setDeclaredMethodByQName("clear","",c->getSystemState()->getBuiltinFunction(clear),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("compress","",c->getSystemState()->getBuiltinFunction(_compress),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("uncompress","",c->getSystemState()->getBuiltinFunction(_uncompress),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("deflate","",c->getSystemState()->getBuiltinFunction(_deflate),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("inflate","",c->getSystemState()->getBuiltinFunction(_inflate),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readBoolean","",c->getSystemState()->getBuiltinFunction(readBoolean,0,Class<Boolean>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readBytes","",c->getSystemState()->getBuiltinFunction(readBytes),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readByte","",c->getSystemState()->getBuiltinFunction(readByte,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readDouble","",c->getSystemState()->getBuiltinFunction(readDouble,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readFloat","",c->getSystemState()->getBuiltinFunction(readFloat,0,Class<Number>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readInt","",c->getSystemState()->getBuiltinFunction(readInt,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readMultiByte","",c->getSystemState()->getBuiltinFunction(readMultiByte,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readShort","",c->getSystemState()->getBuiltinFunction(readShort,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUnsignedByte","",c->getSystemState()->getBuiltinFunction(readUnsignedByte,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUnsignedInt","",c->getSystemState()->getBuiltinFunction(readUnsignedInt,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUnsignedShort","",c->getSystemState()->getBuiltinFunction(readUnsignedShort,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readObject","",c->getSystemState()->getBuiltinFunction(readObject,0,Class<ASObject>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUTF","",c->getSystemState()->getBuiltinFunction(readUTF,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("readUTFBytes","",c->getSystemState()->getBuiltinFunction(readUTFBytes,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeBoolean","",c->getSystemState()->getBuiltinFunction(writeBoolean),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeUTF","",c->getSystemState()->getBuiltinFunction(writeUTF),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeUTFBytes","",c->getSystemState()->getBuiltinFunction(writeUTFBytes),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeBytes","",c->getSystemState()->getBuiltinFunction(writeBytes),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeByte","",c->getSystemState()->getBuiltinFunction(writeByte),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeDouble","",c->getSystemState()->getBuiltinFunction(writeDouble),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeFloat","",c->getSystemState()->getBuiltinFunction(writeFloat),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeInt","",c->getSystemState()->getBuiltinFunction(writeInt),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeMultiByte","",c->getSystemState()->getBuiltinFunction(writeMultiByte),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeUnsignedInt","",c->getSystemState()->getBuiltinFunction(writeUnsignedInt),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeObject","",c->getSystemState()->getBuiltinFunction(writeObject),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("writeShort","",c->getSystemState()->getBuiltinFunction(writeShort),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toString","",c->getSystemState()->getBuiltinFunction(ByteArray::_toString),DYNAMIC_TRAIT);
 	REGISTER_GETTER_SETTER_RESULTTYPE(c,shareable,Boolean);
-	c->setDeclaredMethodByQName("atomicCompareAndSwapIntAt","",Class<IFunction>::getFunction(c->getSystemState(),atomicCompareAndSwapIntAt),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("atomicCompareAndSwapLength","",Class<IFunction>::getFunction(c->getSystemState(),atomicCompareAndSwapLength),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toJSON",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toJSON),DYNAMIC_TRAIT);
+	c->setDeclaredMethodByQName("atomicCompareAndSwapIntAt","",c->getSystemState()->getBuiltinFunction(atomicCompareAndSwapIntAt),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("atomicCompareAndSwapLength","",c->getSystemState()->getBuiltinFunction(atomicCompareAndSwapLength),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toJSON",AS3,c->getSystemState()->getBuiltinFunction(_toJSON),DYNAMIC_TRAIT);
 
 	c->addImplementedInterface(InterfaceClass<IDataInput>::getClass(c->getSystemState()));
 	IDataInput::linkTraits(c);
@@ -206,6 +211,9 @@ uint8_t* ByteArray::getBufferIntern(unsigned int size, bool enableResize)
 
 ASFUNCTIONBODY_ATOM(ByteArray,_constructor)
 {
+	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
+	th->currentObjectEncoding = wrk->getSystemState()->staticByteArrayDefaultObjectEncoding;
+	th->objectEncoding = wrk->getSystemState()->staticByteArrayDefaultObjectEncoding;
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_getPosition)
@@ -264,7 +272,7 @@ ASFUNCTIONBODY_ATOM(ByteArray,_setObjectEncoding)
 
 ASFUNCTIONBODY_ATOM(ByteArray,_getDefaultObjectEncoding)
 {
-	asAtomHandler::setUInt(ret,wrk,wrk->getSystemState()->staticNetConnectionDefaultObjectEncoding);
+	asAtomHandler::setUInt(ret,wrk,wrk->getSystemState()->staticByteArrayDefaultObjectEncoding);
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_setDefaultObjectEncoding)
@@ -311,7 +319,7 @@ void ByteArray::setLength(uint32_t newLen)
 	}
 	len = newLen;
 	if (position > len)
-		position = (len > 0 ? len-1 : 0);
+		position = (len > 0 ? len : 0);
 }
 ASFUNCTIONBODY_ATOM(ByteArray,_getLength)
 {
@@ -478,7 +486,7 @@ asAtom ByteArray::readObject()
 	}
 	catch(LightsparkException& e)
 	{
-		LOG(LOG_ERROR,"Exception caught while parsing AMF3: " << e.cause);
+		LOG(LOG_ERROR,"Exception caught while parsing AMF: " << e.cause);
 		//TODO: throw AS exception
 	}
 	return ret;
@@ -486,7 +494,7 @@ asAtom ByteArray::readObject()
 
 ASObject* ByteArray::readSharedObject()
 {
-	ASObject* ret = Class<ASObject>::getInstanceS(getInstanceWorker());
+	ASObject* ret = new_asobject(getInstanceWorker());
 	if (getLength()<=16)
 		return ret;
 	Amf3Deserializer d(this);
@@ -547,13 +555,28 @@ ASFUNCTIONBODY_ATOM(ByteArray,writeMultiByte)
 	tiny_string charset;
 	ARG_CHECK(ARG_UNPACK(value)(charset));
 
-	// TODO: should convert from UTF-8 to charset
-	LOG(LOG_NOT_IMPLEMENTED, "ByteArray.writeMultiByte doesn't convert charset");
-
 	th->lock();
-	th->getBuffer(th->position+value.numBytes(),true);
-	memcpy(th->bytes+th->position,value.raw_buf(),value.numBytes());
-	th->position+=value.numBytes();
+	if (charset != "utf-8")
+	{
+		gsize bytes_read;
+		gsize bytes_written;
+		gchar* v = g_convert(value.raw_buf(),value.numBytes(),charset.raw_buf(),"utf-8",&bytes_read,&bytes_written,nullptr);
+		if (v)
+		{
+			th->getBuffer(th->position+bytes_written,true);
+			memcpy(th->bytes+th->position,v,bytes_written);
+			th->position+=bytes_written;
+			g_free(v);
+		}
+		else
+			LOG(LOG_ERROR,"ByteArray.writeMultiByte charset conversion failed");
+	}
+	else
+	{
+		th->getBuffer(th->position+value.numBytes(),true);
+		memcpy(th->bytes+th->position,value.raw_buf(),value.numBytes());
+		th->position+=value.numBytes();
+	}
 	th->unlock();
 }
 
@@ -796,9 +819,9 @@ bool ByteArray::readU29(uint32_t& ret)
 		{
 			ret <<=1;
 			ret |= tmp;
-//			//Sign extend
-//			if(tmp&0x80)
-//				ret|=0xe0000000;
+			//Sign extend
+			if(ret&0x10000000)
+				ret|=0xe0000000;
 		}
 	}
 	return true;
@@ -986,13 +1009,26 @@ ASFUNCTIONBODY_ATOM(ByteArray,readMultiByte)
 		return;
 	}
 
-	if (charset != "us-ascii" && charset != "utf-8")
-		LOG(LOG_NOT_IMPLEMENTED, "ByteArray.readMultiByte doesn't convert charset "<<charset);
 	char* s = g_newa(char,strlen+1);
 	// ensure that the resulting string cuts off any zeros at the end
 	strncpy(s,(const char*)th->bytes+th->position,strlen);
 	s[strlen] = 0x0;
-	tiny_string res(s,true);
+	tiny_string res;
+	if (charset != "utf-8" && charset != "utf-8")
+	{
+		gsize bytes_read;
+		gsize bytes_written;
+		gchar* v = g_convert(s,strlen,"utf-8",charset.raw_buf(),&bytes_read,&bytes_written,nullptr);
+		if (v)
+		{
+			res = tiny_string(v,true);
+			g_free(v);
+		}
+		else
+			LOG(LOG_ERROR,"ByteArray.readMultiByte charset conversion failed");
+	}
+	else
+		res = tiny_string(s,true);
 	th->unlock();
 	ret = asAtomHandler::fromObject(abstract_s(wrk,res));
 }
@@ -1165,7 +1201,11 @@ ASFUNCTIONBODY_ATOM(ByteArray,_toString)
 			start = 3;
 	}
 	if (validateUtf8(th->bytes+start,th->len-start))
-		ret = asAtomHandler::fromObject(abstract_s(wrk,(const char*)th->bytes+start,th->len-start));
+	{
+		// use std::string to properly handle \0 inside bytes
+		tiny_string s = std::string((const char*)th->bytes+start,th->len-start);
+		ret = asAtomHandler::fromObject(abstract_s(wrk,s));
+	}
 	else
 		ret = asAtomHandler::fromObject(abstract_s(wrk,parseUtf8(th->bytes+start,th->len-start)));
 }
@@ -1177,7 +1217,7 @@ bool ByteArray::hasPropertyByMultiname(const multiname& name, bool considerDynam
 	if (!isConstructed())
 		return false;
 	unsigned int index=0;
-	if(!Array::isValidMultiname(getSystemState(),name,index))
+	if(!Array::isValidMultiname(getInstanceWorker(),name,index))
 		return ASObject::hasPropertyByMultiname(name, considerDynamic, considerPrototype,wrk);
 
 	return index<len;
@@ -1186,7 +1226,7 @@ bool ByteArray::hasPropertyByMultiname(const multiname& name, bool considerDynam
 GET_VARIABLE_RESULT ByteArray::getVariableByMultiname(asAtom& ret, const multiname& name, GET_VARIABLE_OPTION opt, ASWorker* wrk)
 {
 	unsigned int index=0;
-	if((opt & GET_VARIABLE_OPTION::SKIP_IMPL)!=0  || !implEnable || !Array::isValidMultiname(getSystemState(),name,index))
+	if((opt & GET_VARIABLE_OPTION::SKIP_IMPL)!=0  || !implEnable || !Array::isValidMultiname(getInstanceWorker(),name,index))
 	{
 		return getVariableByMultinameIntern(ret,name,this->getClass(),opt,wrk);
 	}
@@ -1221,7 +1261,7 @@ int32_t ByteArray::getVariableByMultiname_i(const multiname& name, ASWorker* wrk
 {
 	assert_and_throw(implEnable);
 	unsigned int index=0;
-	if(!Array::isValidMultiname(getSystemState(),name,index))
+	if(!Array::isValidMultiname(getInstanceWorker(),name,index))
 		return ASObject::getVariableByMultiname_i(name,wrk);
 
 	if(index<len)
@@ -1239,7 +1279,7 @@ multiname *ByteArray::setVariableByMultiname(multiname& name, asAtom& o, CONST_A
 {
 	assert_and_throw(implEnable);
 	unsigned int index=0;
-	if(!Array::isValidMultiname(getSystemState(),name,index))
+	if(!Array::isValidMultiname(getInstanceWorker(),name,index))
 		return ASObject::setVariableByMultiname(name,o,allowConst,alreadyset,wrk);
 	if (index > BA_MAX_SIZE) 
 	{
@@ -1325,7 +1365,6 @@ void ByteArray::writeU29(uint32_t val)
 		}
 		else
 			b=val&0x7f;
-
 		writeByte(b);
 	}
 }
@@ -1433,9 +1472,18 @@ void ByteArray::append(streambuf *data, int length)
 	s.read((char*)bytes+oldlen,length);
 	unlock();
 }
+void ByteArray::append(uint8_t* data, int length)
+{
+	lock();
+	int oldlen = len;
+	getBuffer(len+length,true);
+	memcpy(bytes+oldlen,data,length);
+	unlock();
+}
 void ByteArray::removeFrontBytes(int count)
 {
-	memmove(bytes,bytes+count,count);
+	if (count < (int)len)
+		memmove(bytes,bytes+count,len-count);
 	position -= count;
 	len -= count;
 }
@@ -1496,11 +1544,14 @@ void ByteArray::uncompress_zlib(bool raw)
 	status=inflateInit2(&strm,raw ? -15 : 15);
 	if(status==Z_VERSION_ERROR)
 	{
-		createError<IOError>(getInstanceWorker(),0,"not valid compressed data");
+		createError<IOError>(getInstanceWorker(),kCompressedDataError);
 		return;
 	}
 	else if(status!=Z_OK)
-		throw RunTimeException("zlib uncompress failed");
+	{
+		createError<IOError>(getInstanceWorker(),kCompressedDataError);
+		return;
+	}
 
 	vector<uint8_t> buf(3*len);
 	do
@@ -1512,7 +1563,7 @@ void ByteArray::uncompress_zlib(bool raw)
 		if(status!=Z_OK && status!=Z_STREAM_END)
 		{
 			inflateEnd(&strm);
-			createError<IOError>(getInstanceWorker(),0,"not valid compressed data");
+			createError<IOError>(getInstanceWorker(),kCompressedDataError);
 			return;
 		}
 
@@ -1534,26 +1585,147 @@ void ByteArray::uncompress_zlib(bool raw)
 	memcpy(bytes, &buf[0], len);
 	position=0;
 }
+void ByteArray::compress_lzma()
+{
+	if(len==0)
+		return;
+	lzma_stream strm = LZMA_STREAM_INIT;
+	lzma_options_lzma opt;
+	lzma_lzma_preset(&opt,0);
+	lzma_ret ret = lzma_alone_encoder(&strm, &opt);
+	if (ret != LZMA_OK)
+	{
+		LOG(LOG_ERROR,"Failed to initialize lzma encoder in ByteArray::compress_lzma");
+		createError<IOError>(getInstanceWorker(),kCompressedDataError);
+		return;
+	}
+	uint32_t inputlen = getLength();
+	uint32_t outputlen = BA_CHUNK_SIZE;
+	uint8_t* inbuffer = new uint8_t[inputlen];
+	memcpy(inbuffer,getBufferNoCheck(),inputlen);
+	strm.next_in = inbuffer;
+	strm.avail_in = inputlen;
+	strm.avail_out = outputlen;
+	this->setLength(outputlen);
+	strm.next_out = this->getBufferNoCheck();
+	
+	bool abort=false;
+	while (strm.avail_in!=0 && !abort)
+	{
+		do
+		{
+			lzma_ret ret=lzma_code(&strm, LZMA_FINISH);
+			
+			if(ret==LZMA_STREAM_END || strm.avail_in==0)
+				break;
+			else if(ret!=LZMA_OK)
+			{
+				LOG(LOG_ERROR,"lzma decoder error:"<<ret);
+				createError<IOError>(getInstanceWorker(),kCompressedDataError);
+				abort=true;
+				break;
+			}
+			if (strm.avail_out==0)
+			{
+				uint32_t oldlen = getLength();
+				strm.avail_out=outputlen;
+				setLength(getLength()+outputlen);
+				strm.next_out=getBufferNoCheck()+oldlen;
+			}
+		}
+		while(strm.avail_out!=0);
+	}
+	this->setLength(strm.total_out);
+	lzma_end(&strm);
+	setPosition(0);
+	delete[] inbuffer;
+}
+void ByteArray::uncompress_lzma()
+{
+	if(len==0)
+		return;
+	lzma_stream strm = LZMA_STREAM_INIT;
+	lzma_ret ret = lzma_alone_decoder(&strm, UINT64_MAX);
+	if (ret != LZMA_OK)
+	{
+		LOG(LOG_ERROR,"Failed to initialize lzma decoder in ByteArray::uncompress_lzma");
+		createError<IOError>(getInstanceWorker(),kCompressedDataError);
+		return;
+	}
+	uint32_t inputlen = getLength();
+	uint32_t outputlen = getLength();// TODO: I don't know what's the best value for chunk size, we just use the length of the input for now
+	uint8_t* inbuffer = new uint8_t[inputlen];
+	memcpy(inbuffer,getBufferNoCheck(),inputlen);
+	strm.next_in = inbuffer;
+	strm.avail_in = inputlen;
+	strm.avail_out = outputlen;
+	strm.next_out = this->getBufferNoCheck();
+	bool abort=false;
+	while (strm.avail_in!=0 && !abort)
+	{
+		do
+		{
+			lzma_ret ret=lzma_code(&strm, LZMA_RUN);
+			
+			if(ret==LZMA_STREAM_END)
+				break;
+			else if(ret!=LZMA_OK)
+			{
+				LOG(LOG_ERROR,"lzma decoder error:"<<ret);
+				createError<IOError>(getInstanceWorker(),kCompressedDataError);
+				this->setLength(inputlen);
+				memcpy(getBufferNoCheck(),inbuffer,inputlen);
+				abort=true;
+				break;
+			}
+			if (strm.avail_out==0)
+			{
+				uint32_t oldlen = getLength();
+				strm.avail_out=outputlen;
+				setLength(getLength()+outputlen);
+				strm.next_out=getBufferNoCheck()+oldlen;
+			}
+			if(strm.avail_in==0)
+				break;
+		}
+		while(strm.avail_out!=0);
+	}
+	if (!abort && strm.avail_out!=0)
+	{
+		this->setLength(this->getLength()-strm.avail_out);
+	}
+	lzma_end(&strm);
+	setPosition(0);
+	delete[] inbuffer;
+}
 
 ASFUNCTIONBODY_ATOM(ByteArray,_compress)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	// flash throws an error if compress is called with a compression algorithm,
-	// and always uses the zlib algorithm
-	// but tamarin tests do not catch it, so we simply ignore any parameters provided
+	tiny_string algorithm;
+	ARG_CHECK(ARG_UNPACK(algorithm,"zlib"));
 	th->lock();
-	th->compress_zlib(false);
+	if (algorithm == "lzma")
+		th->compress_lzma();
+	else if (algorithm == "deflate")
+		th->compress_zlib(true);
+	else
+		th->compress_zlib(false);
 	th->unlock();
 }
 
 ASFUNCTIONBODY_ATOM(ByteArray,_uncompress)
 {
 	ByteArray* th=asAtomHandler::as<ByteArray>(obj);
-	// flash throws an error if uncompress is called with a compression algorithm,
-	// and always uses the zlib algorithm
-	// but tamarin tests do not catch it, so we simply ignore any parameters provided
+	tiny_string algorithm;
+	ARG_CHECK(ARG_UNPACK(algorithm,"zlib"));
 	th->lock();
-	th->uncompress_zlib(false);
+	if (algorithm == "lzma")
+		th->uncompress_lzma();
+	else if (algorithm == "deflate")
+		th->uncompress_zlib(true);
+	else
+		th->uncompress_zlib(false);
 	th->unlock();
 }
 
