@@ -18,7 +18,6 @@
 **************************************************************************/
 
 #include "scripting/toplevel/Boolean.h"
-#include "scripting/toplevel/toplevel.h"
 #include "scripting/toplevel/Number.h"
 #include "scripting/toplevel/Integer.h"
 #include "scripting/toplevel/UInteger.h"
@@ -51,10 +50,14 @@ bool lightspark::Boolean_concrete(const ASObject* o)
 			return false;
 		return !o->as<ASString>()->isEmpty();
 	case T_FUNCTION:
+		// not constructed functions return false
+		if (!o->isConstructed())
+			return false;
+		return true;
 	case T_ARRAY:
 	case T_OBJECT:
-		// not constructed objects return false
-		if (!o->isConstructed())
+		// not initialized objects return false
+		if (!o->isInitialized())
 			return false;
 		return true;
 	default:
@@ -76,9 +79,9 @@ void Boolean::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_SEALED | CLASS_FINAL);
 	c->isReusable = true;
-	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toString),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),_toString),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("valueOf","",Class<IFunction>::getFunction(c->getSystemState(),_valueOf),DYNAMIC_TRAIT);
+	c->setDeclaredMethodByQName("toString",AS3,c->getSystemState()->getBuiltinFunction(_toString),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toString","",c->getSystemState()->getBuiltinFunction(_toString),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("valueOf","",c->getSystemState()->getBuiltinFunction(_valueOf),DYNAMIC_TRAIT);
 }
 
 ASFUNCTIONBODY_ATOM(Boolean,_constructor)
