@@ -20,9 +20,9 @@
 #include <sstream>
 #include "scripting/toplevel/Error.h"
 #include "asobject.h"
-#include "scripting/toplevel/toplevel.h"
 #include "scripting/toplevel/Integer.h"
 #include "scripting/flash/system/flashsystem.h"
+#include "scripting/flash/display/RootMovieClip.h"
 #include "scripting/class.h"
 #include "scripting/argconv.h"
 #include "scripting/abc.h"
@@ -30,6 +30,10 @@
 using namespace std;
 using namespace lightspark;
 
+tiny_string lightspark::createErrorMessage(ASWorker* wrk, int errorID, asAtom arg1, const tiny_string& arg2, const tiny_string& arg3)
+{
+	return createErrorMessage(errorID,asAtomHandler::toErrorString(arg1,wrk),arg2,arg3);
+}
 tiny_string lightspark::createErrorMessage(int errorID, const tiny_string& arg1, const tiny_string& arg2, const tiny_string& arg3)
 {
 	auto found = errorMessages.find(errorID);
@@ -93,6 +97,8 @@ tiny_string lightspark::createErrorMessage(int errorID, const tiny_string& arg1,
 //	}
 //	Log::setLogLevel(LOG_CALLS);
 	LOG(LOG_INFO,"throwing exception:"<<getWorker()<<" id:"<<errorID<<" "<<msg.str());
+//	getWorker()->dumpStacktrace();
+//	getWorker()->rootClip->dumpDisplayList();
 	return msg.str();
 }
 void lightspark::setError(ASObject* error)
@@ -104,35 +110,44 @@ void lightspark::setError(ASObject* error)
 		else
 			error->getInstanceWorker()->currentCallContext->exceptionthrown = error;
 	}
+	else if  (!error->getInstanceWorker()->needsActionScript3())
+	{
+		if (error->getInstanceWorker()->AVM1callStack.back()->exceptionthrown)
+			error->decRef();
+		else
+			error->getInstanceWorker()->AVM1callStack.back()->exceptionthrown = error;
+	}
 	else
 		throw error;
 }
 
-ASError::ASError(ASWorker* wrk,Class_base* c, const tiny_string& error_message, int id, const tiny_string& error_name):
-	ASObject(wrk,c),errorID(id),name(error_name),message(error_message)
+ASError::ASError(ASWorker* wrk, Class_base* c, const tiny_string& error_message, int id, const tiny_string& error_name, CLASS_SUBTYPE subtype):
+	ASObject(wrk,c,T_OBJECT,subtype),errorID(id),name(error_name),message(error_message)
 {
-	stacktrace = "";
 	for (uint32_t i = wrk->cur_recursion; i > 0; i--)
 	{
-		stacktrace += "    at ";
-		stacktrace += asAtomHandler::toObject(wrk->stacktrace[i-1].object,wrk)->getClassName();
-		stacktrace += "/";
-		stacktrace += c->getSystemState()->getStringFromUniqueId(wrk->stacktrace[i-1].name);
-		stacktrace += "()\n";
+		ASObject* o = asAtomHandler::toObject(wrk->stacktrace[i-1].object,wrk);
+		stacktrace.push_back(make_pair(o->getClass() ? o->getClass()->getQualifiedClassNameID() : (uint32_t)BUILTIN_STRINGS::EMPTY,wrk->stacktrace[i-1].name));
 	}
 }
 
 ASFUNCTIONBODY_ATOM(ASError,_getStackTrace)
 {
 	ASError* th=asAtomHandler::as<ASError>(obj);
-
 	ret = asAtomHandler::fromObject(abstract_s(wrk,th->getStackTraceString()));
 }
 tiny_string ASError::getStackTraceString()
 {
 	tiny_string ret = toString();
 	ret += "\n";
-	ret += stacktrace;
+	for (auto it = stacktrace.begin(); it != stacktrace.end(); it++)
+	{
+		ret += "    at ";
+		ret += getSystemState()->getStringFromUniqueId((*it).first);
+		ret += "/";
+		ret += getSystemState()->getStringFromUniqueId((*it).second);
+		ret += "()\n";
+	}
 	return ret;
 }
 
@@ -140,7 +155,7 @@ tiny_string ASError::toString()
 {
 	tiny_string ret;
 	ret = name;
-	if(errorID != 0)
+	if(errorID != 0 && !message.startsWith("Error #"))
 		ret += tiny_string(": Error #") + Integer::toString(errorID);
 	if (!message.empty())
 		ret += tiny_string(": ") + message;
@@ -176,7 +191,6 @@ ASFUNCTIONBODY_ATOM(ASError,generator)
 
 void ASError::errorGenerator(ASError* obj, asAtom* args, const unsigned int argslen)
 {
-	assert_and_throw(argslen <= 2);
 	if(argslen >= 1)
 	{
 		obj->message = asAtomHandler::toString(args[0],obj->getInstanceWorker());
@@ -190,25 +204,20 @@ void ASError::errorGenerator(ASError* obj, asAtom* args, const unsigned int args
 void ASError::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_DYNAMIC_NOT_FINAL);
-	c->setDeclaredMethodByQName("getStackTrace","",Class<IFunction>::getFunction(c->getSystemState(),_getStackTrace),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),_toString),DYNAMIC_TRAIT);
-	c->setDeclaredMethodByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),_toString),NORMAL_METHOD,true);
-	REGISTER_GETTER(c, errorID);
-	REGISTER_GETTER_SETTER(c, message);
-	REGISTER_GETTER_SETTER(c, name);
+	c->setDeclaredMethodByQName("getStackTrace","",c->getSystemState()->getBuiltinFunction(_getStackTrace,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toString","",c->getSystemState()->getBuiltinFunction(_toString,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),DYNAMIC_TRAIT);
+	c->setDeclaredMethodByQName("toString","",c->getSystemState()->getBuiltinFunction(_toString,0,Class<ASString>::getRef(c->getSystemState()).getPtr()),NORMAL_METHOD,true);
+	REGISTER_GETTER_RESULTTYPE(c, errorID,Integer);
+	REGISTER_GETTER_SETTER_RESULTTYPE(c, message, ASString);
+	REGISTER_GETTER_SETTER_RESULTTYPE(c, name, ASString);
 }
 
 ASFUNCTIONBODY_GETTER(ASError, errorID)
 ASFUNCTIONBODY_GETTER_SETTER(ASError, message)
 ASFUNCTIONBODY_GETTER_SETTER(ASError, name)
 
-void ASError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(SecurityError,_constructor)
 {
-	assert(argslen<=1);
 	SecurityError* th=asAtomHandler::as<SecurityError>(obj);
 	if(argslen == 1)
 	{
@@ -228,13 +237,8 @@ void SecurityError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void SecurityError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(ArgumentError,_constructor)
 {
-	assert(argslen<=1);
 	ArgumentError* th=asAtomHandler::as<ArgumentError>(obj);
 	if(argslen == 1)
 	{
@@ -254,13 +258,8 @@ void ArgumentError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void ArgumentError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(DefinitionError,_constructor)
 {
-	assert(argslen<=1);
 	DefinitionError* th=asAtomHandler::as<DefinitionError>(obj);
 	if(argslen == 1)
 	{
@@ -280,13 +279,8 @@ void DefinitionError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void DefinitionError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(EvalError,_constructor)
 {
-	assert(argslen<=1);
 	EvalError* th=asAtomHandler::as<EvalError>(obj);
 	if(argslen == 1)
 	{
@@ -306,13 +300,8 @@ void EvalError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void EvalError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(RangeError,_constructor)
 {
-	assert(argslen<=1);
 	RangeError* th=asAtomHandler::as<RangeError>(obj);
 	if(argslen == 1)
 	{
@@ -332,13 +321,8 @@ void RangeError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void RangeError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(ReferenceError,_constructor)
 {
-	assert(argslen<=1);
 	ReferenceError* th=asAtomHandler::as<ReferenceError>(obj);
 	if(argslen == 1)
 	{
@@ -358,13 +342,8 @@ void ReferenceError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void ReferenceError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(SyntaxError,_constructor)
 {
-	assert(argslen<=1);
 	SyntaxError* th=asAtomHandler::as<SyntaxError>(obj);
 	if(argslen == 1)
 	{
@@ -384,13 +363,8 @@ void SyntaxError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void SyntaxError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(TypeError,_constructor)
 {
-	assert(argslen<=1);
 	TypeError* th=asAtomHandler::as<TypeError>(obj);
 	if(argslen == 1)
 	{
@@ -410,13 +384,8 @@ void TypeError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void TypeError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(URIError,_constructor)
 {
-	assert(argslen<=1);
 	URIError* th=asAtomHandler::as<URIError>(obj);
 	if(argslen == 1)
 	{
@@ -436,13 +405,8 @@ void URIError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void URIError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(VerifyError,_constructor)
 {
-	assert(argslen<=1);
 	VerifyError* th=asAtomHandler::as<VerifyError>(obj);
 	if(argslen == 1)
 	{
@@ -462,13 +426,8 @@ void VerifyError::sinit(Class_base* c)
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
 
-void VerifyError::buildTraits(ASObject* o)
-{
-}
-
 ASFUNCTIONBODY_ATOM(UninitializedError,_constructor)
 {
-	assert(argslen<=1);
 	UninitializedError* th=asAtomHandler::as<UninitializedError>(obj);
 	if(argslen == 1)
 	{
@@ -487,7 +446,3 @@ void UninitializedError::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASError, _constructor, CLASS_DYNAMIC_NOT_FINAL);
 }
-
-void UninitializedError::buildTraits(ASObject* o)
-{
-}
