@@ -19,6 +19,9 @@
 
 #include "scripting/argconv.h"
 #include "scripting/toplevel/RegExp.h"
+#include "scripting/toplevel/Array.h"
+#include "scripting/toplevel/Null.h"
+#include "scripting/toplevel/Undefined.h"
 
 using namespace std;
 using namespace lightspark;
@@ -36,16 +39,16 @@ RegExp::RegExp(ASWorker* wrk,Class_base* c, const tiny_string& _re):ASObject(wrk
 void RegExp::sinit(Class_base* c)
 {
 	CLASS_SETUP(c, ASObject, _constructor, CLASS_DYNAMIC_NOT_FINAL);
-	c->setDeclaredMethodByQName("exec","",Class<IFunction>::getFunction(c->getSystemState(),exec),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("exec",AS3,Class<IFunction>::getFunction(c->getSystemState(),exec),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("test","",Class<IFunction>::getFunction(c->getSystemState(),test),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("test",AS3,Class<IFunction>::getFunction(c->getSystemState(),test),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),_toString),NORMAL_METHOD,true);
-	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),_toString),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("exec","",Class<IFunction>::getFunction(c->getSystemState(),exec),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("exec",AS3,Class<IFunction>::getFunction(c->getSystemState(),exec),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("test","",Class<IFunction>::getFunction(c->getSystemState(),test),CONSTANT_TRAIT);
-	c->prototype->setVariableByQName("test",AS3,Class<IFunction>::getFunction(c->getSystemState(),test),CONSTANT_TRAIT);
+	c->setDeclaredMethodByQName("exec","",c->getSystemState()->getBuiltinFunction(exec),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("exec",AS3,c->getSystemState()->getBuiltinFunction(exec),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("test","",c->getSystemState()->getBuiltinFunction(test),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("test",AS3,c->getSystemState()->getBuiltinFunction(test),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("toString","",c->getSystemState()->getBuiltinFunction(_toString),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toString","",c->getSystemState()->getBuiltinFunction(_toString),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("exec","",c->getSystemState()->getBuiltinFunction(exec),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("exec",AS3,c->getSystemState()->getBuiltinFunction(exec),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("test","",c->getSystemState()->getBuiltinFunction(test),CONSTANT_TRAIT);
+	c->prototype->setVariableByQName("test",AS3,c->getSystemState()->getBuiltinFunction(test),CONSTANT_TRAIT);
 	REGISTER_GETTER(c,dotall);
 	REGISTER_GETTER(c,global);
 	REGISTER_GETTER(c,ignoreCase);
@@ -81,37 +84,38 @@ ASFUNCTIONBODY_ATOM(RegExp,_constructor)
 	else if(argslen > 0)
 		th->source=asAtomHandler::toString(args[0],wrk).raw_buf();
 	if(argslen>1 && !asAtomHandler::is<Undefined>(args[1]))
+		th->setFlagsFromAtom(args[1]);
+}
+
+void RegExp::setFlagsFromAtom(asAtom& a)
+{
+	const tiny_string& flags=asAtomHandler::toString(a,getInstanceWorker());
+	for(auto i=flags.begin();i!=flags.end();++i)
 	{
-		const tiny_string& flags=asAtomHandler::toString(args[1],wrk);
-		for(auto i=flags.begin();i!=flags.end();++i)
+		switch(*i)
 		{
-			switch(*i)
-			{
-				case 'g':
-					th->global=true;
-					break;
-				case 'i':
-					th->ignoreCase=true;
-					break;
-				case 'x':
-					th->extended=true;
-					break;
-				case 'm':
-					th->multiline=true;
-					break;
-				case 's':
-					// Defined in the Adobe online
-					// help but not in ECMA
-					th->dotall=true;
-					break;
-				default:
-					break;
-			}
+			case 'g':
+				global=true;
+				break;
+			case 'i':
+				ignoreCase=true;
+				break;
+			case 'x':
+				extended=true;
+				break;
+			case 'm':
+				multiline=true;
+				break;
+			case 's':
+				// Defined in the Adobe online
+				// help but not in ECMA
+				dotall=true;
+				break;
+			default:
+				break;
 		}
 	}
 }
-
-
 ASFUNCTIONBODY_ATOM(RegExp,generator)
 {
 	if(argslen == 0)
@@ -125,9 +129,10 @@ ASFUNCTIONBODY_ATOM(RegExp,generator)
 	}
 	else
 	{
-		if (argslen > 1)
-			LOG(LOG_NOT_IMPLEMENTED, "RegExp generator: flags argument not implemented");
-		ret = asAtomHandler::fromObject(Class<RegExp>::getInstanceS(wrk,asAtomHandler::toString(args[0],wrk)));
+		RegExp* re = Class<RegExp>::getInstanceS(wrk,asAtomHandler::toString(args[0],wrk));
+		if(argslen>1 && !asAtomHandler::is<Undefined>(args[1]))
+			re->setFlagsFromAtom(args[1]);
+		ret = asAtomHandler::fromObject(re);
 	}
 }
 
