@@ -30,19 +30,19 @@ class UInteger: public ASObject
 friend ASObject* abstract_ui(SystemState* sys, uint32_t i);
 public:
 	uint32_t val;
-	UInteger(Class_base* c,uint32_t v=0):ASObject(c,T_UINTEGER),val(v){}
+	UInteger(ASWorker* wrk,Class_base* c,uint32_t v=0):ASObject(wrk,c,T_UINTEGER),val(v){}
 
 	static void sinit(Class_base* c);
-	tiny_string toString();
+	tiny_string toString() const;
 	static tiny_string toString(uint32_t val);
-	inline number_t toNumber() { return val; }
-	inline bool destruct() { val=0; return destructIntern();}
-	inline int32_t toInt() { return val; }
-	inline int64_t toInt64() { return val; }
-	inline uint32_t toUInt() { return val; }
-	TRISTATE isLess(ASObject* r);
-	TRISTATE isLessAtom(asAtom& r);
-	bool isEqual(ASObject* o);
+	inline number_t toNumber() override { return val; }
+	inline bool destruct() override { val=0; return destructIntern();}
+	inline int32_t toInt() override { return val; }
+	inline int64_t toInt64() override { return val; }
+	inline uint32_t toUInt() override { return val; }
+	TRISTATE isLess(ASObject* r) override;
+	TRISTATE isLessAtom(asAtom& r) override;
+	bool isEqual(ASObject* o) override;
 	ASFUNCTION_ATOM(_constructor);
 	ASFUNCTION_ATOM(generator);
 	ASFUNCTION_ATOM(_toString);
@@ -50,10 +50,11 @@ public:
 	ASFUNCTION_ATOM(_toExponential);
 	ASFUNCTION_ATOM(_toFixed);
 	ASFUNCTION_ATOM(_toPrecision);
-	std::string toDebugString() { return toString()+"ui"; }
+	std::string toDebugString() const override;
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
-				std::map<const Class_base*, uint32_t>& traitsMap);
+				std::map<const Class_base*, uint32_t>& traitsMap, ASWorker* wrk) override;
+	static void serializeValue(ByteArray* out,uint32_t val);
 };
 
 }
